@@ -152,7 +152,7 @@ collect_cycle (AnjutaPluginManager *plugin_manager,
 			if (dep == base_plugin)
 			{
 				*cycle = g_list_prepend (NULL, dep);
-				/* DEBUG_PRINT ("%s ", anjuta_plugin_handle_get_name (dep)); */
+				/* DEBUG_PRINT ("%s", anjuta_plugin_handle_get_name (dep)); */
 				return TRUE;
 			}
 			else
@@ -160,7 +160,7 @@ collect_cycle (AnjutaPluginManager *plugin_manager,
 				if (collect_cycle (plugin_manager, base_plugin, dep, cycle))
 				{
 					*cycle = g_list_prepend (*cycle, dep);
-					/* DEBUG_PRINT ("%s ", anjuta_plugin_handle_get_name (dep)); */
+					/* DEBUG_PRINT ("%s", anjuta_plugin_handle_get_name (dep)); */
 					return TRUE;
 				}
 			}
@@ -731,7 +731,7 @@ plugin_set_update (AnjutaPluginManager *plugin_manager,
 				if (!anjuta_plugin_deactivate (ANJUTA_PLUGIN (anjuta_plugin)))
 				{
 					anjuta_util_dialog_info (GTK_WINDOW (priv->shell),
-								 "Plugin '%s' do not want to be deactivated",
+								 _("Plugin '%s' do not want to be deactivated"),
 								 anjuta_plugin_handle_get_name (plugin));
 				}
 			}
@@ -791,7 +791,8 @@ plugin_toggled (GtkCellRendererToggle *cell, char *path_str, gpointer data)
 	GtkTreePath *path;
 	AnjutaPluginHandle *plugin;
 	gboolean enabled;
-	GHashTable *activated_plugins;
+	GList *activated_plugins;
+	GList *node;
 	AnjutaPlugin* plugin_object;
 	
 	path = gtk_tree_path_new_from_string (path_str);
@@ -805,9 +806,15 @@ plugin_toggled (GtkCellRendererToggle *cell, char *path_str, gpointer data)
 			    COL_PLUGIN, &plugin,
 			    -1);
 	
-	if (enabled)
+	/* Activate one plugin can force the loading of other ones, instead of
+	 * searching which plugins have to be activated, we just unmerge all 
+	 * current plugins and merge all plugins after the modification */
+	
+	/* unmerge all plugins */
+	activated_plugins = g_hash_table_get_values (priv->activated_plugins);
+	for (node = g_list_first (activated_plugins); node != NULL; node = g_list_next (node))
 	{
-		plugin_object = g_hash_table_lookup (priv->activated_plugins, plugin);
+		plugin_object = (AnjutaPlugin *)node->data;
 		if (plugin_object &&
 			IANJUTA_IS_PREFERENCES(plugin_object))
 		{
@@ -816,24 +823,29 @@ plugin_toggled (GtkCellRendererToggle *cell, char *path_str, gpointer data)
 									   NULL);
 		}
 	}
-	enabled = !enabled;
-
-	activated_plugins = plugin_set_update (plugin_manager, plugin, enabled);
-	plugin_object = g_hash_table_lookup (priv->activated_plugins, plugin);
+	g_list_free (activated_plugins);
+	
+	plugin_set_update (plugin_manager, plugin, !enabled);
 	
 	/* Make sure that it appears in the preferences. This method
 		can only be called when the preferences dialog is active so
 		it should be save
 	*/
-	if (plugin_object &&
-		IANJUTA_IS_PREFERENCES(plugin_object))
+	activated_plugins = g_hash_table_get_values (priv->activated_plugins);
+	for (node = g_list_first (activated_plugins); node != NULL; node = g_list_next (node))
 	{
-		ianjuta_preferences_merge (IANJUTA_PREFERENCES (plugin_object),
-								   anjuta_shell_get_preferences (ANJUTA_SHELL (priv->shell), NULL),
-								   NULL);
+		plugin_object = (AnjutaPlugin *)node->data;
+		if (plugin_object &&
+			IANJUTA_IS_PREFERENCES(plugin_object))
+		{
+			ianjuta_preferences_merge (IANJUTA_PREFERENCES (plugin_object),
+									   anjuta_shell_get_preferences (ANJUTA_SHELL (priv->shell), NULL),
+									   NULL);
+		}
 	}
-	
-	update_enabled (GTK_TREE_MODEL (store), activated_plugins);
+	g_list_free (activated_plugins);
+
+	update_enabled (GTK_TREE_MODEL (store), priv->activated_plugins);
 	gtk_tree_path_free (path);
 }
 
@@ -1209,7 +1221,7 @@ anjuta_plugin_manager_get_remembered_plugins_page (AnjutaPluginManager *plugin_m
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-	forget_button = gtk_button_new_with_label ("Forget selected plugin");
+	forget_button = gtk_button_new_with_label (_("Forget selected plugin"));
 	gtk_widget_set_sensitive (forget_button, FALSE);
 	gtk_box_pack_end (GTK_BOX (hbox), forget_button, FALSE, FALSE, 0);
 	
@@ -1363,6 +1375,55 @@ get_plugin_factory (AnjutaPluginManager *plugin_manager,
 	return NULL;
 }
 
+static void
+on_is_active_plugins_foreach (gpointer key, gpointer data, gpointer user_data)
+{
+	AnjutaPluginHandle *handle = ANJUTA_PLUGIN_HANDLE (key);
+	gchar const **search_iface = (gchar const **)user_data;
+	
+	if (*search_iface != NULL)
+	{
+		GList *interfaces;
+		GList *found;
+			
+ 		interfaces = anjuta_plugin_handle_get_interfaces (handle);
+		
+		for (found = g_list_first (interfaces); found != NULL; found = g_list_next (found))
+		{
+		}
+		
+		found = g_list_find_custom (interfaces, *search_iface, (GCompareFunc)strcmp);
+		
+		if (found != NULL) *search_iface = NULL;
+	}
+}
+
+/**
+ * anjuta_plugin_manager_is_active_plugin:
+ * @plugin_manager: A #AnjutaPluginManager object
+ * @iface_name: The interface implemented by the object to be found
+ * 
+ * Searches if a currently loaded plugins implements
+ * the given interface.
+ *
+ * Return value: True is the plugin is currently loaded.
+ */
+
+gboolean
+anjuta_plugin_manager_is_active_plugin (AnjutaPluginManager *plugin_manager,
+								  const gchar *iface_name)
+{
+	const gchar *search_iface = iface_name;
+
+	g_return_val_if_fail (ANJUTA_IS_PLUGIN_MANAGER (plugin_manager), FALSE);
+	
+	g_hash_table_foreach (plugin_manager->priv->activated_plugins,
+						  on_is_active_plugins_foreach,
+						  &search_iface);
+	
+	return search_iface == NULL;
+}
+
 /**
  * anjuta_plugin_manager_get_plugin:
  * @plugin_manager: A #AnjutaPluginManager object
@@ -1600,30 +1661,24 @@ anjuta_plugin_manager_unload_plugin (AnjutaPluginManager *plugin_manager,
 }
 
 GList*
-anjuta_plugin_manager_query (AnjutaPluginManager *plugin_manager,
-							 const gchar *section_name,
-							 const gchar *attribute_name,
-							 const gchar *attribute_value,
-							 ...)
+anjuta_plugin_manager_list_query (AnjutaPluginManager *plugin_manager,
+							 GList *secs,
+							 GList *anames,
+							 GList *avalues)
 {
 	AnjutaPluginManagerPriv *priv;
-	va_list var_args;
-	GList *secs = NULL;
-	GList *anames = NULL;
-	GList *avalues = NULL;
-	const gchar *sec = section_name;
-	const gchar *aname = attribute_name;
-	const gchar *avalue = attribute_value;
 	GList *selected_plugins = NULL;
+	const gchar *sec;
+	const gchar *aname;
+	const gchar *avalue;
 	GList *available;
 	
-	
 	g_return_val_if_fail (ANJUTA_IS_PLUGIN_MANAGER (plugin_manager), NULL);
 	
 	priv = plugin_manager->priv;
 	available = priv->available_plugins;
 	
-	if (section_name == NULL)
+	if (secs == NULL)
 	{
 		/* If no query is given, select all plugins */
 		while (available)
@@ -1637,38 +1692,9 @@ anjuta_plugin_manager_query (AnjutaPluginManager *plugin_manager,
 		return g_list_reverse (selected_plugins);
 	}
 	
-	g_return_val_if_fail (section_name != NULL, NULL);
-	g_return_val_if_fail (attribute_name != NULL, NULL);
-	g_return_val_if_fail (attribute_value != NULL, NULL);
-	
-	secs = g_list_prepend (secs, g_strdup (section_name));
-	anames = g_list_prepend (anames, g_strdup (attribute_name));
-	avalues = g_list_prepend (avalues, g_strdup (attribute_value));
-	
-	va_start (var_args, attribute_value);
-	while (sec)
-	{
-		sec = va_arg (var_args, const gchar *);
-		if (sec)
-		{
-			aname = va_arg (var_args, const gchar *);
-			if (aname)
-			{
-				avalue = va_arg (var_args, const gchar *);
-				if (avalue)
-				{
-					secs = g_list_prepend (secs, g_strdup (sec));
-					anames = g_list_prepend (anames, g_strdup (aname));
-					avalues = g_list_prepend (avalues, g_strdup (avalue));
-				}
-			}
-		}
-	}
-	va_end (var_args);
-	
-	secs = g_list_reverse (secs);
-	anames = g_list_reverse (anames);
-	avalues = g_list_reverse (avalues);
+	g_return_val_if_fail (secs != NULL, NULL);
+	g_return_val_if_fail (anames != NULL, NULL);
+	g_return_val_if_fail (avalues != NULL, NULL);
 	
 	while (available)
 	{
@@ -1758,11 +1784,77 @@ anjuta_plugin_manager_query (AnjutaPluginManager *plugin_manager,
 		}
 		available = g_list_next (available);
 	}
+	
+	return g_list_reverse (selected_plugins);
+}
+
+GList*
+anjuta_plugin_manager_query (AnjutaPluginManager *plugin_manager,
+							 const gchar *section_name,
+							 const gchar *attribute_name,
+							 const gchar *attribute_value,
+							 ...)
+{
+	va_list var_args;
+	GList *secs = NULL;
+	GList *anames = NULL;
+	GList *avalues = NULL;
+	const gchar *sec;
+	const gchar *aname;
+	const gchar *avalue;
+	GList *selected_plugins;
+	
+	
+	if (section_name == NULL)
+	{
+		/* If no query is given, select all plugins */
+		return anjuta_plugin_manager_list_query (plugin_manager, NULL, NULL, NULL);
+	}
+	
+	g_return_val_if_fail (section_name != NULL, NULL);
+	g_return_val_if_fail (attribute_name != NULL, NULL);
+	g_return_val_if_fail (attribute_value != NULL, NULL);
+	
+	secs = g_list_prepend (secs, g_strdup (section_name));
+	anames = g_list_prepend (anames, g_strdup (attribute_name));
+	avalues = g_list_prepend (avalues, g_strdup (attribute_value));
+	
+	va_start (var_args, attribute_value);
+	do
+	{
+		sec = va_arg (var_args, const gchar *);
+		if (sec)
+		{
+			aname = va_arg (var_args, const gchar *);
+			if (aname)
+			{
+				avalue = va_arg (var_args, const gchar *);
+				if (avalue)
+				{
+					secs = g_list_prepend (secs, g_strdup (sec));
+					anames = g_list_prepend (anames, g_strdup (aname));
+					avalues = g_list_prepend (avalues, g_strdup (avalue));
+				}
+			}
+		}
+	}
+	while (sec);
+	va_end (var_args);
+	
+	secs = g_list_reverse (secs);
+	anames = g_list_reverse (anames);
+	avalues = g_list_reverse (avalues);
+	
+	selected_plugins = anjuta_plugin_manager_list_query (plugin_manager,
+														 secs,
+														 anames,
+														 avalues);
+	
 	anjuta_util_glist_strings_free (secs);
 	anjuta_util_glist_strings_free (anames);
 	anjuta_util_glist_strings_free (avalues);
 	
-	return g_list_reverse (selected_plugins);
+	return selected_plugins;
 }
 
 enum {
@@ -2429,6 +2521,19 @@ on_collect (gpointer key, gpointer value, gpointer user_data)
 	g_free (id);
 }
 
+/**
+ * anjuta_plugin_manager_get_remembered_plugins:
+ * @plugin_manager: A #AnjutaPluginManager object
+ * 
+ * Get the list of plugins loaded when there is a choice between several
+ * ones without asking the user.
+ *
+ * The list format is returned as a string with the format detailed in 
+ * anjuta_plugin_manager_set_remembered_plugins().
+ *
+ * Return value: a newly-allocated string that must be freed with g_free().
+ */
+
 gchar*
 anjuta_plugin_manager_get_remembered_plugins (AnjutaPluginManager *plugin_manager)
 {
@@ -2449,6 +2554,24 @@ on_foreach_remove_true (gpointer k, gpointer v, gpointer d)
 	return TRUE;
 }
 
+/**
+ * anjuta_plugin_manager_set_remembered_plugins:
+ * @plugin_manager: A #AnjutaPluginManager object
+ * @remembered_plugins: A list of prefered plugins
+ * 
+ * Set the list of plugins loaded when there is a choice between several
+ * ones without asking the user.
+ * The list is a string composed of elements separated by ';'. Each element
+ * is defined with "key=value", where key is the list of possible plugins and
+ * the value is the choosen plugin.
+ *
+ * By the example the following element
+ * <programlisting>
+ *   anjuta-symbol-browser:SymbolBrowserPlugin,anjuta-symbol-db:SymbolDBPlugin,=anjuta-symbol-db:SymbolDBPlugin;
+ * </programlisting>
+ * means if Anjuta has to choose between SymbolBrowserPlugin and
+ * SymbolDBPlugin, it will choose SymbolDBPlugin.
+ */
 void
 anjuta_plugin_manager_set_remembered_plugins (AnjutaPluginManager *plugin_manager,
 											  const gchar *remembered_plugins)
