@@ -28,9 +28,7 @@
  */
 
 #include <config.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkprogressbar.h>
-#include <gtk/gtkstatusbar.h>
+#include <gtk/gtk.h>
 #include <libanjuta/anjuta-status.h>
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/resources.h>
@@ -125,6 +123,21 @@ anjuta_status_dispose (GObject *widget)
 						   (gpointer*)(gpointer)&status->priv->window);
 		status->priv->window = NULL;
 	}
+	if (status->priv->progress_bar)
+	{
+		g_object_remove_weak_pointer (G_OBJECT (status->priv->progress_bar), 
+							(gpointer)&status->priv->progress_bar);
+		gtk_widget_destroy (status->priv->progress_bar);
+		status->priv->progress_bar = NULL;
+	}
+	if (status->priv->status_bar)
+	{
+		g_object_remove_weak_pointer (G_OBJECT (status->priv->status_bar), 
+							(gpointer)&status->priv->status_bar);
+		gtk_widget_destroy (status->priv->status_bar);
+		status->priv->status_bar = NULL;
+	}
+
 	G_OBJECT_CLASS (parent_class)->dispose (widget);
 }
 
@@ -135,9 +148,13 @@ anjuta_status_instance_init (AnjutaStatus *status)
 	status->priv->progress_bar = gtk_progress_bar_new ();
 	gtk_box_pack_start (GTK_BOX (status), status->priv->progress_bar, FALSE, TRUE, 0);
 	gtk_widget_show (status->priv->progress_bar);
+	g_object_add_weak_pointer (G_OBJECT (status->priv->progress_bar),
+							   (gpointer)&status->priv->progress_bar);
 	status->priv->status_bar = gtk_statusbar_new ();
 	gtk_box_pack_start (GTK_BOX (status), status->priv->status_bar, TRUE, TRUE, 0);
 	gtk_widget_show (status->priv->status_bar);
+	g_object_add_weak_pointer (G_OBJECT(status->priv->status_bar), 
+							   (gpointer)&status->priv->status_bar);
 	status->priv->status_message = gtk_statusbar_get_context_id (GTK_STATUSBAR (status->priv->status_bar),
 																 "status-message");
 	status->priv->push_message = gtk_statusbar_get_context_id (GTK_STATUSBAR (status->priv->status_bar),
@@ -438,16 +455,19 @@ anjuta_status_progress_add_ticks (AnjutaStatus *status, gint ticks)
 	}
 	else
 	{	
-		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (status->priv->progress_bar),
-									   percentage);
-		gtk_widget_queue_draw (GTK_WIDGET (status->priv->status_bar));
-		gtk_widget_queue_draw (GTK_WIDGET (status->priv->progress_bar));
-		if (GTK_WIDGET(status->priv->progress_bar)->window != NULL &&
-			GDK_IS_WINDOW(GTK_WIDGET(status->priv->progress_bar)->window))
-			gdk_window_process_updates (GTK_WIDGET(status->priv->progress_bar)->window, TRUE);
-		if (GTK_WIDGET(status->priv->status_bar)->window != NULL &&
-			GDK_IS_WINDOW(GTK_WIDGET(status->priv->status_bar)->window))
-			gdk_window_process_updates (GTK_WIDGET(status->priv->status_bar)->window, TRUE);
+		if (status->priv->progress_bar && status->priv->status_bar)
+		{
+			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (status->priv->progress_bar),
+										   percentage);
+			gtk_widget_queue_draw (GTK_WIDGET (status->priv->status_bar));
+			gtk_widget_queue_draw (GTK_WIDGET (status->priv->progress_bar));
+			if (GTK_WIDGET(status->priv->progress_bar)->window != NULL &&
+				GDK_IS_WINDOW(GTK_WIDGET(status->priv->progress_bar)->window))
+				gdk_window_process_updates (GTK_WIDGET(status->priv->progress_bar)->window, TRUE);
+			if (GTK_WIDGET(status->priv->status_bar)->window != NULL &&
+				GDK_IS_WINDOW(GTK_WIDGET(status->priv->status_bar)->window))
+				gdk_window_process_updates (GTK_WIDGET(status->priv->status_bar)->window, TRUE);
+		}
 	}
 }
 
@@ -490,7 +510,6 @@ anjuta_status_progress_tick (AnjutaStatus *status,
 	if (status->priv->splash)
 	{
 		e_splash_set (E_SPLASH(status->priv->splash), icon, text, NULL, percentage);
-		while (g_main_context_iteration(NULL, FALSE));
 	}
 	else
 	{
@@ -533,10 +552,10 @@ anjuta_status_progress_increment_ticks (AnjutaStatus *status, gint ticks,
 	
 	if (text)
 		anjuta_status_set (status, "%s", text);
-	gnome_appbar_set_progress_percentage (GNOME_APPBAR (status),
-										  percentage);
-	progressbar = gnome_appbar_get_progress (GNOME_APPBAR (status));
-	statusbar = gnome_appbar_get_status (GNOME_APPBAR (status));
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (status->priv->progress_bar),
+									   percentage);
+	progressbar = GTK_PROGRESS_BAR (status->priv->progress_bar);
+	statusbar = status->priv->status_bar;
 	gtk_widget_queue_draw (GTK_WIDGET (statusbar));
 	gtk_widget_queue_draw (GTK_WIDGET (progressbar));
 	if (GTK_WIDGET(progressbar)->window != NULL &&
@@ -580,7 +599,7 @@ anjuta_status (AnjutaStatus *status, const gchar *mesg, gint timeout)
 	g_return_if_fail (ANJUTA_IS_STATUS (status));
 	g_return_if_fail (mesg != NULL);
 	anjuta_status_push (status, "%s", mesg);
-	g_timeout_add (timeout * 1000, (void*) anjuta_status_timeout, status);
+	g_timeout_add_seconds (timeout, (void*) anjuta_status_timeout, status);
 }
 
 void
