@@ -29,7 +29,7 @@
 
 G_BEGIN_DECLS
 
-gboolean anjuta_util_copy_file (gchar * src, gchar * dest, gboolean show_error);
+gboolean anjuta_util_copy_file (const gchar * src, const gchar * dest, gboolean show_error);
 
 gboolean anjuta_util_diff(const gchar* uri, const gchar* text);
 
@@ -110,17 +110,19 @@ pid_t anjuta_util_execute_shell (const gchar *dir, const gchar *command);
 
 gchar* anjuta_util_escape_quotes(const gchar* str);
 
-gboolean anjuta_util_path_has_extension (const gchar *path, const gchar *ext);
 gchar* anjuta_util_get_real_path (const gchar *path);
 
 gchar* anjuta_util_uri_get_dirname (const gchar *uri);
 gchar* anjuta_util_replace_home_dir_with_tilde (const gchar *uri);
+gchar* anjuta_util_shell_expand (const gchar *string);
 gchar* anjuta_util_str_middle_truncate (const gchar *string,
 										 guint        truncate_length);
 
-gchar* anjuta_util_get_uri_mime_type (const gchar *uri);
+gboolean anjuta_util_is_project_file (const gchar *filename);
+gchar* anjuta_util_get_file_mime_type (GFile *file);
+gchar* anjuta_util_get_local_path_from_uri (const gchar *uri);
 
-void anjuta_util_help_display (GtkWindow   *parent,
+void anjuta_util_help_display (GtkWidget   *parent,
 							   const gchar *doc_id,
 							   const gchar *file_name);
 
