@@ -52,6 +52,9 @@
 #endif
 
 #define ICON_FILE "anjuta-build-basic-autotools-plugin-48.png"
+#define ANJUTA_PIXMAP_BUILD            "anjuta-build"
+#define ANJUTA_STOCK_BUILD             "anjuta-build"
+
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-build-basic-autotools-plugin.ui"
 #define MAX_BUILD_PANES 3
 #define PREF_INDICATORS_AUTOMATIC "indicators.automatic"
@@ -271,7 +274,7 @@ build_context_pop_dir (BuildContext *context, const gchar *key,
 	
 	if (strcmp (top_dir, dir) != 0)
 	{
-		DEBUG_PRINT("Directory stack misaligned!!");
+		DEBUG_PRINT("%s", "Directory stack misaligned!!");
 	}
 	g_free (top_dir);
 	if (dir_stack)
@@ -939,12 +942,12 @@ on_build_terminated (AnjutaLauncher *launcher,
 		if (status)
 		{
 			ianjuta_message_view_buffer_append (context->message_view,
-									_("Completed unsuccessful\n"), NULL);
+									_("Completed unsuccessfully\n"), NULL);
 		}
 		else
 		{
 			ianjuta_message_view_buffer_append (context->message_view,
-									   _("Completed successful\n"), NULL);
+									   _("Completed successfully\n"), NULL);
 		}
 		ianjuta_message_view_buffer_append (context->message_view, buff1, NULL);
 		g_free (buff1);
@@ -963,7 +966,7 @@ on_build_terminated (AnjutaLauncher *launcher,
 static void
 on_message_view_destroyed (BuildContext *context, GtkWidget *view)
 {
-	DEBUG_PRINT ("Destroying build context");
+	DEBUG_PRINT ("%s", "Destroying build context");
 	context->message_view = NULL;
 	
 	build_context_destroy_view (context);
@@ -1360,14 +1363,11 @@ build_is_file_built (BasicAutotoolsPlugin *plugin, const gchar *filename,
 {
 	BuildContext *context;
 	gchar *build_dir;
-	gchar *src_dir;
 	gchar *target;
 	BuildProgram *prog;
 	
-		
-	src_dir = g_path_get_dirname (filename);
-	build_dir = build_dir_from_source (plugin, src_dir);
-	g_free (src_dir);
+	/* the file of this command is a target, no need to use source directory */		
+	build_dir = g_path_get_dirname (filename);
 	target = g_path_get_basename (filename);
 
 	prog = build_program_new_with_command (build_dir,
@@ -1778,7 +1778,7 @@ on_build_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	if (plugin->current_editor_filename)
 	{
-		gchar *dirname = g_dirname (plugin->current_editor_filename);
+		gchar *dirname = g_path_get_dirname (plugin->current_editor_filename);
 		
 		build_build_file_or_dir (plugin, dirname, NULL, NULL, NULL);
 		
@@ -1791,7 +1791,7 @@ on_install_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	if (plugin->current_editor_filename)
 	{
-		gchar *dirname = g_dirname (plugin->current_editor_filename);
+		gchar *dirname = g_path_get_dirname (plugin->current_editor_filename);
 		
 		build_install_dir (plugin, dirname, NULL);
 		
@@ -1804,7 +1804,7 @@ on_clean_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	if (plugin->current_editor_filename)
 	{
-		gchar *dirname = g_dirname (plugin->current_editor_filename);
+		gchar *dirname = g_path_get_dirname (plugin->current_editor_filename);
 		
 		build_clean_dir (plugin, dirname, NULL);
 		
@@ -2019,8 +2019,8 @@ static GtkActionEntry build_actions[] =
 		N_("_Build"), NULL, NULL, NULL
 	},
 	{
-		"ActionBuildBuildProject", NULL,
-		N_("_Build Project"), "<shift>F11",
+		"ActionBuildBuildProject", GTK_STOCK_CONVERT,
+		N_("_Build Project"), "<shift>F7",
 		N_("Build whole project"),
 		G_CALLBACK (on_build_project)
 	},
@@ -2049,8 +2049,8 @@ static GtkActionEntry build_actions[] =
 		G_CALLBACK (on_build_tarball)
 	},
 	{
-		"ActionBuildBuildModule", GTK_STOCK_EXECUTE,
-		N_("_Build Module"), "F11",
+		"ActionBuildBuildModule", ANJUTA_STOCK_BUILD,
+		N_("_Build Module"), "F7",
 		N_("Build module associated with current file"),
 		G_CALLBACK (on_build_module)
 	},
@@ -2067,7 +2067,7 @@ static GtkActionEntry build_actions[] =
 		G_CALLBACK (on_clean_module)
 	},
 	{
-		"ActionBuildCompileFile", GTK_STOCK_CONVERT,
+		"ActionBuildCompileFile", NULL,
 		N_("Co_mpile File"), "F9",
 		N_("Compile current editor file"),
 		G_CALLBACK (on_compile_file)
@@ -2236,7 +2236,7 @@ update_module_ui (BasicAutotoolsPlugin *bb_plugin)
 
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (bb_plugin)->shell, NULL);
 	
-	DEBUG_PRINT ("Updating module UI");
+	DEBUG_PRINT ("%s", "Updating module UI");
 
 	has_file = bb_plugin->current_editor_filename != NULL;
 	if (has_file)
@@ -2244,11 +2244,11 @@ update_module_ui (BasicAutotoolsPlugin *bb_plugin)
 		gchar *dirname;
 		gchar *build_dirname;
 
-		dirname = g_dirname (bb_plugin->current_editor_filename);
+		dirname = g_path_get_dirname (bb_plugin->current_editor_filename);
 		build_dirname = build_dir_from_source (bb_plugin, dirname);
 		
-		module = escape_label (g_basename (dirname));
-		filename = escape_label (g_basename (bb_plugin->current_editor_filename));
+		module = escape_label (g_path_get_basename (dirname));
+		filename = escape_label (g_path_get_basename (bb_plugin->current_editor_filename));
 		has_makefile = directory_has_makefile (build_dirname);
 		g_free (build_dirname);
 		g_free (dirname);
@@ -2295,7 +2295,7 @@ update_project_ui (BasicAutotoolsPlugin *bb_plugin)
 	gboolean has_makefile;
 	gboolean has_project;
 	
-	DEBUG_PRINT ("Updating project UI");
+	DEBUG_PRINT ("%s", "Updating project UI");
 	
 	has_project = bb_plugin->project_root_dir != NULL;
 	has_makefile = has_project && directory_has_makefile (bb_plugin->project_build_dir);
@@ -2476,12 +2476,9 @@ value_added_pm_current_uri (AnjutaPlugin *plugin, const char *name,
 	const gchar *uri;
 	gchar *dirname, *filename;
 	gboolean makefile_exists, is_dir;
-	GFile* file;
 	
 	uri = g_value_get_string (value);
-	file = g_file_new_for_uri (uri);
-	filename = g_file_get_path (file);
-	g_object_unref (file);
+	filename = anjuta_util_get_local_path_from_uri (uri);
 	g_return_if_fail (filename != NULL);
 	
 	/*
@@ -2756,6 +2753,20 @@ value_removed_current_editor (AnjutaPlugin *plugin,
 	update_module_ui (ba_plugin);
 }
 
+static void
+register_stock_icons (AnjutaPlugin *plugin)
+{	
+	static gboolean registered = FALSE;
+
+	if (registered)
+		return;
+	registered = TRUE;
+
+	BEGIN_REGISTER_ICON (plugin);
+	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BUILD, ANJUTA_STOCK_BUILD);
+	END_REGISTER_ICON;
+}
+
 static gboolean
 activate_plugin (AnjutaPlugin *plugin)
 {
@@ -2763,6 +2774,10 @@ activate_plugin (AnjutaPlugin *plugin)
 	static gboolean initialized = FALSE;
 	BasicAutotoolsPlugin *ba_plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (plugin);
 	
+	if (!initialized)
+	{
+		register_stock_icons (plugin);
+	}
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	
 	g_signal_connect (plugin->shell, "save-session",
@@ -3049,7 +3064,7 @@ ifile_open (IAnjutaFile *manager, GFile* file,
 static GFile*
 ifile_get_file (IAnjutaFile *manager, GError **err)
 {
-	DEBUG_PRINT ("Unsupported operation");
+	DEBUG_PRINT ("%s", "Unsupported operation");
 	return NULL;
 }
 
@@ -3072,11 +3087,8 @@ ibuilder_is_built (IAnjutaBuilder *builder, const gchar *uri,
 	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (builder);
 	BuildContext *context;
 	gchar *filename;
-	GFile* file;
 
-	file = g_file_new_for_uri (uri);
-	filename = g_file_get_path (file);
-	g_object_unref (file);
+	filename = anjuta_util_get_local_path_from_uri (uri);
 	if (filename == NULL) return NULL;
 	
 	context = build_is_file_built (plugin, filename, callback, user_data, err);
@@ -3094,11 +3106,8 @@ ibuilder_build (IAnjutaBuilder *builder, const gchar *uri,
 	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (builder);
 	BuildContext *context;
 	gchar *filename;
-	GFile* file;
 	
-	file = g_file_new_for_uri (uri);
-	filename = g_file_get_path (file);
-	g_object_unref (file);
+	filename = anjuta_util_get_local_path_from_uri (uri);
 	if (filename == NULL) return NULL;
 	
 	context = build_build_file_or_dir (plugin, filename, callback, user_data, err);
