@@ -18,8 +18,7 @@
  */
 
 #include <config.h>
-#include <gtk/gtkactiongroup.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <libanjuta/anjuta-preferences.h>
 #include <libanjuta/anjuta-debug.h>
@@ -54,7 +53,7 @@ project_root_added (AnjutaPlugin *plugin,
 	
 	if (root_uri)
 	{
-		gchar *root_dir = gnome_vfs_get_local_path_from_uri (root_uri);
+		gchar *root_dir = anjuta_util_get_local_path_from_uri (root_uri);
 		if (root_dir)
 			cg_plugin->top_dir = g_strdup(root_dir);
 		else
@@ -83,7 +82,7 @@ activate_plugin (AnjutaPlugin *plugin)
 {
 	AnjutaClassGenPlugin *cg_plugin;
 	
-	DEBUG_PRINT ("AnjutaClassGenPlugin: Activating ClassGen plugin...");
+	DEBUG_PRINT ("%s", "AnjutaClassGenPlugin: Activating ClassGen plugin...");
 	cg_plugin = ANJUTA_PLUGIN_CLASS_GEN (plugin);
 	cg_plugin->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
 	
@@ -117,7 +116,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 {
 	AnjutaClassGenPlugin *cg_plugin;
 	cg_plugin = ANJUTA_PLUGIN_CLASS_GEN (plugin);
-	DEBUG_PRINT ("AnjutaClassGenPlugin: Deactivating ClassGen plugin ...");
+	DEBUG_PRINT ("%s", "AnjutaClassGenPlugin: Deactivating ClassGen plugin ...");
 	
 	/* Remove watches */
 	anjuta_plugin_remove_watch (plugin, cg_plugin->root_watch_id, TRUE);
@@ -216,14 +215,21 @@ cg_plugin_add_to_project (AnjutaClassGenPlugin *plugin,
 	}
 	else
 	{
-		*new_header_file = added_files->data;
-		*new_source_file = g_list_next (added_files)->data;
+		GFile *file;
 
+		file = g_file_new_for_uri ((gchar *)added_files->data);
+		*new_header_file = g_file_get_path(file);
+		g_object_unref (file);
+		file = g_file_new_for_uri ((gchar *)g_list_next (added_files)->data);
+		*new_source_file = g_file_get_path(file);
+		g_object_unref (file);
+		
 		result = TRUE;
 	}
 
 	g_free (curdir);
 	g_free (dirname);
+	g_list_foreach (added_files, (GFunc)g_free, NULL);
 	g_list_free (added_files);
 	g_list_free (filenames);
 
@@ -241,8 +247,12 @@ cg_plugin_add_to_repository (AnjutaClassGenPlugin *plugin,
 
 	if(vcs != NULL)
 	{
-		ianjuta_vcs_add (vcs, header_file, NULL);
-		ianjuta_vcs_add (vcs, source_file, NULL);
+		GList* files = NULL;
+		files = g_list_append (files, g_file_new_for_path (header_file));
+		files = g_list_append (files, g_file_new_for_path (source_file));		
+		ianjuta_vcs_add (vcs, files, NULL, NULL);
+		g_list_foreach (files, (GFunc) g_object_unref, NULL);
+		g_list_free (files);
 	}
 }
 
@@ -349,7 +359,7 @@ cg_plugin_window_response_cb (G_GNUC_UNUSED GtkDialog *dialog,
 {
 	AnjutaClassGenPlugin *plugin;
 	IAnjutaProjectManager *manager;
-	NPWValueHeap *values;
+	GHashTable *values;
 	NPWValue *value;
 	GError *error;
 	gchar *name;
@@ -392,7 +402,7 @@ cg_plugin_window_response_cb (G_GNUC_UNUSED GtkDialog *dialog,
 				/* Use basename of the project's root URI as project name. */
 				name = g_path_get_basename (plugin->top_dir);
 				value = npw_value_heap_find_value (values, "ProjectName");
-				npw_value_heap_set_value (values, value, name, NPW_VALID_VALUE);
+				npw_value_set_value (value, name, NPW_VALID_VALUE);
 				g_free (name);
 			}
 			else
@@ -400,7 +410,7 @@ cg_plugin_window_response_cb (G_GNUC_UNUSED GtkDialog *dialog,
 				name = g_path_get_basename (cg_window_get_source_file(
 				                            plugin->window));
 				value = npw_value_heap_find_value (values, "ProjectName");
-				npw_value_heap_set_value (values, value, name, NPW_VALID_VALUE);
+				npw_value_set_value (value, name, NPW_VALID_VALUE);
 				g_free (name);
 			}
 
