@@ -368,7 +368,7 @@ dma_plugin_debugger_started (DebugManagerPlugin *this)
 	GtkAction *action;
 	AnjutaStatus* status;
 
-	DEBUG_PRINT ("DMA: dma_plugin_debugger_started");
+	DEBUG_PRINT ("%s", "DMA: dma_plugin_debugger_started");
 	
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (this)->shell, NULL);
 	
@@ -394,7 +394,7 @@ dma_plugin_program_loaded (DebugManagerPlugin *this)
 	AnjutaUI *ui;
 	AnjutaStatus* status;
 
-	DEBUG_PRINT ("DMA: dma_plugin_program_loaded");
+	DEBUG_PRINT ("%s", "DMA: dma_plugin_program_loaded");
 	
 	if (this->sharedlibs == NULL)
 	{
@@ -423,7 +423,7 @@ dma_plugin_program_running (DebugManagerPlugin *this)
 	AnjutaUI *ui;
 	AnjutaStatus* status;
 
-	DEBUG_PRINT ("DMA: dma_plugin_program_running");
+	DEBUG_PRINT ("%s", "DMA: dma_plugin_program_running");
 	
 	/* Update ui */
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (this)->shell, NULL);
@@ -445,7 +445,7 @@ dma_plugin_program_stopped (DebugManagerPlugin *this)
 	AnjutaUI *ui;
 	AnjutaStatus* status;
 	
-	DEBUG_PRINT ("DMA: dma_plugin_program_broken");
+	DEBUG_PRINT ("%s", "DMA: dma_plugin_program_broken");
 	
 	/* Update ui */
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (this)->shell, NULL);
@@ -474,7 +474,7 @@ dma_plugin_program_unload (DebugManagerPlugin *this)
 	AnjutaUI *ui;
 	AnjutaStatus* status;
 
-	DEBUG_PRINT ("DMA: dma_plugin_program_unload");
+	DEBUG_PRINT ("%s", "DMA: dma_plugin_program_unload");
 	
 	if (this->sharedlibs != NULL)
 	{
@@ -511,7 +511,7 @@ dma_plugin_debugger_stopped (DebugManagerPlugin *self, GError *err)
 	GtkAction *action;
 	AnjutaStatus* state;
 
-	DEBUG_PRINT ("DMA: dma_plugin_debugger_stopped");
+	DEBUG_PRINT ("%s", "DMA: dma_plugin_debugger_stopped");
 
 	dma_plugin_program_unload (self);
 	
@@ -583,7 +583,7 @@ static void
 on_start_debug_activate (GtkAction* action, DebugManagerPlugin* this)
 {
 	enable_log_view (this, TRUE);
-	dma_run_target (this->start);
+	dma_run_target (this->start, NULL);
 }
 
 static void
@@ -593,6 +593,14 @@ on_attach_to_project_action_activate (GtkAction* action, DebugManagerPlugin* thi
 	dma_attach_to_process (this->start);
 }
 
+static void
+on_start_remote_debug_action_activate (GtkAction* action, DebugManagerPlugin* this)
+{
+	/* Returns true if user clicked "Connect" */
+	enable_log_view (this, TRUE);
+	dma_run_remote_target (this->start, NULL, NULL);
+}
+
 static void
 on_debugger_stop_activate (GtkAction* action, DebugManagerPlugin* plugin)
 {
@@ -847,6 +855,14 @@ static GtkActionEntry actions_start[] =
 		N_("Start debugger and attach to a running program"),
 		G_CALLBACK (on_attach_to_project_action_activate)
 	},
+	{
+		"ActionDebuggerDebugRemote",
+		"debugger-remote-target",
+		N_("Debug _Remote Target..."),
+		NULL,
+		N_("Connect to a remote debugging target"),
+		G_CALLBACK (on_start_remote_debug_action_activate),
+	},
 	{
 		"ActionDebuggerStop",
 		GTK_STOCK_STOP,
@@ -937,7 +953,7 @@ static GtkActionEntry actions_stopped[] =
 {
 	{
 		"ActionDebuggerRunContinue",                   /* Action name */
-		GTK_STOCK_EXECUTE,                             /* Stock icon, if any */
+		GTK_STOCK_MEDIA_PLAY,                             /* Stock icon, if any */
 		N_("Run/_Continue"),                           /* Display label */
 		"F4",                                          /* short-cut */
 		N_("Continue the execution of the program"),   /* Tooltip */
@@ -963,7 +979,7 @@ static GtkActionEntry actions_stopped[] =
 		"ActionDebuggerStepOut",
 		"debugger-step-out",
 		N_("Step _Out"),                    
-		"F7",                              
+		"<shift>F5",                              
 		N_("Single step out of the function"), 
 		G_CALLBACK (on_step_out_action_activate) 
 	},
@@ -1054,8 +1070,9 @@ dma_plugin_activate (AnjutaPlugin* plugin)
 	DebugManagerPlugin *this;
     static gboolean initialized = FALSE;
 	AnjutaUI *ui;
+	GtkAction *action;
 	
-	DEBUG_PRINT ("DebugManagerPlugin: Activating Debug Manager plugin...");
+	DEBUG_PRINT ("%s", "DebugManagerPlugin: Activating Debug Manager plugin...");
 	this = ANJUTA_PLUGIN_DEBUG_MANAGER (plugin);
 	
     if (!initialized)
@@ -1153,7 +1170,7 @@ dma_plugin_deactivate (AnjutaPlugin* plugin)
 	DebugManagerPlugin *this;
 	AnjutaUI *ui;
 
-	DEBUG_PRINT ("DebugManagerPlugin: Deactivating Debug Manager plugin...");
+	DEBUG_PRINT ("%s", "DebugManagerPlugin: Deactivating Debug Manager plugin...");
 
 	this = ANJUTA_PLUGIN_DEBUG_MANAGER (plugin);
 
@@ -1207,7 +1224,7 @@ dma_plugin_deactivate (AnjutaPlugin* plugin)
 		g_object_remove_weak_pointer (G_OBJECT (this->view), (gpointer*)(gpointer)&this->view);
         this->view = NULL;
 	}
-	
+
 	return TRUE;
 }
 
@@ -1301,9 +1318,36 @@ dma_plugin_class_init (GObjectClass* klass)
 /* Implementation of IAnjutaDebugManager interface
  *---------------------------------------------------------------------------*/
 
+static gboolean
+idebug_manager_start (IAnjutaDebugManager *plugin, const gchar *uri, GError **err)
+{
+	DebugManagerPlugin *self = ANJUTA_PLUGIN_DEBUG_MANAGER (plugin);
+
+	return dma_run_target (self->start, uri);
+}
+
+static gboolean
+idebug_manager_start_remote (IAnjutaDebugManager *plugin, const gchar *server, const gchar *uri, GError **err)
+{
+	DebugManagerPlugin *self = ANJUTA_PLUGIN_DEBUG_MANAGER (plugin);
+
+	return dma_run_remote_target (self->start, server, uri);
+}
+
+static gboolean
+idebug_manager_quit (IAnjutaDebugManager *plugin, GError **err)
+{
+	DebugManagerPlugin *self = ANJUTA_PLUGIN_DEBUG_MANAGER (plugin);
+
+	return dma_quit_debugger (self->start);
+}
+
 static void
 idebug_manager_iface_init (IAnjutaDebugManagerIface *iface)
 {
+	iface->start = idebug_manager_start;
+	iface->start_remote = idebug_manager_start_remote;
+	iface->quit = idebug_manager_quit;
 }
 
 ANJUTA_PLUGIN_BEGIN (DebugManagerPlugin, dma_plugin);
