@@ -28,7 +28,6 @@
 #include <libanjuta/interfaces/ianjuta-editor-line-mode.h>
 #include <libanjuta/interfaces/ianjuta-editor-view.h>
 #include <libanjuta/interfaces/ianjuta-editor-folds.h>
-#include <libanjuta/interfaces/ianjuta-bookmark.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
 #include <libanjuta/interfaces/ianjuta-print.h>
@@ -45,6 +44,7 @@
 #include "plugin.h"
 #include "file_history.h"
 #include "search-box.h"
+#include "anjuta-bookmarks.h"
 
 static IAnjutaDocument *
 get_current_document (gpointer user_data)
@@ -276,7 +276,7 @@ on_reload_file_activate (GtkAction *action, gpointer user_data)
 	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
 									 GTK_DIALOG_DESTROY_WITH_PARENT,
 									 GTK_MESSAGE_QUESTION,
-									 GTK_BUTTONS_NONE, msg);
+									 GTK_BUTTONS_NONE, "%s", msg);
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
 						   GTK_STOCK_CANCEL, GTK_RESPONSE_NO);
 	anjuta_util_dialog_add_button (GTK_DIALOG (dialog), _("_Reload"),
@@ -605,68 +605,6 @@ on_editor_command_toggle_fold_activate (GtkAction *action, gpointer user_data)
 		ianjuta_editor_folds_toggle_current (IANJUTA_EDITOR_FOLDS (doc), NULL);
 }
 
-void
-on_editor_command_bookmark_toggle_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_toggle (IANJUTA_BOOKMARK (doc),
-								ianjuta_editor_get_lineno (IANJUTA_EDITOR (doc), NULL),
-								FALSE, NULL);
-}
-
-void
-on_editor_command_bookmark_first_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_first (IANJUTA_BOOKMARK (doc), NULL);
-}
-
-void
-on_editor_command_bookmark_next_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_next (IANJUTA_BOOKMARK (doc), NULL);
-}
-
-void
-on_editor_command_bookmark_prev_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_previous (IANJUTA_BOOKMARK (doc), NULL);
-}
-
-void
-on_editor_command_bookmark_last_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_last (IANJUTA_BOOKMARK (doc), NULL);
-}
-
-void
-on_editor_command_bookmark_clear_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_clear_all (IANJUTA_BOOKMARK (doc), NULL);
-}
-
 void
 on_transform_eolchars1_activate (GtkAction *action, gpointer user_data)
 {
@@ -933,24 +871,92 @@ on_editor_remove_view_activate (GtkAction *action, gpointer user_data)
 		ianjuta_editor_view_remove_current (IANJUTA_EDITOR_VIEW (doc), NULL);
 }
 
-void on_next_document (GtkAction *action, gpointer user_data)
+void
+on_next_document (GtkAction *action, gpointer user_data)
 {
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (docman),
-								   gtk_notebook_get_current_page (GTK_NOTEBOOK(docman)) + 1);
+	GtkNotebook* notebook = GTK_NOTEBOOK (docman); 
+	gint cur_page = gtk_notebook_get_current_page(notebook);
+	if (cur_page <
+		gtk_notebook_get_n_pages(notebook) - 1)
+		cur_page++;
+	else
+		cur_page = 0;
+
+	gtk_notebook_set_current_page (notebook,
+								   cur_page);
 }
 
-void on_previous_document (GtkAction *action, gpointer user_data)
+void 
+on_previous_document (GtkAction *action, gpointer user_data)
 {
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
+	GtkNotebook* notebook = GTK_NOTEBOOK (docman); 
+	gint cur_page = gtk_notebook_get_current_page(notebook);
+	if (cur_page > 0)
+		cur_page--;
+	else
+		cur_page = -1; /* last_page */
 	
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (docman),
-								   gtk_notebook_get_current_page (GTK_NOTEBOOK(docman)) - 1);
+	gtk_notebook_set_current_page (notebook,
+								   cur_page);
+}
+
+void
+on_bookmark_add_activate (GtkAction *action, gpointer user_data)
+{
+	IAnjutaDocument *doc;
+	DocmanPlugin *plugin;
+	doc = get_current_document (user_data);
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	if (doc && IANJUTA_IS_EDITOR(doc))
+	{
+		IAnjutaEditor* editor = IANJUTA_EDITOR(doc);
+		anjuta_bookmarks_add (ANJUTA_BOOKMARKS (plugin->bookmarks), editor, 
+							  ianjuta_editor_get_lineno (editor, NULL), NULL, TRUE);
+	}
+}
+
+void 
+on_bookmark_next_activate (GtkAction *action, gpointer user_data)
+{
+	IAnjutaDocument *doc;
+	DocmanPlugin *plugin;
+	doc = get_current_document (user_data);
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	if (doc && IANJUTA_IS_EDITOR(doc))
+	{
+		IAnjutaEditor* editor = IANJUTA_EDITOR(doc);
+		anjuta_bookmarks_next (ANJUTA_BOOKMARKS (plugin->bookmarks), editor, 
+							  ianjuta_editor_get_lineno (editor, NULL));
+	}
+}
+
+void
+on_bookmark_prev_activate (GtkAction *action, gpointer user_data)
+{
+	IAnjutaDocument *doc;
+	DocmanPlugin *plugin;
+	doc = get_current_document (user_data);
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	if (doc && IANJUTA_IS_EDITOR(doc))
+	{
+		IAnjutaEditor* editor = IANJUTA_EDITOR(doc);
+		anjuta_bookmarks_prev (ANJUTA_BOOKMARKS (plugin->bookmarks), editor, 
+							   ianjuta_editor_get_lineno (editor, NULL));
+	}
+}
+
+void
+on_bookmarks_clear_activate (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	anjuta_bookmarks_clear (ANJUTA_BOOKMARKS(plugin->bookmarks));
 }
