@@ -48,17 +48,16 @@
 #include "action-callbacks.h"
 #include "plugin.h"
 #include "search-box.h"
+#include "anjuta-bookmarks.h"
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-document-manager.ui"
 #define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-document-manager.glade"
 #define ICON_FILE "anjuta-document-manager-plugin-48.png"
 
 #define ANJUTA_PIXMAP_BOOKMARK_TOGGLE     "anjuta-bookmark-toggle"
-#define ANJUTA_PIXMAP_BOOKMARK_FIRST      "anjuta-bookmark-first"
-#define ANJUTA_PIXMAP_BOOKMARK_PREV       "anjuta-bookmark-prev"
-#define ANJUTA_PIXMAP_BOOKMARK_NEXT       "anjuta-bookmark-next"
-#define ANJUTA_PIXMAP_BOOKMARK_LAST       "anjuta-bookmark-last"
-#define ANJUTA_PIXMAP_BOOKMARK_CLEAR      "anjuta-bookmark-clear"
+#define ANJUTA_PIXMAP_BOOKMARK_PREV		  "anjuta-bookmark-prev"
+#define ANJUTA_PIXMAP_BOOKMARK_NEXT		  "anjuta-bookmark-next" 
+#define ANJUTA_PIXMAP_BOOKMARK_CLEAR	  "anjuta-bookmark-clear" 
 
 #define ANJUTA_PIXMAP_FOLD_TOGGLE         "anjuta-fold-toggle"
 #define ANJUTA_PIXMAP_FOLD_CLOSE          "anjuta-fold-close"
@@ -89,11 +88,9 @@
 #define ANJUTA_STOCK_BLOCK_START              "anjuta-block-start"
 #define ANJUTA_STOCK_BLOCK_END                "anjuta-block-end"
 #define ANJUTA_STOCK_BOOKMARK_TOGGLE          "anjuta-bookmark-toggle"
-#define ANJUTA_STOCK_BOOKMARK_FIRST           "anjuta-bookmark-first"
 #define ANJUTA_STOCK_BOOKMARK_PREV            "anjuta-bookmark-previous"
 #define ANJUTA_STOCK_BOOKMARK_NEXT            "anjuta-bookmark-next"
-#define ANJUTA_STOCK_BOOKMARK_LAST            "anjuta-bookmark-last"
-#define ANJUTA_STOCK_BOOKMARK_CLEAR           "anjuta-bookmark-clear"
+#define ANJUTA_STOCK_BOOKMARK_CLEAR           "anjuta-bookmark-clear" 
 #define ANJUTA_STOCK_GOTO_LINE				  "anjuta-goto-line"
 #define ANJUTA_STOCK_HISTORY_NEXT			  "anjuta-history-next"
 #define ANJUTA_STOCK_HISTORY_PREV			  "anjuta-history-prev"
@@ -152,7 +149,7 @@ static GtkActionEntry actions_file[] = {
   { "ActionFileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."),  "<shift><control>s",
 	N_("Save the current file with a different name"),
     G_CALLBACK (on_save_as_activate)},
-  { "ActionFileSaveAll", NULL, N_("Save A_ll"), NULL,
+  { "ActionFileSaveAll", GTK_STOCK_SAVE, N_("Save A_ll"), "<shift><control>l",
 	N_("Save all currently open files, except new files"),
     G_CALLBACK (on_save_all_activate)},
   { "ActionFileClose", GTK_STOCK_CLOSE, N_("_Close File"), "<control>w",
@@ -258,10 +255,10 @@ static GtkActionEntry actions_navigation[] = {
 	"<control><alt>e", N_("Go to the end of the current block"),
     G_CALLBACK (on_goto_block_end1_activate)},
   { "ActionEditGotoHistoryPrev", ANJUTA_STOCK_HISTORY_PREV, N_("Previous _History"),
-	NULL, N_("Goto previous history"),
+	"<alt>Left", N_("Goto previous history"),
     G_CALLBACK (on_prev_history)},
   { "ActionEditGotoHistoryNext", ANJUTA_STOCK_HISTORY_NEXT, N_("Next Histor_y"),
-	 NULL, N_("Goto next history"),
+	"<alt>Right", N_("Goto next history"),
     G_CALLBACK (on_next_history)}
 };
 
@@ -293,7 +290,7 @@ static GtkActionEntry actions_edit[] = {
   { "ActionEditUndo", GTK_STOCK_UNDO, N_("U_ndo"), "<control>z",
 	N_("Undo the last action"),
     G_CALLBACK (on_editor_command_undo_activate)},
-  { "ActionEditRedo", GTK_STOCK_REDO, N_("_Redo"), "<control>r",
+  { "ActionEditRedo", GTK_STOCK_REDO, N_("_Redo"), "<shift><control>z",
 	N_("Redo the last undone action"),
     G_CALLBACK (on_editor_command_redo_activate)},
   { "ActionEditCut", GTK_STOCK_CUT, N_("C_ut"), "<control>x",
@@ -341,10 +338,10 @@ static GtkToggleActionEntry actions_view[] = {
 };
 
 static GtkActionEntry actions_zoom[] = {
-  { "ActionViewEditorZoomIn", GTK_STOCK_ZOOM_IN, N_("Zoom In"), "<control>plus",
+  { "ActionViewEditorZoomIn", GTK_STOCK_ZOOM_IN, N_("Zoom In"), "<control>KP_Add",
 	N_("Zoom in: Increase font size"),
     G_CALLBACK (on_zoom_in_text_activate)},
-  { "ActionViewEditorZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom Out"), "<control>minus",
+  { "ActionViewEditorZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom Out"), "<control>KP_Subtract",
 	N_("Zoom out: Decrease font size"),
     G_CALLBACK (on_zoom_out_text_activate)}
 };
@@ -365,28 +362,6 @@ static GtkActionEntry actions_format[] = {
     G_CALLBACK (on_editor_command_toggle_fold_activate)},
 };
 
-static GtkActionEntry actions_bookmark[] = {
-  { "ActionMenuBookmark", NULL, N_("Bookmar_k"), NULL, NULL, NULL},
-  { "ActionBookmarkToggle", ANJUTA_STOCK_BOOKMARK_TOGGLE, N_("_Toggle Bookmark"),
-	"<control>k", N_("Toggle a bookmark at the current line position"),
-    G_CALLBACK (on_editor_command_bookmark_toggle_activate)},
-  { "ActionBookmarkFirst", ANJUTA_STOCK_BOOKMARK_FIRST, N_("_First Bookmark"),
-	NULL, N_("Jump to the first bookmark in the file"),
-    G_CALLBACK (on_editor_command_bookmark_first_activate)},
-  { "ActionBookmarkPrevious", ANJUTA_STOCK_BOOKMARK_PREV, N_("_Previous Bookmark"),
-	"<control>comma", N_("Jump to the previous bookmark in the file"),
-    G_CALLBACK (on_editor_command_bookmark_prev_activate)},
-  { "ActionBookmarkNext", ANJUTA_STOCK_BOOKMARK_NEXT, N_("_Next Bookmark"),
-	"<control>period", N_("Jump to the next bookmark in the file"),
-    G_CALLBACK (on_editor_command_bookmark_next_activate)},
-  { "ActionBookmarkLast", ANJUTA_STOCK_BOOKMARK_LAST, N_("_Last Bookmark"),
-	NULL, N_("Jump to the last bookmark in the file"),
-    G_CALLBACK (on_editor_command_bookmark_last_activate)},
-  { "ActionBookmarkClear", ANJUTA_STOCK_BOOKMARK_CLEAR, N_("_Clear All Bookmarks"),
-	NULL, N_("Clear bookmarks"),
-    G_CALLBACK (on_editor_command_bookmark_clear_activate)},
-};
-
 static GtkActionEntry actions_documents[] = {
   {"ActionMenuDocuments", NULL, N_("_Documents"), NULL, NULL, NULL},
   { "ActionDocumentsPrevious", GTK_STOCK_GO_BACK, N_("Previous Document"),
@@ -397,6 +372,22 @@ static GtkActionEntry actions_documents[] = {
     G_CALLBACK (on_next_document)},
 };
 
+static GtkActionEntry actions_bookmarks[] = {
+	{ "ActionMenuBookmark", NULL, N_("Bookmar_k"), NULL, NULL, NULL},
+ 	{ "ActionBookmarkAdd", GTK_STOCK_ADD, N_("_Add Bookmark"),
+		 "<control>k", N_("Add a bookmark at the current line position"),
+	G_CALLBACK (on_bookmark_add_activate)},
+	{ "ActionBookmarkPrev", ANJUTA_STOCK_BOOKMARK_PREV, N_("_Previous Bookmark"),
+	"<control>comma", N_("Jump to the previous bookmark in the file"),
+	G_CALLBACK (on_bookmark_prev_activate)},
+ 	{ "ActionBookmarkNext", ANJUTA_STOCK_BOOKMARK_NEXT, N_("_Next Bookmark"),
+ 	"<control>period", N_("Jump to the next bookmark in the file"),
+ 	G_CALLBACK (on_bookmark_next_activate)},
+ 	{ "ActionBookmarksClear", ANJUTA_STOCK_BOOKMARK_CLEAR, N_("_Clear All Bookmarks"),
+ 	NULL, N_("Clear bookmarks"),
+ 	G_CALLBACK (on_bookmarks_clear_activate)},
+ }; 
+
 struct ActionGroupInfo {
 	GtkActionEntry *group;
 	gint size;
@@ -423,9 +414,9 @@ static struct ActionGroupInfo action_groups[] = {
 	{ actions_zoom, G_N_ELEMENTS (actions_zoom), "ActionGroupEditorZoom", N_("Editor zoom operations") },
 	{ actions_style, G_N_ELEMENTS (actions_style), "ActionGroupEditorStyle", N_("Editor syntax highlighting styles") },
 	{ actions_format, G_N_ELEMENTS (actions_format), "ActionGroupEditorFormat", N_("Editor text formating") },
-	{ actions_bookmark, G_N_ELEMENTS (actions_bookmark), "ActionGroupEditorBookmark", N_("Editor bookmarks") },
 	{ actions_search, G_N_ELEMENTS (actions_search), "ActionGroupEditorSearch", N_("Simple searching") },
-	{ actions_documents, G_N_ELEMENTS (actions_documents), "ActionGroupDocuments", N_("Documents") }
+	{ actions_documents, G_N_ELEMENTS (actions_documents), "ActionGroupDocuments", N_("Documents") },
+	{ actions_bookmarks, G_N_ELEMENTS (actions_bookmarks), "ActionGroupBookmarks", N_("Bookmarks") }
 };
 
 static struct ActionToggleGroupInfo action_toggle_groups[] = {
@@ -533,7 +524,7 @@ value_added_project_root_uri (AnjutaPlugin *plugin, const gchar *name,
 
 	doc_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 	
-	DEBUG_PRINT ("Project added");
+	DEBUG_PRINT ("%s", "Project added");
 	
 	
 	g_free (doc_plugin->project_name);
@@ -565,7 +556,7 @@ value_removed_project_root_uri (AnjutaPlugin *plugin, const gchar *name,
 
 	doc_plugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 	
-	DEBUG_PRINT ("Project removed");
+	DEBUG_PRINT ("%s", "Project removed");
 	
 	g_free (doc_plugin->project_name);
 	doc_plugin->project_name = NULL;
@@ -785,6 +776,13 @@ update_document_ui_interface_items (AnjutaPlugin *plugin, IAnjutaDocument *doc)
 	action = anjuta_ui_get_action (ui, "ActionGroupEditorEdit",
 								   "ActionViewEditorRemoveView");
 	g_object_set (G_OBJECT (action), "visible", flag, "sensitive", flag, NULL);
+
+	/* Actually this does not really belong here but it would be annoying to
+	 * add another interface for it
+	 */
+	action = anjuta_ui_get_action (ui, "ActionGroupEditorView",
+								   "ActionViewEditorGuides");
+	g_object_set (G_OBJECT (action), "visible", flag, "sensitive", flag, NULL);
 	
 	/* IAnjutaEditorFolds */
 	flag = IANJUTA_IS_EDITOR_FOLDS (doc);
@@ -887,11 +885,9 @@ register_stock_icons (AnjutaPlugin *plugin)
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_INDENT_INC, ANJUTA_STOCK_INDENT_INC);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BLOCK_SELECT, ANJUTA_STOCK_BLOCK_SELECT);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_TOGGLE, ANJUTA_STOCK_BOOKMARK_TOGGLE);
-	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_FIRST, ANJUTA_STOCK_BOOKMARK_FIRST);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_PREV, ANJUTA_STOCK_BOOKMARK_PREV);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_NEXT, ANJUTA_STOCK_BOOKMARK_NEXT);
-	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_LAST, ANJUTA_STOCK_BOOKMARK_LAST);
-	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_CLEAR, ANJUTA_STOCK_BOOKMARK_CLEAR);
+	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BOOKMARK_CLEAR, ANJUTA_STOCK_BOOKMARK_CLEAR); 
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BLOCK_START, ANJUTA_STOCK_BLOCK_START);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BLOCK_END, ANJUTA_STOCK_BLOCK_END);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_GOTO_LINE, ANJUTA_STOCK_GOTO_LINE);
@@ -947,7 +943,12 @@ update_status (DocmanPlugin *plugin, IAnjutaEditor *te)
 		{
 			edit = g_strdup (_("INS"));
 		}
-		anjuta_status_set_default (status, _("Zoom"), "%d", zoom);
+		
+		if (IANJUTA_IS_EDITOR_ZOOM(te))
+			anjuta_status_set_default (status, _("Zoom"), "%d", zoom);
+		else
+			anjuta_status_set_default (status, _("Zoom"), NULL);
+		
 		anjuta_status_set_default (status, _("Line"), "%04d", line);
 		anjuta_status_set_default (status, _("Col"), "%03d", col);
 		anjuta_status_set_default (status, _("Mode"), edit);
@@ -1152,7 +1153,7 @@ unload_unused_support_plugins (DocmanPlugin* docman_plugin,
 		AnjutaPlugin* plugin = ANJUTA_PLUGIN (node->data);
 		if (!g_list_find (needed_plugins, plugin))
 		{
-			DEBUG_PRINT ("Unloading plugin");
+			DEBUG_PRINT ("%s", "Unloading plugin");
 			anjuta_plugin_deactivate (plugin);
 		}
 	}
@@ -1190,7 +1191,7 @@ on_document_changed (AnjutaDocman *docman, IAnjutaDocument *doc,
 
 		if (IANJUTA_IS_EDITOR_LANGUAGE (doc))
 		{
-			DEBUG_PRINT("Beginning language support");
+			DEBUG_PRINT("%s", "Beginning language support");
 			AnjutaPluginManager *plugin_manager;
 			IAnjutaLanguage *lang_manager;
 			GList *new_support_plugins, *support_plugin_descs, *needed_plugins, *node;
@@ -1209,7 +1210,7 @@ on_document_changed (AnjutaDocman *docman, IAnjutaDocument *doc,
 															  NULL);
 			if (!language)
 			{
-				DEBUG_PRINT ("Unloading all language support plugins");
+				DEBUG_PRINT ("%s", "Unloading all language support plugins");
 				/* Unload all language support plugins */
 				/* Copy the list because the "deactivate"-signal handler modifies
 				 * the original list */
@@ -1372,6 +1373,17 @@ on_window_key_release_event (AnjutaShell *shell,
 	return FALSE;
 }
 
+static void
+on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
+				 AnjutaSession *session, DocmanPlugin *plugin)
+{
+	if (phase != ANJUTA_SESSION_PHASE_NORMAL)
+		return;
+	
+	anjuta_bookmarks_session_load (ANJUTA_BOOKMARKS (plugin->bookmarks),
+								   session);
+}
+
 static void
 on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 				 AnjutaSession *session, DocmanPlugin *plugin)
@@ -1416,6 +1428,9 @@ on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 		g_list_foreach (files, (GFunc)g_free, NULL);
 		g_list_free (files);
 	}
+	
+	anjuta_bookmarks_session_save (ANJUTA_BOOKMARKS (plugin->bookmarks),
+								   session);
 }
 
 static gboolean
@@ -1445,16 +1460,19 @@ on_save_prompt (AnjutaShell *shell, AnjutaSavePrompt *save_prompt,
 			if (ianjuta_file_savable_is_dirty (editor, NULL))
 			{
 				const gchar *name;
-				gchar *uri;
+				gchar *uri = NULL;
 				GFile* file;
 			
 				name = ianjuta_document_get_filename (IANJUTA_DOCUMENT (editor), NULL);
 				file = ianjuta_file_get_file (IANJUTA_FILE (editor), NULL);
-				uri = g_file_get_uri (file);
+				if (file)
+				{
+					uri = g_file_get_uri (file);
+					g_object_unref (file);
+				}
 				anjuta_save_prompt_add_item (save_prompt, name, uri, editor,
 											 on_save_prompt_save_editor, plugin);
 				g_free (uri);
-				g_object_unref (file);
 			}
 		}
 		g_list_free (buffers);
@@ -1592,23 +1610,15 @@ on_gconf_notify_timer (GConfClient *gclient, guint cnxn_id,
 			{
 				g_source_remove (plugin->autosave_id);
 				plugin->autosave_id =
-#if GLIB_CHECK_VERSION (2,14,0)
 					g_timeout_add_seconds (auto_save_timer * 60,
-#else
-					g_timeout_add (auto_save_timer * 60000,
-#endif
-									on_docman_auto_save, plugin);
+										   on_docman_auto_save, plugin);
 			}
 		}
 		else
 		{
 			plugin->autosave_id =
-#if GLIB_CHECK_VERSION (2,14,0)
 				g_timeout_add_seconds (auto_save_timer * 60,
-#else
-				g_timeout_add (auto_save_timer * 60000,
-#endif
-							 on_docman_auto_save, plugin);
+									   on_docman_auto_save, plugin);
 		}
 		plugin->autosave_it = auto_save_timer;
 		plugin->autosave_on = TRUE;
@@ -1664,7 +1674,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	static gboolean initialized = FALSE;
 	GList *actions, *act;
 	
-	DEBUG_PRINT ("DocmanPlugin: Activating document manager plugin...");
+	DEBUG_PRINT ("%s", "DocmanPlugin: Activating document manager plugin...");
 	
 	dplugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 	dplugin->ui = anjuta_shell_get_ui (plugin->shell, NULL);
@@ -1758,10 +1768,17 @@ activate_plugin (AnjutaPlugin *plugin)
 	/* Connect to save session */
 	g_signal_connect (G_OBJECT (plugin->shell), "save-session",
 					  G_CALLBACK (on_session_save), plugin);
+	/* Connect to load session */
+	g_signal_connect (G_OBJECT (plugin->shell), "load-session",
+					  G_CALLBACK (on_session_load), plugin);
+	
 	/* Connect to save prompt */
 	g_signal_connect (G_OBJECT (plugin->shell), "save-prompt",
 					  G_CALLBACK (on_save_prompt), plugin);
 	
+	/* Add bookmarks widget */
+	dplugin->bookmarks = G_OBJECT(anjuta_bookmarks_new (dplugin));
+	
 	dplugin->project_watch_id =
 		anjuta_plugin_add_watch (plugin, IANJUTA_PROJECT_MANAGER_PROJECT_ROOT_URI,
 								 value_added_project_root_uri,
@@ -1783,7 +1800,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	AnjutaStatus *status;
 	GList *node;
 	
-	DEBUG_PRINT ("DocmanPlugin: Deactivating document manager plugin...");
+	DEBUG_PRINT ("%s", "DocmanPlugin: Deactivating document manager plugin...");
 	
 	eplugin = ANJUTA_PLUGIN_DOCMAN (plugin);
 
@@ -1811,6 +1828,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	/* Widget is removed from the container when destroyed */
 	gtk_widget_destroy (eplugin->docman);
 	gtk_widget_destroy (eplugin->search_box);	/* the default searchbox instance may still be unparented */
+	g_object_unref (eplugin->bookmarks);
 	anjuta_ui_unmerge (ui, eplugin->uiid);
 	node = eplugin->action_groups;
 	while (node)
@@ -2005,6 +2023,16 @@ ianjuta_docman_remove_document(IAnjutaDocumentManager *plugin,
 	return ret_val;
 }
 
+static void
+ianjuta_docman_add_bookmark (IAnjutaDocumentManager* plugin,
+									  GFile* file,
+									  gint  line,
+									  GError **e)
+{
+	AnjutaBookmarks* bookmarks = ANJUTA_BOOKMARKS (ANJUTA_PLUGIN_DOCMAN(plugin)->bookmarks);
+	anjuta_bookmarks_add_file (bookmarks, file, line, NULL);
+}
+
 static void
 ianjuta_document_manager_iface_init (IAnjutaDocumentManagerIface *iface)
 {
@@ -2019,6 +2047,7 @@ ianjuta_document_manager_iface_init (IAnjutaDocumentManagerIface *iface)
 	iface->remove_document = ianjuta_docman_remove_document;
 	iface->set_current_document = ianjuta_docman_set_current_document;
 	iface->set_message_area = ianjuta_docman_set_message_area;
+	iface->add_bookmark = ianjuta_docman_add_bookmark;
 }
 
 /* Implement IAnjutaFile interface */
@@ -2082,7 +2111,7 @@ isaveable_save (IAnjutaFileSavable* plugin, GError** e)
 static void
 isavable_save_as (IAnjutaFileSavable* plugin, GFile* file, GError** e)
 {
-	DEBUG_PRINT("save_as: Not implemented in DocmanPlugin");
+	DEBUG_PRINT("%s", "save_as: Not implemented in DocmanPlugin");
 }
 
 static gboolean
@@ -2116,7 +2145,7 @@ isavable_is_dirty (IAnjutaFileSavable* plugin, GError **e)
 static void
 isavable_set_dirty(IAnjutaFileSavable* plugin, gboolean dirty, GError** e)
 {
-	DEBUG_PRINT("set_dirty: Not implemented in DocmanPlugin");
+	DEBUG_PRINT("%s", "set_dirty: Not implemented in DocmanPlugin");
 }
 
 static void
