@@ -34,25 +34,31 @@
 #define GLADE_FILE PACKAGE_DATA_DIR"/glade/anjuta-editor-scintilla.glade"
 
 gchar *hilite_style[] = {
-	"Normal <Default>", "style.*.32",
-	"Comments", "style.default.comment",
-	"Numbers", "style.default.number",
-	"Keywords", "style.default.keyword",
-	"System Keywords", "style.default.syskeyword",
-	"Local Keywords", "style.default.localkeyword",
-	"Double Quoted Strings", "style.default.doublequote",
-	"Single Quoted Strings", "style.default.singlequote",
-	"Unclosed Strings", "style.default.unclosedstring",
-	"Operators","style.default.operator",
-	"Preprocessor Directives", "style.default.preprocessor",
-	"Identifiers (Not C Style)", "style.default.identifier",
-	"Definitions (Not C Style)", "style.default.definition",
-	"Functions (Not C Style)", "style.default.function",
-	"Matched Braces", "style.*.34",
-	"Incomplete Brace", "style.*.35",
-	"Control Characters", "style.*.36",
-	"Line Numbers", "style.*.33",
-	"Indentation Guides", "style.*.37",
+	"Normal <Default>", "style.anjuta.normal",
+	"White space", "style.anjuta.whitespace",
+	"Comments", "style.anjuta.comment",
+	"Numbers", "style.anjuta.number",
+	"Operators","style.anjuta.operator",
+	"Keywords", "style.anjuta.keyword",
+	"System Keywords", "style.anjuta.syskeyword",
+	"Local Keywords", "style.anjuta.localkeyword",
+	"Additional Keywords", "style.anjuta.extrakeyword",
+	"Characters", "style.anjuta.char",
+	"Strings", "style.anjuta.string",
+	"Raw Strings", "style.anjuta.raws",
+	"Regular Expression", "style.anjuta.regex",
+	"Unclosed Strings", "style.anjuta.unclosed",
+	"Preprocessor Directives", "style.anjuta.preprocessor",
+	"Errors", "style.anjuta.error",
+	"Identifiers (Not C Style)", "style.anjuta.identifier",
+	"Definitions (Not C Style)", "style.anjuta.definition",
+	"Functions (Not C Style)", "style.anjuta.function",
+	"Attribute (Not C Style)", "style.anjuta.attribute",
+	"Matched Braces", "style.anjuta.matchbrace",
+	"Incomplete Brace", "style.anjuta.singlebrace",
+	"Control Characters", "style.anjuta.controlchar",
+	"Line Numbers", "style.anjuta.linenumber",
+	"Indentation Guides", "style.anjuta.indentguide",
 	NULL, NULL
 };
 
