@@ -35,6 +35,9 @@
 #include <libanjuta/anjuta-encodings.h>
 #include <libanjuta/anjuta-convert.h>
 #include <libanjuta/anjuta-debug.h>
+#include <libanjuta/anjuta-message-area.h>
+#include <libanjuta/anjuta-shell.h>
+#include <libanjuta/interfaces/ianjuta-document-manager.h>
 #include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-editor-selection.h>
 #include <libanjuta/interfaces/ianjuta-editor-convert.h>
@@ -48,7 +51,6 @@
 #include <libanjuta/interfaces/ianjuta-editor-assist.h>
 #include <libanjuta/interfaces/ianjuta-editor-search.h>
 #include <libanjuta/interfaces/ianjuta-editor-hover.h>
-#include <libanjuta/interfaces/ianjuta-bookmark.h>
 #include <libanjuta/interfaces/ianjuta-editor-factory.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
@@ -181,13 +183,13 @@ initialize_markers (TextEditor* te, GtkWidget *scintilla)
 static void
 on_scintila_already_destroyed (gpointer te, GObject *obj)
 {
-	/* DEBUG_PRINT ("Scintilla object has been destroyed"); */
+	/* DEBUG_PRINT ("%s", "Scintilla object has been destroyed"); */
 }
 
 static void
 on_te_already_destroyed (gpointer te, GObject *obj)
 {
-	/* DEBUG_PRINT ("TextEditor object has been destroyed"); */
+	/* DEBUG_PRINT ("%s", "TextEditor object has been destroyed"); */
 }
 #endif
 
@@ -304,59 +306,15 @@ text_editor_remove_view (TextEditor *te)
 }
 
 static void
-on_reload_dialog_response (GtkWidget *dlg, gint res, TextEditor *te)
+on_reload_dialog_response (GtkWidget *message_area, gint res, TextEditor *te)
 {
 	if (res == GTK_RESPONSE_YES)
 	{
 		text_editor_load_file (te);
 	}
-	gtk_widget_destroy (dlg);
+	gtk_widget_destroy (message_area);
 	te->file_modified_widget = NULL;
-	/* DEBUG_PRINT ("File modified dialog responded"); */
-}
-
-static gboolean
-on_text_editor_uri_changed_prompt (TextEditor *te)
-{
-	GtkWidget *dlg;
-	GtkWidget *parent;
-	gchar *buff;
-	
-	buff =
-		g_strdup_printf (_
-						 ("The file '%s' on the disk is more recent than\n"
-						  "the current buffer.\nDo you want to reload it?"),
-						 te->filename);
-	
-	parent = gtk_widget_get_toplevel (GTK_WIDGET (te));
-	
-	dlg = gtk_message_dialog_new (GTK_WINDOW (parent),
-								  GTK_DIALOG_DESTROY_WITH_PARENT,
-								  GTK_MESSAGE_WARNING,
-								  GTK_BUTTONS_NONE, buff);
-	gtk_dialog_add_button (GTK_DIALOG (dlg),
-						   GTK_STOCK_NO,
-						   GTK_RESPONSE_NO);
-	anjuta_util_dialog_add_button (GTK_DIALOG (dlg),
-								   _("_Reload"),
-								   GTK_STOCK_REFRESH,
-								   GTK_RESPONSE_YES);
-	g_free (buff);
-	
-	gtk_window_set_transient_for (GTK_WINDOW (dlg),
-								  GTK_WINDOW (parent));
-	
-	g_signal_connect (dlg, "response",
-					  G_CALLBACK (on_reload_dialog_response),
-					  te);
-	g_signal_connect_swapped (dlg, "delete-event",
-					  G_CALLBACK (gtk_widget_destroy),
-					  dlg);
-	gtk_widget_show (dlg);
-	
-	te->file_modified_widget = dlg;
-
-	return FALSE;
+	/* DEBUG_PRINT ("%s", "File modified dialog responded"); */
 }
 
 static void
@@ -367,13 +325,17 @@ on_text_editor_uri_changed (GFileMonitor *monitor,
 							gpointer user_data)
 {
 	TextEditor *te = TEXT_EDITOR (user_data);
+	GtkWidget *message_area;
+	IAnjutaDocumentManager *docman;
+	IAnjutaDocument *doc;
+	gchar *buff;
 	
-	/* DEBUG_PRINT ("File changed!!!"); */
+	/* DEBUG_PRINT ("%s", "File changed!!!"); */
 	
 	if (!(event_type == G_FILE_MONITOR_EVENT_CHANGED ||
 		  event_type == G_FILE_MONITOR_EVENT_CREATED))
 		return;
-	
+
 	if (!anjuta_util_diff (te->uri, te->last_saved_content))
 	{
 		/* The file content is same. Remove any previous prompt for reload */
@@ -382,18 +344,33 @@ on_text_editor_uri_changed (GFileMonitor *monitor,
 		te->file_modified_widget = NULL;
 		return;
 	}
+
+	buff =
+		g_strdup_printf (_
+						 ("The file '%s' on the disk is more recent than\n"
+						  "the current buffer.\nDo you want to reload it?"),
+						 te->filename);
 	
-	/* If the file modified dialog is already shown, don't bother showing it
-	 * again.
-	 */
-	if (te->file_modified_widget)
-		return;
+	docman = anjuta_shell_get_interface (te->shell, IAnjutaDocumentManager, NULL);
+	if (!docman)
+  		return;
+  	doc = IANJUTA_DOCUMENT (te);
+
+	message_area = anjuta_message_area_new (buff, GTK_STOCK_DIALOG_WARNING);
+	anjuta_message_area_add_button (ANJUTA_MESSAGE_AREA (message_area),
+									GTK_STOCK_REFRESH,
+									GTK_RESPONSE_YES);
+	anjuta_message_area_add_button (ANJUTA_MESSAGE_AREA (message_area),
+								    GTK_STOCK_CANCEL,
+									GTK_RESPONSE_NO);
+	g_free (buff);	
+	te->file_modified_widget = message_area;
 	
-	/* Set up 1 sec timer */
-	if (te->file_modified_timer > 0)
- 		g_source_remove (te->file_modified_timer);
-	te->file_modified_timer = g_timeout_add (1000,
-							(GSourceFunc)on_text_editor_uri_changed_prompt, te);
+	g_signal_connect (G_OBJECT(message_area), "response",
+					  G_CALLBACK (on_reload_dialog_response),
+					  te);					  
+	ianjuta_document_manager_set_message_area (docman, doc, message_area, NULL);
+
 }
 
 static void
@@ -409,7 +386,7 @@ text_editor_update_monitor (TextEditor *te, gboolean disable_it)
 	{
 		GFile *gio_uri;
 		GError *error = NULL;
-		/* DEBUG_PRINT ("Setting up Monitor for %s", te->uri); */
+		/* DEBUG_PRINT ("%s", "Setting up Monitor for %s", te->uri); */
 
 		gio_uri = g_file_new_for_uri (te->uri);
 		te->monitor = g_file_monitor_file (gio_uri, 
@@ -422,7 +399,7 @@ text_editor_update_monitor (TextEditor *te, gboolean disable_it)
 		
 		if (error != NULL)
 		{
-			DEBUG_PRINT ("Error while setting up file monitor: %s",
+			DEBUG_PRINT ("%s", "Error while setting up file monitor: %s",
 					   error->message);
 			g_error_free (error);
 		}
@@ -431,13 +408,14 @@ text_editor_update_monitor (TextEditor *te, gboolean disable_it)
 }
 
 GtkWidget *
-text_editor_new (AnjutaStatus *status, AnjutaPreferences *eo, const gchar *uri, const gchar *name)
+text_editor_new (AnjutaStatus *status, AnjutaPreferences *eo, AnjutaShell *shell, const gchar *uri, const gchar *name)
 {
 	gint zoom_factor;
 	static guint new_file_count;
 	TextEditor *te = TEXT_EDITOR (gtk_widget_new (TYPE_TEXT_EDITOR, NULL));
 	
 	te->status = status; 
+	te->shell = shell;
 	
 	te->preferences = eo;
 	te->props_base = text_editor_get_props();
@@ -479,7 +457,7 @@ text_editor_new (AnjutaStatus *status, AnjutaPreferences *eo, const gchar *uri,
 	
 	/* Apply font zoom separately */
 	zoom_factor = anjuta_preferences_get_int (te->preferences, TEXT_ZOOM_FACTOR);
-	/* DEBUG_PRINT ("Initializing zoom factor to: %d", zoom_factor); */
+	/* DEBUG_PRINT ("%s", "Initializing zoom factor to: %d", zoom_factor); */
 	text_editor_set_zoom_factor (te, zoom_factor);
 	
 #ifdef DEBUG
@@ -492,11 +470,6 @@ void
 text_editor_dispose (GObject *obj)
 {
 	TextEditor *te = TEXT_EDITOR (obj);
-	if (te->file_modified_timer > 0)
-	{
-		g_source_remove (te->file_modified_timer);
-		te->file_modified_timer = 0;
-	}
 	if (te->monitor)
 	{
 		text_editor_update_monitor (te, TRUE);
@@ -1293,7 +1266,7 @@ load_from_file (TextEditor *te, gchar *uri, gchar **err)
 	buffer = g_malloc (size + 1);
 	if (buffer == NULL && size != 0)
 	{
-		/* DEBUG_PRINT ("This file is too big. Unable to allocate memory."); */
+		/* DEBUG_PRINT ("%s", "This file is too big. Unable to allocate memory."); */
 		*err = g_strdup (_("This file is too big. Unable to allocate memory."));
 		g_object_unref (gio_uri);
 
@@ -1380,7 +1353,7 @@ load_from_file (TextEditor *te, gchar *uri, gchar **err)
 							nchars, (long) buffer);
 	
 	g_free (buffer);
-	
+
 	/* Save the buffer as last saved content */
 	g_free (te->last_saved_content);
 	te->last_saved_content = file_content;
@@ -1401,7 +1374,7 @@ save_to_file (TextEditor *te, gchar *uri, GError **error)
 	gboolean result;
 
 	gio_uri = g_file_new_for_uri (uri);
-	stream = g_file_replace (gio_uri, NULL, FALSE, G_FILE_CREATE_NONE, NULL, NULL);
+	stream = g_file_replace (gio_uri, NULL, FALSE, G_FILE_CREATE_NONE, NULL, error);
 
  	if (stream == NULL)
 		return FALSE;
@@ -1562,6 +1535,8 @@ text_editor_save_file (TextEditor * te, gboolean update)
 	if (!save_to_file (te, te->uri, &error))
 	{	
 		text_editor_thaw (te);
+		g_return_val_if_fail (error != NULL, FALSE);
+		
 		anjuta_util_dialog_error (parent,
 								  _("Could not save intermediate file %s: %s"),
 								  te->uri,
@@ -2104,6 +2079,7 @@ static void
 itext_editor_goto_line (IAnjutaEditor *editor, gint lineno, GError **e)
 {
 	text_editor_goto_line (TEXT_EDITOR (editor), lineno, FALSE, TRUE);
+	gtk_widget_grab_focus (TEXT_EDITOR (editor)->scintilla);
 }
 
 static void
@@ -3158,56 +3134,6 @@ iview_iface_init (IAnjutaEditorViewIface *iface)
 	iface->get_count = iview_get_count;
 }
 
-/* IAnjutaBookmark implementation */
-static void
-ibookmark_toggle(IAnjutaBookmark* view, gint location,
-			  gboolean ensure_visible, GError **e)
-{
-	text_editor_goto_line (TEXT_EDITOR(view), location, FALSE, ensure_visible);
-	text_editor_command(TEXT_EDITOR(view), ANE_BOOKMARK_TOGGLE, 0, 0);
-}
-
-static void
-ibookmark_first(IAnjutaBookmark* view, GError **e)
-{
-	text_editor_command(TEXT_EDITOR(view), ANE_BOOKMARK_FIRST, 0, 0);
-}
-
-static void
-ibookmark_last(IAnjutaBookmark* view, GError **e)
-{
-	text_editor_command(TEXT_EDITOR(view), ANE_BOOKMARK_LAST, 0, 0);
-}
-
-static void
-ibookmark_next(IAnjutaBookmark* view, GError **e)
-{
-	text_editor_command(TEXT_EDITOR(view), ANE_BOOKMARK_NEXT, 0, 0);
-}
-
-static void
-ibookmark_previous(IAnjutaBookmark* view, GError **e)
-{
-	text_editor_command(TEXT_EDITOR(view), ANE_BOOKMARK_PREV, 0, 0);
-}
-
-static void
-ibookmark_clear_all(IAnjutaBookmark* view, GError **e)
-{
-	text_editor_command(TEXT_EDITOR(view), ANE_BOOKMARK_CLEAR, 0, 0);
-}
-
-static void
-ibookmark_iface_init(IAnjutaBookmarkIface* iface)
-{
-	iface->toggle = ibookmark_toggle;
-	iface->first = ibookmark_first;
-	iface->last = ibookmark_last;
-	iface->next = ibookmark_next;
-	iface->previous = ibookmark_previous;
-	iface->clear_all = ibookmark_clear_all;
-}
-
 static void
 iindicable_set (IAnjutaIndicable *te, IAnjutaIterable *begin_location,
 				IAnjutaIterable *end_location,
@@ -3656,7 +3582,6 @@ ANJUTA_TYPE_ADD_INTERFACE(iassist, IANJUTA_TYPE_EDITOR_ASSIST);
 ANJUTA_TYPE_ADD_INTERFACE(ilanguage, IANJUTA_TYPE_EDITOR_LANGUAGE);
 ANJUTA_TYPE_ADD_INTERFACE(iview, IANJUTA_TYPE_EDITOR_VIEW);
 ANJUTA_TYPE_ADD_INTERFACE(ifolds, IANJUTA_TYPE_EDITOR_FOLDS);
-ANJUTA_TYPE_ADD_INTERFACE(ibookmark, IANJUTA_TYPE_BOOKMARK);
 ANJUTA_TYPE_ADD_INTERFACE(imarkable, IANJUTA_TYPE_MARKABLE);
 ANJUTA_TYPE_ADD_INTERFACE(iindicable, IANJUTA_TYPE_INDICABLE);
 ANJUTA_TYPE_ADD_INTERFACE(iprint, IANJUTA_TYPE_PRINT);
