@@ -19,10 +19,6 @@
 */
 
 #include <config.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
 
 #include <glib/gi18n.h>
 
@@ -67,46 +63,49 @@ sort_wizards(gconstpointer wizard1, gconstpointer wizard2)
 		return 0;
 }
 
+/* The add argument is here to remember that recent menu items should be removed
+ * when the destination does not exist anymore */
 static void
-set_recent_file (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
-				 const gchar *mime)
+update_recent_file (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
+				 const gchar *mime, gboolean add)
 {
-	GtkRecentData *recent_data;
-	gchar *name;
 	
-	DEBUG_PRINT ("Adding recent item of mimi-type: %s", mime);
-	
-	name = g_path_get_basename (uri);
-	
-	recent_data = g_slice_new (GtkRecentData);
-
-	recent_data->display_name   = name;
-	recent_data->description    = NULL;
-	recent_data->mime_type      = (gchar *)mime;
-	recent_data->app_name       = (gchar *) g_get_application_name ();
-	recent_data->app_exec       = g_strjoin (" ", g_get_prgname (), "%u", NULL);
-	recent_data->groups         = NULL;
-	recent_data->is_private     = FALSE;
-
-	if (!gtk_recent_manager_add_full (plugin->recent_manager, uri, recent_data))
+	if (add)
 	{
+		GtkRecentData *recent_data;
+	
+		recent_data = g_slice_new (GtkRecentData);
+
+		recent_data->display_name = NULL;
+		recent_data->description = NULL;
+		recent_data->mime_type = (gchar *)mime;
+		recent_data->app_name = (gchar *) g_get_application_name ();
+		recent_data->app_exec = g_strjoin (" ", g_get_prgname (), "%u", NULL);
+		recent_data->groups = NULL;
+		recent_data->is_private = FALSE;
+	
+		if (!gtk_recent_manager_add_full (plugin->recent_manager, uri, recent_data))
+		{
       		g_warning ("Unable to add '%s' to the list of recently used documents", uri);
+		}
+
+		g_free (recent_data->app_exec);
+		g_slice_free (GtkRecentData, recent_data);
+	}
+	else
+	{
+		gtk_recent_manager_remove_item (plugin->recent_manager, uri, NULL);
 	}
 
-	g_free (name);
-	g_free (recent_data->app_exec);
-	g_slice_free (GtkRecentData, recent_data);
 }
 
 static void
 launch_application_failure (AnjutaFileLoaderPlugin *plugin,
 							const gchar *uri,
-							GnomeVFSResult result)
+							const gchar *errmsg)
 {
-	const gchar *errmsg;
 	GtkWidget *parent;
 	
-	errmsg = gnome_vfs_result_to_string (result);
 	parent =
 		gtk_widget_get_toplevel (GTK_WIDGET(ANJUTA_PLUGIN (plugin)->shell));
 	anjuta_util_dialog_error (GTK_WINDOW (parent),
@@ -115,7 +114,7 @@ launch_application_failure (AnjutaFileLoaderPlugin *plugin,
 }
 
 static GList *
-get_available_plugins_for_mime (AnjutaFileLoaderPlugin* plugin,
+get_available_plugins_for_mime (AnjutaPlugin* plugin,
 							   const gchar *mime_type)
 {
 	AnjutaPluginManager *plugin_manager;
@@ -123,8 +122,10 @@ get_available_plugins_for_mime (AnjutaFileLoaderPlugin* plugin,
 	
 	g_return_val_if_fail (mime_type != NULL, NULL);
 	
-	plugin_manager = anjuta_shell_get_plugin_manager (ANJUTA_PLUGIN(plugin)->shell,
+	plugin_manager = anjuta_shell_get_plugin_manager (plugin->shell,
 													  NULL);
+	
+	/* Check an exact match */
 	plugin_descs = anjuta_plugin_manager_query (plugin_manager,
 												"Anjuta Plugin",
 												"Interfaces", "IAnjutaFile",
@@ -132,17 +133,55 @@ get_available_plugins_for_mime (AnjutaFileLoaderPlugin* plugin,
 												"SupportedMimeTypes",
 												mime_type,
 												NULL);
-	if (!plugin_descs);
+	
+	/* Check for plugins supporting one supertype */
+	if (plugin_descs == NULL)
 	{
-		gchar* supertype = gnome_vfs_get_supertype_from_mime_type (mime_type);
-		plugin_descs = anjuta_plugin_manager_query (plugin_manager,
+		GList *node;
+		GList *loader_descs = NULL;	
+		
+		loader_descs = anjuta_plugin_manager_query (plugin_manager,
 												"Anjuta Plugin",
 												"Interfaces", "IAnjutaFile",
-												"File Loader",
-												"SupportedMimeTypes",
-												supertype,
 												NULL);
-		g_free (supertype);
+		for (node = g_list_first (loader_descs); node != NULL; node = g_list_next (node))
+		{
+			gchar *value;
+			
+			if (anjuta_plugin_description_get_string ((AnjutaPluginDescription *)node->data,
+													  "File Loader", "SupportedMimeTypes", &value))
+			{
+				gchar **split_value;
+				
+				split_value = g_strsplit (value, ",", -1);
+				g_free (value);
+				if (split_value)
+				{
+					gchar **mime;
+					
+					for (mime = split_value; *mime != NULL; mime++)
+					{
+						/* The following line is working on unix only where
+						 * content and mime type are the same. Normally the
+						 * mime type has to be converted to a content type.
+						 * But it is a recent (glib 2.18) function, I think we can
+						 * wait a bit to fix this */
+						if (g_content_type_is_a (mime_type, *mime))
+						{
+							gchar *loc;
+							anjuta_plugin_description_get_string ((AnjutaPluginDescription *)node->data,
+													  "Anjuta Plugin", "Location", &loc);
+							
+							plugin_descs = g_list_prepend (plugin_descs, node->data);
+							break;
+						}
+					}
+				}
+				g_strfreev (split_value);
+			}
+		}
+		g_list_free (loader_descs);
+		plugin_descs = g_list_reverse (plugin_descs);
 	}
 	
 	return plugin_descs;
@@ -155,7 +194,7 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 	GList *plugin_descs, *snode;
 	GList *mime_apps, *node;
 	GtkWidget *menu, *menuitem;
-	GnomeVFSMimeApplication *mime_app;
+	GAppInfo *mime_app;
 	
 	GtkWidget *dialog, *parent, *hbox, *label;
 	GtkWidget *options;
@@ -196,7 +235,7 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 	gtk_menu_append (menu, menuitem);
 	
 	/* Open with plugins menu items */
-	plugin_descs = get_available_plugins_for_mime (plugin, mime_type);
+	plugin_descs = get_available_plugins_for_mime (ANJUTA_PLUGIN (plugin), mime_type);
 	snode = plugin_descs;
 	while (snode)
 	{
@@ -227,7 +266,7 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 	}
 	
 	/* Open with application menu items */
-	mime_apps = gnome_vfs_mime_get_all_applications (mime_type);
+	mime_apps = g_app_info_get_all_for_type (mime_type);
 	if (mime_apps)
 	{
 		/* Separator */
@@ -237,9 +276,13 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 	node = mime_apps;
 	while (node)
 	{
-		mime_app = (GnomeVFSMimeApplication *)(node->data);
-		menuitem = gtk_menu_item_new_with_label (mime_app->name);
-		gtk_menu_append (menu, menuitem);
+		mime_app = (GAppInfo *)(node->data);
+		if (g_app_info_should_show (mime_app))
+		{
+			menuitem = gtk_menu_item_new_with_label (
+					g_app_info_get_name (mime_app));
+			gtk_menu_append (menu, menuitem);
+		}
 		node = g_list_next (node);
 	}
 	
@@ -289,7 +332,7 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 				{
 					GFile* file = g_file_new_for_uri (uri);
 					ianjuta_file_open (IANJUTA_FILE (loaded_plugin), file, NULL);
-					set_recent_file (plugin, uri, mime_type);
+					update_recent_file (plugin, uri, mime_type, TRUE);
 					g_object_unref (file);
 				}
 				else
@@ -304,77 +347,44 @@ open_with_dialog (AnjutaFileLoaderPlugin *plugin, const gchar *uri,
 		else
 		{
 			GList *uris = NULL;
-			GnomeVFSResult res;
+			GError *error = NULL;
 			
 			option -= (g_list_length (plugin_descs) + 2);
 			mime_app = g_list_nth_data (mime_apps, option);
 			uris = g_list_prepend (uris, (gpointer)uri);
-			res = gnome_vfs_mime_application_launch (mime_app, uris);
-			if (res != GNOME_VFS_OK)
-				launch_application_failure (plugin, uri, res);
-			else
-				set_recent_file (plugin, uri, mime_type);
+			g_app_info_launch_uris(mime_app, uris, NULL, &error);
+			if (error)
+			{
+				launch_application_failure (plugin, uri, error->message);
+				g_error_free (error);
+			}
+			update_recent_file (plugin, uri, mime_type, error == NULL);
 			g_list_free (uris);
 		}
 	}
-	gnome_vfs_mime_application_list_free (mime_apps);
+	g_list_foreach (mime_apps, (GFunc) g_object_unref, NULL);
+	g_list_free (mime_apps);
 	if (plugin_descs)
 		g_list_free (plugin_descs);
 	gtk_widget_destroy (dialog);
 }
 
-static gboolean
-launch_in_default_application (AnjutaFileLoaderPlugin *plugin,
-							   const gchar *mime_type, const gchar *uri)
-{
-	GnomeVFSMimeAction *action;
-	GList *uris = NULL;
-	gboolean ret;
-	
-	uris = g_list_prepend (uris, (gpointer)uri);
-	
-	ret = TRUE;
-	action = gnome_vfs_mime_get_default_action (mime_type);
-	if (!action || gnome_vfs_mime_action_launch (action, uris) != GNOME_VFS_OK)
-	{
-		GnomeVFSMimeApplication *app;
-		GnomeVFSResult res;
-		app = gnome_vfs_mime_get_default_application (mime_type);
-		if (!app ||
-			(res =
-				gnome_vfs_mime_application_launch (app, uris)) != GNOME_VFS_OK)
-		{
-			open_with_dialog (plugin, uri, mime_type);
-		}
-		if (app)
-			gnome_vfs_mime_application_free (app);
-	}
-	if (action)
-		gnome_vfs_mime_action_free (action);
-	g_list_free (uris);
-	return ret;
-}
-
 static void
 open_file (AnjutaFileLoaderPlugin *plugin, const gchar *uri)
 {
-	GnomeVFSURI *vfs_uri;	
 	gchar *dirname;
+	gchar *path;
 	GFile* file;
 	
-	vfs_uri = gnome_vfs_uri_new (uri);
-	dirname = gnome_vfs_uri_extract_dirname (vfs_uri);
-	gnome_vfs_uri_unref (vfs_uri);
+	file = g_file_new_for_uri (uri);
+	path = g_file_get_path (file);
+	dirname = g_path_get_dirname (uri);
 	chdir (dirname);
+	g_free (path);
 	g_free (dirname);
 	
-	file = g_file_new_for_uri (uri);
-	
-	/* FIXME: We have to manage the error to know if we have to remove the recent file
-	 */
 	ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin),
 							  file, FALSE, NULL);
-	
 	g_object_unref (file);
 }
 
@@ -382,14 +392,14 @@ typedef struct
 {
 	AnjutaFileLoaderPlugin *plugin;
 	const gchar *uri;
-} RecentIdelOpenData;
+} RecentIdleOpenData;
 
 static gboolean
 on_open_recent_file_idle (gpointer data)
 {
-	RecentIdelOpenData *rdata;
+	RecentIdleOpenData *rdata;
 	
-	rdata = (RecentIdelOpenData*)data;
+	rdata = (RecentIdleOpenData*)data;
 	open_file (rdata->plugin, rdata->uri);
 	g_free (rdata);
 	return FALSE;
@@ -399,17 +409,14 @@ static gboolean
 on_open_recent_file (GtkRecentChooser *chooser, AnjutaFileLoaderPlugin *plugin)
 {
 	const gchar *uri;
-	GnomeVFSURI *vfs_uri;
 	gboolean ret = TRUE;
-	RecentIdelOpenData *rdata;
+	RecentIdleOpenData *rdata;
 	
 	uri = gtk_recent_chooser_get_current_uri (chooser);
-	vfs_uri = gnome_vfs_uri_new (uri);
-	rdata = g_new0 (RecentIdelOpenData, 1);
+	rdata = g_new0 (RecentIdleOpenData, 1);
 	rdata->plugin = plugin;
 	rdata->uri = uri;
 	g_idle_add (on_open_recent_file_idle, rdata);
-	gnome_vfs_uri_unref (vfs_uri);
 
 	return ret;
 }
@@ -700,28 +707,25 @@ on_create_submenu (gpointer user_data)
 }
 
 static void
-open_file_with (AnjutaFileLoaderPlugin *plugin, GtkMenuItem *menuitem,
+open_uri_with (AnjutaFileLoaderPlugin *plugin, GtkMenuItem *menuitem,
 				const gchar *uri)
 {
-	GList *mime_apps;
-	GnomeVFSMimeApplication *mime_app;
-	gchar *mime_type;
-	gint idx;
+	GAppInfo *app;
 	AnjutaPluginDescription *desc;
-	AnjutaPluginManager *plugin_manager;
-	
-	idx = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menuitem), "index"));
-	desc = (AnjutaPluginDescription*) g_object_get_data (G_OBJECT (menuitem),
-														 "desc");
-	plugin_manager = anjuta_shell_get_plugin_manager (ANJUTA_PLUGIN (plugin)->shell,
-													  NULL);
-	mime_type = anjuta_util_get_uri_mime_type (uri);
-	mime_apps = gnome_vfs_mime_get_all_applications (mime_type);
+	const gchar *mime_type;
 	
 	/* Open with plugin */
+	desc = (AnjutaPluginDescription*) g_object_get_data (G_OBJECT (menuitem),
+														 "desc");
+	mime_type = (const gchar*) g_object_get_data (G_OBJECT (menuitem),
+														 "mime_type");
 	if (desc)
 	{
+		AnjutaPluginManager *plugin_manager;
 		gchar *location = NULL;
+
+		plugin_manager = anjuta_shell_get_plugin_manager (ANJUTA_PLUGIN (plugin)->shell,
+	 														  NULL);
 		
 		anjuta_plugin_description_get_string (desc, "Anjuta Plugin",
 											  "Location", &location);
@@ -736,9 +740,12 @@ open_file_with (AnjutaFileLoaderPlugin *plugin, GtkMenuItem *menuitem,
 			if (loaded_plugin)
 			{
 				GFile* file = g_file_new_for_uri (uri);
-				ianjuta_file_open (IANJUTA_FILE (loaded_plugin), file, NULL);
-				set_recent_file (plugin, uri, mime_type);
+				GError *error = NULL;
+				
+				ianjuta_file_open (IANJUTA_FILE (loaded_plugin), file, &error);
 				g_object_unref (file);
+				update_recent_file (plugin, uri, mime_type, error == NULL);
+				g_free (error);
 			}
 			else
 			{
@@ -751,20 +758,25 @@ open_file_with (AnjutaFileLoaderPlugin *plugin, GtkMenuItem *menuitem,
 	}
 	else
 	{
-		GList *uris = NULL;
-		GnomeVFSResult res;
+		/* Open with application */
+		app = (GAppInfo *) g_object_get_data (G_OBJECT (menuitem),
+															 "app");
+		if (app)
+		{
+			GList *uris = NULL;
+			GError *error = NULL;
 		
-		mime_app = g_list_nth_data (mime_apps, idx);
-		uris = g_list_prepend (uris, (gpointer)uri);
-		res = gnome_vfs_mime_application_launch (mime_app, uris);
-		if (res != GNOME_VFS_OK)
-			launch_application_failure (plugin, uri, res);
-		else
-			set_recent_file (plugin, uri, mime_type);
-		g_list_free (uris);
+			uris = g_list_prepend (uris, (gpointer)uri);
+			g_app_info_launch_uris (app, uris, NULL, &error);
+			g_list_free (uris);
+			if (error)
+			{
+				launch_application_failure (plugin, uri, error->message);
+				g_error_free (error);
+			}
+			update_recent_file (plugin, uri, mime_type, error == NULL);
+		}
 	}
-	gnome_vfs_mime_application_list_free (mime_apps);
-	g_free (mime_type);
 }
 
 static void
@@ -778,7 +790,7 @@ static void
 fm_open_with (GtkMenuItem *menuitem, AnjutaFileLoaderPlugin *plugin)
 {
 	if (plugin->fm_current_uri)
-		open_file_with (plugin, menuitem, plugin->fm_current_uri);
+		open_uri_with (plugin, menuitem, plugin->fm_current_uri);
 }
 
 static void
@@ -792,7 +804,7 @@ static void
 pm_open_with (GtkMenuItem *menuitem, AnjutaFileLoaderPlugin *plugin)
 {
 	if (plugin->pm_current_uri)
-		open_file_with (plugin, menuitem, plugin->pm_current_uri);
+		open_uri_with (plugin, menuitem, plugin->pm_current_uri);
 }
 
 static GtkActionEntry actions_file[] = {
@@ -848,13 +860,12 @@ create_open_with_submenu (AnjutaFileLoaderPlugin *plugin, GtkWidget *parentmenu,
 						  const gchar *uri, GCallback callback,
 						  gpointer callback_data)
 {
-	GList *mime_apps, *node;
-	GnomeVFSMimeApplication *mime_app;
-	GList *plugin_descs, *snode;
+	GList *mime_apps;
+	GList *plugin_descs;
+	GList *node;	
 	GtkWidget *menu, *menuitem;
 	gchar *mime_type;
-	gint idx;
-	gboolean ret;
+	GFile *file;
 	
 	g_return_val_if_fail (GTK_IS_MENU_ITEM (parentmenu), FALSE);
 	
@@ -862,21 +873,20 @@ create_open_with_submenu (AnjutaFileLoaderPlugin *plugin, GtkWidget *parentmenu,
 	gtk_widget_show (menu);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (parentmenu), menu);
 	
-	mime_type = anjuta_util_get_uri_mime_type (uri);
+	file = g_file_new_for_uri (uri);
+	mime_type = anjuta_util_get_file_mime_type (file);
+	g_object_unref (file);
 	if (mime_type == NULL)
 		return FALSE;
 	
-	idx = 0;
-	
 	/* Open with plugins menu items */
-	plugin_descs = get_available_plugins_for_mime (plugin, mime_type);
-	snode = plugin_descs;
-	while (snode)
+	plugin_descs = get_available_plugins_for_mime (ANJUTA_PLUGIN (plugin), mime_type);
+	for (node = plugin_descs; node != NULL; node = g_list_next (node))
 	{
 		gchar *name;
 		AnjutaPluginDescription *desc;
 		
-		desc = (AnjutaPluginDescription *)(snode->data);
+		desc = (AnjutaPluginDescription *)(node->data);
 		name = NULL;
 		anjuta_plugin_description_get_locale_string (desc, "File Loader",
 													 "Title", &name);
@@ -891,50 +901,58 @@ create_open_with_submenu (AnjutaFileLoaderPlugin *plugin, GtkWidget *parentmenu,
 												  "Location", &name);
 		}
 		menuitem = gtk_menu_item_new_with_label (name);
-		g_object_set_data (G_OBJECT (menuitem), "index", GINT_TO_POINTER (idx));
 		g_object_set_data (G_OBJECT (menuitem), "desc", (gpointer)(desc));
+		g_object_set_data (G_OBJECT (menuitem), "mime_type", mime_type);			
 		g_signal_connect (G_OBJECT (menuitem), "activate",
 						  G_CALLBACK (callback), callback_data);
 		gtk_menu_append (menu, menuitem);
 		g_free (name);
-		snode = g_list_next (snode);
-		idx++;
 	}
+	g_list_free (plugin_descs);
 	
 	/* Open with applications */
-	mime_apps = gnome_vfs_mime_get_all_applications (mime_type);
-	if (idx > 0 && mime_apps)
+	mime_apps = g_app_info_get_all_for_type (mime_type);
+	if (plugin_descs && mime_apps)
 	{
 		menuitem = gtk_menu_item_new ();
 		gtk_menu_append (menu, menuitem);
-		idx++;
 	}
-	g_free (mime_type);
-	idx = 0;
-	node = mime_apps;
-	while (node)
+	
+	for (node = mime_apps; node != NULL; node = g_list_next (node))
 	{
-		mime_app = (GnomeVFSMimeApplication *)(node->data);
-		menuitem = gtk_menu_item_new_with_label (mime_app->name);
-		g_object_set_data (G_OBJECT (menuitem), "index", GINT_TO_POINTER (idx));
-		g_signal_connect (G_OBJECT (menuitem), "activate",
-						  G_CALLBACK (callback), callback_data);
-		gtk_menu_append (menu, menuitem);
-		node = g_list_next (node);
-		idx++;
+		GAppInfo *mime_app;
+		
+		mime_app = (GAppInfo *)(node->data);
+		if (g_app_info_should_show (mime_app))
+		{
+			menuitem = gtk_menu_item_new_with_label ( g_app_info_get_name (mime_app));
+			g_object_set_data_full (G_OBJECT (menuitem), "app", mime_app, g_object_unref);			
+			g_object_set_data (G_OBJECT (menuitem), "mime_type", mime_type);			
+			g_signal_connect (G_OBJECT (menuitem), "activate",
+							  G_CALLBACK (callback), callback_data);
+			gtk_menu_append (menu, menuitem);
+		}
+		else
+		{
+			g_object_unref (mime_app);
+		}
 	}
+	g_list_free (mime_apps);
+
 	gtk_widget_show_all (menu);
-	
-	if (mime_apps == NULL && plugin_descs == NULL)
-		ret = FALSE;
+
+	if ((mime_apps != NULL) || (plugin_descs != NULL))
+	{
+		g_object_set_data_full (G_OBJECT (menu), "mime_type", (gpointer)mime_type, g_free);
+		
+		return TRUE;
+	}
 	else
-		ret = TRUE;
-	
-	gnome_vfs_mime_application_list_free (mime_apps);
-	if (plugin_descs)
-		g_list_free (plugin_descs);
-	
-	return ret;
+	{
+		g_free (mime_type);
+		
+		return FALSE;
+	}
 }
 						  
 static void
@@ -1069,11 +1087,7 @@ on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
 				 AnjutaSession *session,
 				 AnjutaFileLoaderPlugin *plugin)
 {
-	AnjutaStatus *status;
-	const gchar *uri;
-	gchar *mime_type;
 	GList *files, *node;
-	gint i;
 				
 	/* We want to load the files first before other session loads */
 	if (phase != ANJUTA_SESSION_PHASE_FIRST)
@@ -1083,57 +1097,37 @@ on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
 	if (!files)
 		return;
 		
-	/* Open project files first and then regular files */
-	for (i = 0; i < 2; i++)
+	/* Open all files except project files */
+	for (node = g_list_first (files); node != NULL; node = g_list_next (node))
 	{
-		node = files;
-		while (node)
+		gchar *uri = node->data;
+		
+		if (uri)
 		{
-			uri = node->data;
-			if (uri)
+			gchar *fragment;
+			
+			fragment = strchr (uri, '#');
+			if (fragment)
+				*fragment = '\0';
+
+			if (!anjuta_util_is_project_file (uri))
 			{
-				gchar *label, *filename;
-				
-				mime_type = anjuta_util_get_uri_mime_type (uri);
-				
-				filename = g_path_get_basename (uri);
-				if (strchr (filename, '#'))
-					*(strchr (filename, '#')) = '\0';
-				
-				label = g_strconcat (_("Loaded:"), " ", filename, NULL);
-				
-				if (i == 0 && mime_type &&
-					strcmp (mime_type, "application/x-anjuta") == 0)
-				{
-					/* Project files first */
-					/* FIXME: Ignore project files for now */
-					/*
-					ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin),
-											  uri, FALSE, NULL);
-					*/
-				}
-				else if (i != 0 &&
-						 (!mime_type ||
-						  strcmp (mime_type, "application/x-anjuta") != 0))
-				{
-					/* Then rest of the files */
-					GFile* file = g_file_new_for_uri (uri);
-					ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin),
+				GFile* file = g_file_new_for_uri (uri);
+				GObject *loader = ianjuta_file_loader_load (IANJUTA_FILE_LOADER (plugin),
 											  file, FALSE, NULL);
-					g_object_unref (file);
+				if (fragment)
+				{
+					if (IANJUTA_IS_DOCUMENT_MANAGER (loader))
+					{
+						ianjuta_document_manager_goto_file_line (IANJUTA_DOCUMENT_MANAGER (loader), file, atoi(fragment + 1), NULL);
+					}
 				}
-				g_free (filename);
-				g_free (label);
-				g_free (mime_type);
+				g_object_unref (file);
 			}
-			node = g_list_next (node);
 		}
+		g_free (uri);
 	}
-	if (files)
-	{
-		g_list_foreach (files, (GFunc)g_free, NULL);
-		g_list_free (files);
-	}
+	g_list_free (files);
 }
 
 static void
@@ -1168,7 +1162,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	
 	loader_plugin = ANJUTA_PLUGIN_FILE_LOADER (plugin);
 	
-	DEBUG_PRINT ("AnjutaFileLoaderPlugin: Activating File Loader plugin...");
+	DEBUG_PRINT ("%s", "AnjutaFileLoaderPlugin: Activating File Loader plugin...");
 	
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	
@@ -1237,7 +1231,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	setup_recent_chooser_menu (GTK_RECENT_CHOOSER (toolbar_menu), loader_plugin);
 	gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (widget),
 								   toolbar_menu);
-	
+
 	/* Install drag n drop handler */
 	dnd_drop_init (GTK_WIDGET (plugin->shell), dnd_dropped, plugin,
 				   "text/plain", "text/html", "text/source", "application-x/anjuta",
@@ -1267,7 +1261,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	
 	loader_plugin = ANJUTA_PLUGIN_FILE_LOADER (plugin);
 	
-	DEBUG_PRINT ("AnjutaFileLoaderPlugin: Deactivating File Loader plugin...");
+	DEBUG_PRINT ("%s", "AnjutaFileLoaderPlugin: Deactivating File Loader plugin...");
 	
 	/* Disconnect session */
 	g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->shell),
@@ -1328,35 +1322,26 @@ iloader_load (IAnjutaFileLoader *loader, GFile* file,
 			  gboolean read_only, GError **err)
 {
 	gchar *mime_type;
-	gchar *new_uri;
-	GnomeVFSURI* vfs_uri;
 	AnjutaStatus *status;
 	AnjutaPluginManager *plugin_manager;
 	GList *plugin_descs = NULL;
 	GObject *plugin = NULL;	
-	gchar* uri = g_file_get_uri (file);
+	gchar *uri = g_file_get_uri (file);
 	
 	g_return_val_if_fail (uri != NULL, NULL);
-	vfs_uri = gnome_vfs_uri_new (uri);
-	
-	if (!gnome_vfs_uri_exists (vfs_uri))
-	{
-		launch_application_failure (ANJUTA_PLUGIN_FILE_LOADER (loader),
-									uri, GNOME_VFS_ERROR_NOT_FOUND);
-		gnome_vfs_uri_unref (vfs_uri);
-		return NULL;
-	}
-	
-	new_uri = gnome_vfs_uri_to_string (vfs_uri,
-									   GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER);
-	mime_type = anjuta_util_get_uri_mime_type (new_uri);
-	gnome_vfs_uri_unref (vfs_uri);
+
+	mime_type = anjuta_util_get_file_mime_type (file);
 	
 	if (mime_type == NULL)
 	{
-		launch_application_failure (ANJUTA_PLUGIN_FILE_LOADER (loader),
-									new_uri, GNOME_VFS_ERROR_NOT_FOUND);
-		g_free (new_uri);
+		update_recent_file (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, NULL, FALSE);	
+		
+		if (err == NULL)
+			launch_application_failure (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, _("File not found"));
+		
+		g_set_error (err, G_IO_ERROR, G_IO_ERROR_NOT_FOUND, _("File not found"));
+		
+		g_free (uri);
 		return NULL;
 	}
 	
@@ -1367,22 +1352,16 @@ iloader_load (IAnjutaFileLoader *loader, GFile* file,
 	
 	DEBUG_PRINT ("Opening URI: %s", uri);
 	
-	plugin_descs = anjuta_plugin_manager_query (plugin_manager,
-												"Anjuta Plugin",
-												"Interfaces", "IAnjutaFile",
-												"File Loader",
-												"SupportedMimeTypes",
-												mime_type,
-												NULL);
+	plugin_descs = get_available_plugins_for_mime (ANJUTA_PLUGIN (loader), mime_type);
 	
 	if (g_list_length (plugin_descs) > 1)
 	{
 		gchar* basename = g_path_get_basename (uri);
-		gchar* message = g_strdup_printf ("Please select a plugin to open <b>%s</b>.", 
+		gchar* message = g_strdup_printf (_("Please select a plugin to open <b>%s</b>."), 
 										 basename);
 		plugin =
 			anjuta_plugin_manager_select_and_activate (plugin_manager,
-													   "Open With",
+													   _("Open With"),
 													   message,
 													   plugin_descs);
 		g_free (basename);
@@ -1403,19 +1382,47 @@ iloader_load (IAnjutaFileLoader *loader, GFile* file,
 	}
 	else
 	{
-		launch_in_default_application (ANJUTA_PLUGIN_FILE_LOADER (loader), mime_type, uri);
-	}
-	if (plugin)
-	{
-		ianjuta_file_open (IANJUTA_FILE(plugin), file, NULL);
+		GAppInfo *appinfo;
+		GList *uris = NULL;
+
+		uris = g_list_prepend (uris, (gpointer)uri);
+
+		appinfo = g_app_info_get_default_for_type (mime_type, TRUE);
+		if (appinfo)
+		{
+			GError *error = NULL;
+			if (!g_app_info_launch_uris (appinfo, uris, NULL, &error))
+			{
+				open_with_dialog (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, mime_type);
+			}
+			else
+			{
+				update_recent_file (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, mime_type, error == NULL);
+			}				
+			g_object_unref (G_OBJECT (appinfo));
+		}
+		g_list_free (uris);
 	}
 	
-	set_recent_file (ANJUTA_PLUGIN_FILE_LOADER (loader), new_uri, mime_type);
+	if (plugin)
+	{
+		GError *error = NULL;
+		
+		ianjuta_file_open (IANJUTA_FILE(plugin), file, &error);
+		if (error != NULL)
+		{
+			
+			if (err == NULL)
+				launch_application_failure (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, error->message);
+			g_propagate_error (err, error);
+		}
+		update_recent_file (ANJUTA_PLUGIN_FILE_LOADER (loader), uri, mime_type, error == NULL);
+	}
 	
 	if (plugin_descs)
 		g_list_free (plugin_descs);
+	
 	g_free (mime_type);
-	g_free (new_uri);
 	g_free (uri);
 	anjuta_status_busy_pop (status);
 		
