@@ -131,6 +131,7 @@ on_file_view_current_file_changed (AnjutaFileView* view, GFile* file,
 		g_value_set_object (value, file);
 		anjuta_shell_add_value (ANJUTA_PLUGIN (file_manager)->shell,
 								IANJUTA_FILE_MANAGER_SELECTED_FILE, value, NULL);
+		g_value_unset (value);
 	}
 	else
 	{
@@ -204,7 +205,7 @@ file_manager_activate (AnjutaPlugin *plugin)
 	AnjutaFileManager *file_manager;
 	gint notify_id;
 	
-	DEBUG_PRINT ("AnjutaFileManager: Activating AnjutaFileManager plugin ...");
+	DEBUG_PRINT ("%s", "AnjutaFileManager: Activating AnjutaFileManager plugin ...");
 	file_manager = (AnjutaFileManager*) plugin;
 
 	file_manager->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
@@ -272,7 +273,7 @@ file_manager_deactivate (AnjutaPlugin *plugin)
 	AnjutaUI *ui;
 	AnjutaFileManager *file_manager;
 
-	DEBUG_PRINT ("AnjutaFileManager: Dectivating AnjutaFileManager plugin ...");
+	DEBUG_PRINT ("%s", "AnjutaFileManager: Dectivating AnjutaFileManager plugin ...");
 	
 	file_manager = (AnjutaFileManager*) plugin;
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
