@@ -59,6 +59,7 @@ struct _CppJavaAssistPriv {
 	gchar *search_cache;
 	gchar *scope_context_cache;
 	GCompletion *completion_cache;
+	gboolean editor_only;
 };
 
 static gchar*
@@ -135,10 +136,21 @@ is_word_character (gchar ch)
 	return FALSE;
 }	
 
+/**
+ * If mergeable is NULL than no merge will be made with iter elements, elsewhere
+ * mergeable will be returned with iter elements.
+ */
 static GCompletion*
-create_completion (IAnjutaEditorAssist* iassist, IAnjutaIterable* iter)
-{
-	GCompletion *completion = g_completion_new (completion_function);
+create_completion (IAnjutaEditorAssist* iassist, IAnjutaIterable* iter,
+				   GCompletion* mergeable)
+{	
+	GCompletion *completion;
+	
+	if (mergeable == NULL)
+		completion = g_completion_new (completion_function);
+	else
+		completion = mergeable;
+	
 	GList* suggestions = NULL;
 	do
 	{
@@ -162,11 +174,15 @@ create_completion (IAnjutaEditorAssist* iassist, IAnjutaIterable* iter)
 	}
 	while (ianjuta_iterable_next (iter, NULL));
 	
+	
+	
 	suggestions = g_list_sort (suggestions, completion_compare);
 	g_completion_add_items (completion, suggestions);
 	return completion;
 }
 
+#define SCOPE_BRACE_JUMP_LIMIT 50
+
 static gchar*
 cpp_java_assist_get_scope_context (IAnjutaEditor* editor,
 								   const gchar *scope_operator,
@@ -182,14 +198,27 @@ cpp_java_assist_get_scope_context (IAnjutaEditor* editor,
 	
 	ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (iter), 0, NULL);
 	
-	while (ch && is_scope_context_character (ch))
+	while (ch)
 	{
-		scope_chars_found = TRUE;
+		if (is_scope_context_character (ch))
+		{
+			scope_chars_found = TRUE;
+		}
+		else if (ch == ')')
+		{
+			if (!cpp_java_util_jump_to_matching_brace (iter, ch, SCOPE_BRACE_JUMP_LIMIT))
+			{
+				out_of_range = TRUE;
+				break;
+			}
+		}
+		else
+			break;
 		if (!ianjuta_iterable_previous (iter, NULL))
 		{
 			out_of_range = TRUE;
 			break;
-		}
+		}		
 		ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (iter), 0, NULL);
 	}
 	if (scope_chars_found)
@@ -287,7 +316,7 @@ cpp_java_assist_destroy_completion_cache (CppJavaAssist *assist)
 	{
 		g_free (assist->priv->scope_context_cache);
 		assist->priv->scope_context_cache = NULL;
-	}
+	} 
 	if (assist->priv->completion_cache)
 	{
 		GList* items = assist->priv->completion_cache->items;
@@ -306,29 +335,90 @@ cpp_java_assist_create_scope_completion_cache (CppJavaAssist *assist,
 											   const gchar *scope_operator,
 											   const gchar *scope_context)
 {
-	DEBUG_PRINT ("TODO: cpp_java_assist_create_scope_completion_cache ()");
-#if 0	
+	DEBUG_PRINT ("scope context: %s", scope_context);
 	cpp_java_assist_destroy_completion_cache (assist);
 	if (g_str_equal (scope_operator, "::"))
 	{
-		IAnjutaIterable* iter = 
-			ianjuta_symbol_manager_get_members (assist->priv->isymbol_manager,
-												scope_context, TRUE, NULL);
-		if (iter)
+		/* Go through the possible namespace (Gnome::Glade, for example) */
+		GStrv contexts = g_strsplit (scope_context, "::", -1);
+		gint cur_context = 0;
+		if (contexts[0] != NULL)
 		{
-			assist->priv->completion_cache =
-				create_completion (assist->priv->iassist, iter);
-			assist->priv->scope_context_cache = g_strdup (scope_context);
-			g_object_unref (iter);
+			DEBUG_PRINT ("Scope[%d] = %s", cur_context, contexts[0]);
+			IAnjutaIterable* symbol = 
+				ianjuta_symbol_manager_search (assist->priv->isymbol_manager,
+											   IANJUTA_SYMBOL_TYPE_CLASS | IANJUTA_SYMBOL_TYPE_NAMESPACE,
+											   TRUE,
+											   IANJUTA_SYMBOL_FIELD_SIMPLE,
+											   contexts[0],
+											   FALSE,
+											   TRUE,
+											   TRUE,
+											   1,
+											   -1,
+											   NULL);
+			if (symbol && ianjuta_iterable_get_length(symbol, NULL))
+			{
+				while (contexts[++cur_context] != NULL)
+				{							
+					DEBUG_PRINT ("Scope[%d] = %s", cur_context, contexts[0]);
+					IAnjutaIterable* members = 
+						ianjuta_symbol_manager_get_members(assist->priv->isymbol_manager,
+														   IANJUTA_SYMBOL(symbol),
+														   IANJUTA_SYMBOL_FIELD_SIMPLE,
+														   TRUE, NULL);
+					if (members && ianjuta_iterable_get_length (members, NULL))
+					{
+						gboolean found = FALSE;
+						do
+						{
+							if (g_str_equal (ianjuta_symbol_get_name (IANJUTA_SYMBOL(members),
+																	  NULL),
+											 contexts[cur_context]))
+							{
+								g_object_unref (symbol);
+								symbol = members;
+								found = TRUE;
+								break;
+							}
+						}
+						while (ianjuta_iterable_next (members, NULL));
+						if (found)
+							continue;
+						else
+						{
+							g_object_unref (symbol);
+							symbol = NULL;
+							break;
+						}
+					}
+				}
+				if (symbol)
+				{
+					IAnjutaIterable* members = 
+						ianjuta_symbol_manager_get_members(assist->priv->isymbol_manager,
+														   IANJUTA_SYMBOL(symbol),
+														   IANJUTA_SYMBOL_FIELD_SIMPLE,
+														   TRUE, NULL);
+					if (members)
+					{
+						assist->priv->completion_cache =
+							create_completion (assist->priv->iassist, members, NULL);
+						assist->priv->scope_context_cache = g_strdup (scope_context);
+						g_object_unref (members);
+					}
+					g_object_unref (symbol);
+				}
+			}
 		}
-	} 
+		g_strfreev(contexts);
+	}
 	else if (g_str_equal (scope_operator, ".") ||
-			 g_str_equal (scope_operator, "->"))
+			 g_str_equal (scope_operator, "->"))	
 	{
 		/* TODO: Find the type of context by parsing the file somehow and
 		search for the member as it is done with the :: context */
-	}
-#endif	
+	}	
 }
 
 static void
@@ -336,25 +426,74 @@ cpp_java_assist_create_word_completion_cache (CppJavaAssist *assist,
 											  const gchar *pre_word)
 {
 	gint max_completions;
+	GCompletion *completion = NULL;
+	GList* editor_completions = NULL;
+	assist->priv->editor_only = FALSE;
 	max_completions =
 		anjuta_preferences_get_int_with_default (assist->priv->preferences,
 												 PREF_AUTOCOMPLETE_CHOICES,
 												 MAX_COMPLETIONS);
 	
 	cpp_java_assist_destroy_completion_cache (assist);
-	IAnjutaIterable* iter = 
+	
+	IAnjutaIterable* iter_project = 
+		ianjuta_symbol_manager_search (assist->priv->isymbol_manager,
+									    IANJUTA_SYMBOL_TYPE_MAX,
+									    TRUE,
+										IANJUTA_SYMBOL_FIELD_SIMPLE|IANJUTA_SYMBOL_FIELD_TYPE,
+										pre_word, TRUE, TRUE, FALSE, -1, -1, NULL);
+	
+	IAnjutaIterable* iter_globals = 
 		ianjuta_symbol_manager_search (assist->priv->isymbol_manager,
 										IANJUTA_SYMBOL_TYPE_MAX,
 									    TRUE,
 										IANJUTA_SYMBOL_FIELD_SIMPLE|IANJUTA_SYMBOL_FIELD_TYPE,
-										pre_word, TRUE, TRUE, max_completions, -1, NULL);
-	if (iter)
+										pre_word, TRUE, TRUE, TRUE, -1, -1, NULL);
+
+	if (iter_project) 
 	{
-		assist->priv->completion_cache =
-			create_completion (assist->priv->iassist, iter);
-		assist->priv->search_cache = g_strdup (pre_word);
-		g_object_unref (iter);
+		completion = create_completion (assist->priv->iassist, iter_project, NULL);
+		g_object_unref (iter_project);
 	}
+	
+	if (iter_globals)
+	{
+		
+		completion = create_completion (assist->priv->iassist, iter_globals, completion);
+		g_object_unref (iter_globals);
+	}
+	
+		editor_completions = ianjuta_editor_assist_get_suggestions (assist->priv->iassist,
+																	pre_word,
+																	NULL);
+	if (editor_completions)
+	{
+		GList* tag_list = NULL;
+		GList* node;
+		for (node = editor_completions; node != NULL; node = g_list_next (node))
+		{
+			CppJavaAssistTag *tag = g_new0 (CppJavaAssistTag, 1);
+			tag->name = node->data;
+			tag->type = 0;
+			tag->is_func = FALSE;
+			if (completion && !g_list_find_custom (completion->items, tag, 
+												   completion_compare))
+				tag_list = g_list_append (tag_list, tag);
+			else
+				cpp_java_assist_tag_destroy (tag);
+		}
+		if (!completion)
+		{
+			completion = g_completion_new(completion_function);
+			assist->priv->editor_only = TRUE;
+		}
+		tag_list = g_list_sort (tag_list, completion_compare);
+		g_completion_add_items (completion, tag_list);		
+		g_list_free (editor_completions);
+	}
+	
+	assist->priv->completion_cache = completion;
+	assist->priv->search_cache = g_strdup (pre_word);
 }
 
 static gboolean
@@ -390,8 +529,7 @@ cpp_java_assist_show_autocomplete (CppJavaAssist *assist,
 	 */
 	else if (!pre_word)
 		completion_list = assist->priv->completion_cache->items;
-	
-	/* If there is no cache and no pre_word, it means something else (?) */
+		
 	else
 		return FALSE;
 	
@@ -429,6 +567,7 @@ cpp_java_assist_show_autocomplete (CppJavaAssist *assist,
 										   NULL);
 			g_list_foreach (suggestions, (GFunc) g_free, NULL);
 			g_list_free (suggestions);
+			g_object_unref (position);
 			return TRUE;
 		}
 	}
@@ -495,10 +634,9 @@ cpp_java_assist_show_calltip (CppJavaAssist *assist, gchar *call_context,
 									   IANJUTA_SYMBOL_TYPE_FUNCTION|
 									   IANJUTA_SYMBOL_TYPE_METHOD|
 									   IANJUTA_SYMBOL_TYPE_MACRO_WITH_ARG,
-									   TRUE,
-									   IANJUTA_SYMBOL_FIELD_SIMPLE,
-									   call_context, FALSE, TRUE, max_completions,
-									   -1, NULL);
+									   TRUE, IANJUTA_SYMBOL_FIELD_SIMPLE,
+									   call_context, FALSE, TRUE, TRUE,
+									   max_completions, -1, NULL);
 	if (iter)
 	{
 		do
@@ -566,6 +704,7 @@ cpp_java_assist_check (CppJavaAssist *assist, gboolean autocomplete,
 	IAnjutaIterable *iter, *iter_save;
 	IAnjutaEditorAttribute attribute;
 	gchar *pre_word = NULL, *scope_operator = NULL;
+	GTimer* timer = g_timer_new ();
 	
 	//DEBUG_PRINT ("Autocomplete enable is: %d", autocomplete);
 	//DEBUG_PRINT ("Calltips enable is: %d", calltips);
@@ -578,7 +717,7 @@ cpp_java_assist_check (CppJavaAssist *assist, gboolean autocomplete,
 	iter = ianjuta_editor_get_position (editor, NULL);
 	ianjuta_iterable_previous (iter, NULL);
 	iter_save = ianjuta_iterable_clone (iter, NULL);
-	
+		
 	attribute = ianjuta_editor_cell_get_attribute (IANJUTA_EDITOR_CELL (iter),
 												   NULL);
 	/*
@@ -591,18 +730,18 @@ cpp_java_assist_check (CppJavaAssist *assist, gboolean autocomplete,
 	}
 	*/
 	
-	//DEBUG_PRINT ("assist init: %f", g_timer_elapsed (timer, NULL));
+	DEBUG_PRINT ("assist init: %f", g_timer_elapsed (timer, NULL));
 	
 	if (autocomplete)
 	{
 		pre_word = cpp_java_assist_get_pre_word (editor, iter);
-		//DEBUG_PRINT ("assist pre word: %f", g_timer_elapsed (timer, NULL));
+		DEBUG_PRINT ("assist pre word: %f", g_timer_elapsed (timer, NULL));
 		
 		scope_operator = cpp_java_assist_get_scope_operator (editor, iter);
-		//DEBUG_PRINT ("assist scope operator: %f", g_timer_elapsed (timer, NULL));
+		DEBUG_PRINT ("assist scope operator: %f", g_timer_elapsed (timer, NULL));
 		
-		//DEBUG_PRINT ("Pre word: %s", pre_word);
-		//DEBUG_PRINT ("Scope op: %s", scope_operator);
+		DEBUG_PRINT ("Pre word: %s", pre_word);
+		DEBUG_PRINT ("Scope op: %s", scope_operator);
 		
 		if (scope_operator)
 		{
@@ -630,8 +769,8 @@ cpp_java_assist_check (CppJavaAssist *assist, gboolean autocomplete,
 		else if (pre_word && strlen (pre_word) > 3)
 		{
 			if (!assist->priv->search_cache ||
-				strncmp (assist->priv->search_cache,
-						 pre_word, strlen (assist->priv->search_cache)) != 0)
+				!g_str_has_prefix (pre_word,
+						 assist->priv->search_cache) || assist->priv->editor_only)
 			{
 				cpp_java_assist_create_word_completion_cache (assist, pre_word);
 			}
@@ -642,7 +781,7 @@ cpp_java_assist_check (CppJavaAssist *assist, gboolean autocomplete,
 			ianjuta_editor_assist_hide_suggestions (assist->priv->iassist,
 													NULL);
 		}
-		//DEBUG_PRINT ("assist autocomplete: %f", g_timer_elapsed (timer, NULL));
+		DEBUG_PRINT ("assist autocomplete: %f", g_timer_elapsed (timer, NULL));
 	}
 	if (calltips)
 	{
@@ -651,7 +790,7 @@ cpp_java_assist_check (CppJavaAssist *assist, gboolean autocomplete,
 			gint offset;
 			gchar *call_context =
 				cpp_java_assist_get_calltip_context (assist, iter, &offset);
-			//DEBUG_PRINT ("get calltip context: %f", g_timer_elapsed (timer, NULL));
+			DEBUG_PRINT ("get calltip context: %f", g_timer_elapsed (timer, NULL));
 			if (call_context)
 			{
 				shown = cpp_java_assist_show_calltip (assist, call_context,
@@ -663,12 +802,15 @@ cpp_java_assist_check (CppJavaAssist *assist, gboolean autocomplete,
 			}
 			g_free (call_context);
 		}
-		//DEBUG_PRINT ("assist calltip: %f", g_timer_elapsed (timer, NULL));
+		DEBUG_PRINT ("assist calltip: %f", g_timer_elapsed (timer, NULL));
 	}
+	
 	g_object_unref (iter);
 	g_object_unref (iter_save);
 	g_free (pre_word);
 	g_free (scope_operator);
+	g_timer_destroy (timer);
+	
 	return shown;
 }
 
