@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef PLUGIN_H
@@ -26,7 +26,7 @@
 
 #define GLADE_FILE PACKAGE_DATA_DIR"/glade/anjuta-macro.glade"
 
-extern GType macro_plugin_get_type (AnjutaGluePlugin *plugin);
+extern GType macro_plugin_get_type (GTypeModule *module);
 #define ANJUTA_TYPE_PLUGIN_MACRO         (macro_plugin_get_type (NULL))
 #define ANJUTA_PLUGIN_MACRO(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), ANJUTA_TYPE_PLUGIN_MACRO, MacroPlugin))
 #define ANJUTA_PLUGIN_MACRO_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), ANJUTA_TYPE_PLUGIN_MACRO, MacroPluginClass))
