@@ -15,8 +15,6 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#include <libgnomevfs/gnome-vfs.h>
-
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-message-view.h>
@@ -41,8 +39,8 @@ struct _MessageViewPrivate
 	gint adj_chgd_hdlr;
 	
 	/* Messages filter buttons */
-	GtkWidget *info, *warn, *error;
-	guint info_count, warn_count, error_count;
+	GtkWidget *normal, *info, *warn, *error;
+	guint normal_count, info_count, warn_count, error_count;
 
 	/* Properties */
 	gchar *label;
@@ -125,7 +123,7 @@ static Message*
 message_new (IAnjutaMessageViewType type, const gchar *summary,
 			 const gchar *details)
 {
-	/* DEBUG_PRINT ("Creating message"); */
+	/* DEBUG_PRINT ("%s", "Creating message"); */
 	Message *message = g_new0 (Message, 1);
 	message->type = type;
 	if (summary)
@@ -144,7 +142,7 @@ message_copy (const Message *src)
 static void
 message_free (Message *message)
 {
-	/* DEBUG_PRINT ("Freeing message"); */
+	/* DEBUG_PRINT ("%s", "Freeing message"); */
 	g_free (message->summary);
 	g_free (message->details);
 	g_free (message);
@@ -714,12 +712,14 @@ message_view_instance_init (MessageView * self)
 	GtkTreeSelection *select;
 	GtkListStore *model;	
 	GtkAdjustment* adj;
-
+	gint icon_width = 30; // FIXME: Obtain from theme
+ 
 	g_return_if_fail(self != NULL);
 	self->privat = g_new0 (MessageViewPrivate, 1);
 
 	/* Init private data */
 	self->privat->line_buffer = g_strdup("");
+	self->privat->normal_count = 0;
 	self->privat->info_count = 0;
 	self->privat->warn_count = 0;
 	self->privat->error_count = 0;
@@ -728,7 +728,15 @@ message_view_instance_init (MessageView * self)
 	vbox = gtk_hbox_new (FALSE, 0);
 	filter_buttons_box = gtk_vbox_new (FALSE, 1);
 	
-	self->privat->info = gtk_toggle_button_new_with_label (_("0 Messages"));
+	self->privat->normal = gtk_toggle_button_new_with_label (_("No Messages"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->privat->normal), TRUE);
+	gtk_button_set_focus_on_click (GTK_BUTTON (self->privat->normal), FALSE);
+	gtk_button_set_relief (GTK_BUTTON (self->privat->normal), GTK_RELIEF_HALF);
+	gtk_widget_show (self->privat->normal);
+	g_signal_connect (G_OBJECT (self->privat->normal), "toggled",
+					  G_CALLBACK (on_filter_buttons_toggled), self);
+	
+	self->privat->info = gtk_toggle_button_new_with_label (_("No Infos"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->privat->info), TRUE);
 	gtk_button_set_focus_on_click (GTK_BUTTON (self->privat->info), FALSE);
 	gtk_button_set_relief (GTK_BUTTON (self->privat->info), GTK_RELIEF_HALF);
@@ -739,7 +747,7 @@ message_view_instance_init (MessageView * self)
 	g_signal_connect (G_OBJECT (self->privat->info), "toggled",
 					  G_CALLBACK (on_filter_buttons_toggled), self);
 	
-	self->privat->warn = gtk_toggle_button_new_with_label (_("0 Warnings"));
+	self->privat->warn = gtk_toggle_button_new_with_label (_("No Warnings"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->privat->warn), TRUE);
 	gtk_button_set_focus_on_click (GTK_BUTTON (self->privat->warn), FALSE);
 	gtk_button_set_relief (GTK_BUTTON (self->privat->warn), GTK_RELIEF_HALF);
@@ -751,7 +759,7 @@ message_view_instance_init (MessageView * self)
 	g_signal_connect (G_OBJECT (self->privat->warn), "toggled",
 					  G_CALLBACK (on_filter_buttons_toggled), self);
 	
-	self->privat->error = gtk_toggle_button_new_with_label (_("0 Errors"));
+	self->privat->error = gtk_toggle_button_new_with_label (_("No Errors"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->privat->error), TRUE);
 	gtk_button_set_focus_on_click (GTK_BUTTON (self->privat->error), FALSE);
 	gtk_button_set_relief (GTK_BUTTON (self->privat->error), GTK_RELIEF_HALF);
@@ -762,6 +770,8 @@ message_view_instance_init (MessageView * self)
 	g_signal_connect (G_OBJECT (self->privat->error), "toggled",
 					  G_CALLBACK (on_filter_buttons_toggled), self);
 	
+	gtk_box_pack_start (GTK_BOX (filter_buttons_box), GTK_WIDGET (self->privat->normal),
+						FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (filter_buttons_box), GTK_WIDGET (self->privat->info),
 						FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (filter_buttons_box), GTK_WIDGET (self->privat->warn),
@@ -791,6 +801,7 @@ message_view_instance_init (MessageView * self)
 
 	/* Create pixbuf column */
 	renderer_pixbuf = gtk_cell_renderer_pixbuf_new ();
+	g_object_set (G_OBJECT(renderer_pixbuf), "stock-size", GTK_ICON_SIZE_MENU, NULL);
 	column_pixbuf = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_title (column_pixbuf, _("Icon"));
 	gtk_tree_view_column_pack_start (column_pixbuf, renderer_pixbuf, TRUE);
@@ -804,6 +815,7 @@ message_view_instance_init (MessageView * self)
 	g_object_set (renderer, "yalign", 0.0, "wrap-mode", PANGO_WRAP_WORD,
 				  "wrap-width", 1000, NULL);
 	column = gtk_tree_view_column_new ();
+
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_set_title (column, _("Messages"));
 	gtk_tree_view_column_add_attribute
@@ -817,7 +829,7 @@ message_view_instance_init (MessageView * self)
 	select = gtk_tree_view_get_selection
 		(GTK_TREE_VIEW (self->privat->tree_view));
 	gtk_tree_selection_set_mode (select, GTK_SELECTION_BROWSE);
-
+	
 	/* Add tree view to a scrolled window */
 	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
 	gtk_container_add (GTK_CONTAINER (scrolled_win),
@@ -1098,16 +1110,25 @@ void message_view_previous(MessageView* view)
 
 static gboolean message_view_save_as(MessageView* view, gchar* uri)
 {
-	GnomeVFSHandle* handle;
+	GFile *file;
+	GOutputStream *os;
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	gboolean ok;
 
 	if (uri == NULL) return FALSE;
 
+	file = g_file_new_for_uri (uri);
+	os = G_OUTPUT_STREAM (
+			g_file_replace (file, NULL, 
+				FALSE,
+				G_FILE_CREATE_NONE,
+				NULL,
+				NULL));
 	/* Create file */
-	if (gnome_vfs_create (&handle, uri, GNOME_VFS_OPEN_WRITE, FALSE, 0664) != GNOME_VFS_OK)
+	if (os == NULL)
 	{
+		g_object_unref (file);
 		return FALSE;
 	}
 
@@ -1119,32 +1140,33 @@ static gboolean message_view_save_as(MessageView* view, gchar* uri)
 	while (gtk_tree_model_iter_next (model, &iter))
 	{
 		Message *message;
-		GnomeVFSFileSize written;
 
 		gtk_tree_model_get (model, &iter, COLUMN_MESSAGE, &message, -1);
 		if (message)
 		{
 			if (message->details && (strlen (message->details) > 0))
 			{
-				if (gnome_vfs_write (handle, message->details, strlen (message->details), &written) != GNOME_VFS_OK)
+				if (!g_output_stream_write (os, message->details, strlen (message->details), NULL, NULL))
 				{
 					ok = FALSE;
 				}
 			}
 			else
 			{
-				if (gnome_vfs_write (handle, message->summary, strlen (message->summary), &written) != GNOME_VFS_OK)
+				if (!g_output_stream_write (os, message->summary, strlen (message->summary), NULL, NULL))
 				{
 					ok = FALSE;
 				}
 			}
-			if (gnome_vfs_write (handle, "\n", 1, &written) != GNOME_VFS_OK)
+			if (!g_output_stream_write (os, "\n", 1, NULL, NULL))
 			{
 				ok = FALSE;
 			}
 		}
 	}
-	gnome_vfs_close (handle);
+	g_output_stream_close (os, NULL, NULL);
+	g_object_unref (os);
+	g_object_unref (file);
 
 	return ok;
 }
@@ -1292,6 +1314,12 @@ update_button_labels (MessageView* view)
 	gchar* temp;
 	
 	temp = g_strdup_printf(ngettext ("%d Message", "%d Messages", 
+									 view->privat->normal_count),
+						   view->privat->normal_count);
+	gtk_button_set_label (GTK_BUTTON (view->privat->normal), temp);
+	g_free(temp);
+	
+	temp = g_strdup_printf(ngettext ("%d Info", "%d Infos", 
 									 view->privat->info_count),
 						   view->privat->info_count);
 	gtk_button_set_label (GTK_BUTTON (view->privat->info), temp);
@@ -1359,6 +1387,7 @@ imessage_view_append (IAnjutaMessageView *message_view,
 				break;
 			default:
 				color = NULL;
+				view->privat->normal_count++;
 		}
 	}
 	update_button_labels (view);
@@ -1413,9 +1442,11 @@ imessage_view_clear (IAnjutaMessageView *message_view, GError **e)
 	view = MESSAGE_VIEW (message_view);
 
 	/* filter settings restart */
+	view->privat->normal_count = 0;	
 	view->privat->info_count = 0;
 	view->privat->warn_count = 0;
 	view->privat->error_count = 0;
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->privat->normal), TRUE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->privat->info), TRUE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->privat->warn), TRUE);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->privat->error), TRUE);
@@ -1549,7 +1580,9 @@ message_view_tree_view_filter (GtkTreeModel *model, GtkTreeIter  *iter,
 	gtk_tree_model_get (msgview->privat->model, iter, COLUMN_MESSAGE, &msg, -1);
 
 	if (msg != NULL) {
-		if (msg->type == IANJUTA_MESSAGE_VIEW_TYPE_INFO) {
+		if (msg->type == IANJUTA_MESSAGE_VIEW_TYPE_NORMAL) {
+			return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (msgview->privat->normal));
+		} else if (msg->type == IANJUTA_MESSAGE_VIEW_TYPE_INFO) {
 			return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (msgview->privat->info));
 		} else if (msg->type == IANJUTA_MESSAGE_VIEW_TYPE_WARNING) {
 			return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (msgview->privat->warn));
