@@ -23,7 +23,6 @@
 
 #include <gnome.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-plugin.h>
@@ -33,7 +32,6 @@
 #include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-editor-selection.h>
 #include <libanjuta/interfaces/ianjuta-markable.h>
-#include <libanjuta/interfaces/ianjuta-bookmark.h>
 #include <libanjuta/interfaces/ianjuta-indicable.h>
 
 #include "search-replace_backend.h"
@@ -170,8 +168,8 @@ static GladeWidget glade_widgets[] = {
 	{GE_BOOLEAN, "search.regex", NULL, NULL},
 	/* GREEDY */
 	{GE_BOOLEAN, "search.greedy", NULL, NULL},
-       	/* IGNORE_CASE */
-	{GE_BOOLEAN, "search.ignore.case", NULL, NULL},
+       	/* MATCH_CASE */
+	{GE_BOOLEAN, "search.match.case", NULL, NULL},
 	/* WHOLE_WORD */
 	{GE_BOOLEAN, "search.match.whole.word", NULL, NULL},
 	/* WORD_START */
@@ -368,16 +366,10 @@ search_and_replace (void)
 		else /* if (SE_FILE == se->type) */
 			fb = file_buffer_new_from_path(se->path, NULL, -1, 0);
 
-		if (fb)
+		if (fb && fb->buf)
 		{		
 			fb->pos = se->start_pos;
 			offset = 0;
-/* NO - there's no reason for user to expect existing marks to be removed.
-	And that can easily be done manually by user if so desired.
-			if (s->action == SA_BOOKMARK && IANJUTA_IS_MARKABLE (fb->te))
-				ianjuta_markable_delete_all_markers(IANJUTA_MARKABLE(fb->te), 
-				                                    IANJUTA_MARKABLE_LINEMARKER, NULL);
-*/
 		//FIXME enable clearing of marks by some means other than a 0-match search
 			if (s->action == SA_HIGHLIGHT)	
 				ianjuta_indicable_clear (IANJUTA_INDICABLE(fb->te), NULL); 				
@@ -424,23 +416,13 @@ search_and_replace (void)
 					case SA_BOOKMARK:
 						if (found_line != mi->line)
 						{
-							found_line = mi->line;
-						
-							if (fb->te == NULL)
-								fb->te =
-									IANJUTA_EDITOR (ianjuta_document_manager_get_current_document
-										(sr->docman, NULL));
-
-							if (IANJUTA_IS_MARKABLE (fb->te) &&
-								!ianjuta_markable_is_marker_set (
-														IANJUTA_MARKABLE(fb->te),
-														mi->line,
-														IANJUTA_MARKABLE_BOOKMARK,
-														NULL))
-							{
-								ianjuta_bookmark_toggle (IANJUTA_BOOKMARK(fb->te),
-														mi->line, FALSE, NULL);
-							}
+							GFile* file = g_file_new_for_uri (fb->uri);
+							found_line = mi->line + 1; /* different line count between search and editor */
+							ianjuta_document_manager_add_bookmark (sr->docman,
+																   file,
+																   found_line,
+																   NULL);
+							g_object_unref (file);			  
 						}
 						break;
 						
@@ -450,8 +432,12 @@ search_and_replace (void)
 							if (fb->te)
 								ianjuta_editor_goto_line (fb->te, mi->line, NULL);
 							else
-								fb->te = ianjuta_document_manager_goto_uri_line_mark
-									(sr->docman, fb->uri, mi->line, FALSE, NULL);
+							{
+								GFile* file = g_file_new_for_uri (fb->uri);
+								fb->te = ianjuta_document_manager_goto_file_line_mark
+									(sr->docman, file, mi->line, FALSE, NULL);
+								g_object_unref (file);
+							}
 							found_line = mi->line;
 						}
 						{
@@ -476,8 +462,12 @@ search_and_replace (void)
 							if (fb->te)
 								ianjuta_editor_goto_line (fb->te, mi->line, NULL);
 							else
-								fb->te = ianjuta_document_manager_goto_uri_line_mark
-									(sr->docman, fb->uri, mi->line, FALSE, NULL);
+							{
+								GFile* file = g_file_new_for_uri (fb->uri);
+								fb->te = ianjuta_document_manager_goto_file_line_mark
+									(sr->docman, file, mi->line, FALSE, NULL);
+								g_object_unref (file);
+							}
 							found_line = mi->line;
 							}
 
@@ -585,29 +575,15 @@ search_and_replace (void)
 				if (SA_SELECT == s->action ||
 					((SA_REPLACE == s->action || SA_REPLACEALL == s->action) && interactive))
 					break;
-			} // while
-/* NO - leave the current position unchanged when marking-all
-			if (nb_results > 0)
-			{
-				switch (s->action)
-				{
-					case SA_HIGHLIGHT:
-					case SA_BOOKMARK:
-						ianjuta_editor_goto_line (fb->te, found_line, NULL);
-					default:
-						break;
-				}
 			}
-*/
-		if (save_file)
-		{
+			if (save_file)
+			{
 				save_not_opened_files (fb);
-			save_file = FALSE;
+				save_file = FALSE;
+			}
+			
 		}
-		
-			file_buffer_free (fb);
-		}  // if (fb)
-
+		file_buffer_free (fb);
 		g_free (se->path);
 		g_free (se);
 
@@ -650,26 +626,13 @@ static void
 write_message_pane(IAnjutaMessageView* view, FileBuffer *fb, SearchEntry *se,
 				   MatchInfo *mi)
 {
-	gchar *match_line;
+	gchar *match_line = file_match_line_from_pos(fb, mi->pos);
+	int line = mi->line;
 	char buf[BUFSIZ];
-	gchar *tmp;
 	
-	match_line = file_match_line_from_pos(fb, mi->pos);
-
-	if (SE_BUFFER == se->type)
-	{
-		/* DEBUG_PRINT ("FBPATH  %s\n", fb->path); */
-		const gchar* filename = ianjuta_document_get_filename(IANJUTA_DOCUMENT(se->te), NULL);
-		tmp = g_strrstr(fb->path, "/");
-		tmp = g_strndup(fb->path, tmp + 1 -(fb->path));
-		snprintf(buf, BUFSIZ, "%s%s:%d:%s\n", tmp, filename,
-		         mi->line, match_line);
-		g_free(tmp);
-	}
-	else /* if (SE_FILE == se->type) */
-	{
-		snprintf(buf, BUFSIZ, "%s:%d:%s\n", se->path, mi->line + 1, match_line);
-	}
+	if (se->type == SE_FILE)
+		++line;
+    snprintf(buf, BUFSIZ, "%s:%d:%s\n", fb->path, line, match_line);
 	g_free(match_line);
 	ianjuta_message_view_buffer_append (view, buf, NULL);
 }
@@ -693,7 +656,7 @@ on_message_clicked (GObject* object, gchar* message, gpointer data)
 {
 	gchar *ptr, *ptr2;
 	gchar *path, *nline;
-	gchar *uri;
+	GFile* file;
 	gint line;
 		
 	if (!(ptr = g_strstr_len(message, strlen(message), ":")) )
@@ -706,9 +669,9 @@ on_message_clicked (GObject* object, gchar* message, gpointer data)
 	nline = g_strndup(ptr, ptr2 - ptr);
 	line = atoi(nline);
 	
-	uri = gnome_vfs_get_uri_from_local_path (path);  
-	ianjuta_document_manager_goto_uri_line_mark (sr->docman, uri, line, TRUE, NULL);
-	g_free(uri);
+	file = g_file_new_for_path (path); 
+	ianjuta_document_manager_goto_file_line_mark (sr->docman, file, line, TRUE, NULL);
+	g_object_unref (file);
 	g_free(path);
 	g_free(nline);
 	return FALSE;
@@ -717,13 +680,39 @@ on_message_clicked (GObject* object, gchar* message, gpointer data)
 static void
 save_not_opened_files(FileBuffer *fb)
 {
-	FILE *fp;
+	GFile *file;
+	GOutputStream *os;
+	gsize written;
+	gsize size;
+	gchar *buffer;
 
-	fp = fopen(fb->path, "wb");	
-	if (!fp)
-			return;
-	fwrite(fb->buf, fb->len, 1, fp);
-	fclose(fp);
+	file = g_file_new_for_uri (fb->uri);
+	os = G_OUTPUT_STREAM (g_file_replace (file, 
+			NULL,
+			FALSE,
+			G_FILE_CREATE_NONE,
+			NULL,
+			NULL));
+	if (os == NULL)
+	{
+		g_object_unref (file);
+		return;
+	}
+
+	buffer = fb->buf;
+	size = fb->len;
+	do
+	{
+		written = g_output_stream_write (os, 
+				buffer, size, NULL, NULL);
+		
+		size -= written;
+		buffer += size;
+	}
+	while (written != -1 && size > 0);
+	g_output_stream_close(os, NULL, NULL);
+	g_object_unref (os);
+	g_object_unref (file);
 }
 
 static gboolean
@@ -782,7 +771,7 @@ search_replace_next_previous(SearchDirection dir)
 	}	
 	else
 	{
-		DEBUG_PRINT ("sr null\n");
+		DEBUG_PRINT ("%s", "sr null\n");
 	}
 }
 
@@ -1178,7 +1167,7 @@ search_replace_populate(void)
 	populate_value(SEARCH_STRING, &(sr->search.expr.search_str));
 	populate_value(SEARCH_REGEX, &(sr->search.expr.regex));
 	populate_value(GREEDY, &(sr->search.expr.greedy));
-	populate_value(IGNORE_CASE, &(sr->search.expr.ignore_case));
+	populate_value(MATCH_CASE, &(sr->search.expr.match_case));
 	populate_value(WHOLE_WORD, &(sr->search.expr.whole_word));
 	populate_value(WHOLE_LINE, &(sr->search.expr.whole_line));
 	populate_value(WORD_START, &(sr->search.expr.word_start));
@@ -1439,8 +1428,8 @@ search_update_dialog(void)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), s->expr.regex);
 	widget = sr_get_gladewidget(GREEDY)->widget;
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), s->expr.greedy);
-	widget = sr_get_gladewidget(IGNORE_CASE)->widget;
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), s->expr.ignore_case);
+	widget = sr_get_gladewidget(MATCH_CASE)->widget;
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), s->expr.match_case);
 	widget = sr_get_gladewidget(WHOLE_WORD)->widget;
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), s->expr.whole_word);
 	widget = sr_get_gladewidget(WHOLE_LINE)->widget;
@@ -1452,6 +1441,7 @@ search_update_dialog(void)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), s->expr.no_limit);
 	widget = sr_get_gladewidget(ACTIONS_MAX)->widget;
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), s->expr.actions_max);
+	gtk_widget_set_sensitive (widget, !s->expr.no_limit);
 	
 	widget = sr_get_gladewidget(REPLACE_REGEX)->widget;	
 	gtk_widget_set_sensitive(widget, sr->search.expr.regex);
@@ -1821,7 +1811,7 @@ void search_replace_find_usage(const gchar *symbol)
 	sr->search.expr.search_str = g_strdup (symbol);
 	sr->search.expr.regex = FALSE;
 	sr->search.expr.greedy = FALSE;
-	sr->search.expr.ignore_case = FALSE;
+	sr->search.expr.match_case = TRUE;
 	sr->search.expr.whole_word = TRUE;
 	sr->search.expr.whole_line = FALSE;
 	sr->search.expr.word_start = FALSE;
