@@ -39,12 +39,13 @@
 
 #include <gnome.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-shell.h>
 #include <libanjuta/anjuta-plugin.h>
 #include <libanjuta/anjuta-debug.h>
+#include <libanjuta/anjuta-encodings.h>
+#include <libanjuta/anjuta-convert.h>
 #include <libanjuta/interfaces/ianjuta-editor.h>
 #include <libanjuta/interfaces/ianjuta-document.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
@@ -61,7 +62,6 @@
 */
 
 #include "search-replace_backend.h"
-#include "tm_tagmanager.h"
 
 /* Information about a matched substring */
 typedef struct _MatchSubStr
@@ -74,57 +74,9 @@ static SearchReplace *sr = NULL;
 
 void clear_search_replace_instance(void);
 
-
-static void
-pcre_info_free (PcreInfo *re)
-{
-	if (re)
-	{
-		if (re->re)
-			(*pcre_free)(re->re);
-		if (re->extra)
-			(*pcre_free)(re->extra);
-		if (re->ovector)
-			g_free(re->ovector);
-		g_free(re);
-	}
-}
-
-static PcreInfo *
-pcre_info_new (SearchExpression *s)
-{
-	PcreInfo *re;
-	int options = 0;
-	const char *err;
-	int err_offset;
-	int status;
-
-	g_return_val_if_fail(s && s->search_str, NULL);
-	re = g_new0(PcreInfo, 1);
-	if (s->ignore_case)
-		options |= PCRE_CASELESS;
-	if (!s->greedy)
-		options |= PCRE_UNGREEDY;
-	re->re = pcre_compile(s->search_str, options, &err, &err_offset, NULL);
-	if (NULL == re->re)
-	{
-		/* Compile failed - check error message */
-		g_warning("Regex compile failed! %s at position %d", err, err_offset);
-		pcre_info_free(re);
-		return NULL;
-	}
-	re->extra = pcre_study(re->re, 0, &err);
-	status = pcre_fullinfo(re->re, re->extra, PCRE_INFO_CAPTURECOUNT
-	  , &(re->ovec_count));
-	re->ovector = g_new0(int, 3 *(re->ovec_count + 1));
-	return re;
-}
-
-
 static void match_substr_free(MatchSubStr *ms)
 {
-	if (ms)
-		g_free(ms);
+	g_free(ms);
 }
 
 
@@ -167,60 +119,58 @@ FileBuffer *
 file_buffer_new_from_te (IAnjutaEditor *te)
 {
 	FileBuffer *fb;
-	gchar* uri;
-	gchar* path;
+	GFile* file;
 	
 	g_return_val_if_fail(te, NULL);
 	fb = g_new0(FileBuffer, 1);
 	fb->type = FB_EDITOR;
 	fb->te = te;
 	
-	uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
-	path = gnome_vfs_get_local_path_from_uri(uri);
-	if (path)
-	{
-		fb->path = tm_get_real_path(path);
-		g_free (path);
-	}
-	fb->uri = uri;
+	file = ianjuta_file_get_file(IANJUTA_FILE(te), NULL);
+	fb->path = g_file_get_path (file);
+	fb->uri = g_file_get_uri (file);
 	fb->len = ianjuta_editor_get_length(te, NULL);
 	fb->buf = ianjuta_editor_get_text_all (fb->te, NULL);
 	fb->pos = ianjuta_editor_get_offset(fb->te, NULL);
 	fb->line = ianjuta_editor_get_lineno(fb->te, NULL);
 	
+	g_object_unref (file);
+	
 	return fb;
 }
 
+/* Only use the first 500 chars for validating (yes, I feel lucky...) */
+#define MAX_VALIDATE 500
+
 FileBuffer *
 file_buffer_new_from_path (const char *path, const char *buf, int len, int pos)
 {
 	FileBuffer *fb;
 	IAnjutaEditor *te;
 	IAnjutaDocument* doc;
-	char *real_path;
+	GFile* file;
 	char *uri;
 	int i;
 	int lineno;
 
 	g_return_val_if_fail(path, NULL);
-	real_path = tm_get_real_path(path);
 	
 	/* There might be an already open TextEditor with this path */
-	uri = gnome_vfs_get_uri_from_local_path (real_path);
-	doc = ianjuta_document_manager_find_document_with_uri (sr->docman,
-														 uri, NULL);
-
+	file = g_file_new_for_path (path);
+	uri = g_file_get_uri (file);
+	doc = ianjuta_document_manager_find_document_with_file (sr->docman,
+														 file, NULL);
 	if (doc && IANJUTA_IS_EDITOR (doc))
 	{
 		te = IANJUTA_EDITOR (doc);
-		g_free(real_path);
 		return file_buffer_new_from_te(te);
 	}
 	fb = g_new0(FileBuffer, 1);
 	fb->type = FB_FILE;
-	fb->path = real_path;
+	fb->path = g_file_get_path (file);
 	fb->uri = uri;
 	fb->name = strrchr(path, '/');
+	g_object_unref (file);
 	if (fb->name)
 		++ fb->name;
 	else
@@ -234,37 +184,109 @@ file_buffer_new_from_path (const char *path, const char *buf, int len, int pos)
 	}
 	else
 	{
-		struct stat s;
+		GFileInfo *fi;
+		guint64 size;
 
-		if ((0 == stat(fb->path, &s)) && (S_ISREG(s.st_mode)))
+		file = g_file_new_for_path (fb->path);
+		fi = g_file_query_info (file,
+				G_FILE_ATTRIBUTE_STANDARD_SIZE ","
+				G_FILE_ATTRIBUTE_STANDARD_TYPE,
+				G_FILE_QUERY_INFO_NONE,
+				NULL, NULL);
+		if (fi != NULL && g_file_info_get_file_type(fi) == G_FILE_TYPE_REGULAR)
 		{
-			if ((fb->len = s.st_size) < 0) return NULL;
-			fb->buf = g_new(char, s.st_size + 1);
+			size = g_file_info_get_attribute_uint64(fi, G_FILE_ATTRIBUTE_STANDARD_SIZE);
+			g_object_unref (fi);
+			fi = NULL;
+
+			if ((fb->len = size) < 0) return NULL;
+			fb->buf = g_new(char, size + 1);
 			{
-				int total_bytes = 0, bytes_read, fd;
-				if (0 > (fd = open(fb->path, O_RDONLY)))
+				int total_bytes = 0, bytes_read;
+				GInputStream *is;
+				GError* error = NULL;
+
+				is = G_INPUT_STREAM (g_file_read (file, NULL, &error));
+				if (NULL == is)
 				{
-					perror(fb->path);
-					file_buffer_free(fb);
+					DEBUG_PRINT("Error opening file: %s", error->message);
+					g_error_free (error);
+					file_buffer_free (fb);
+					g_object_unref (file);
 					return NULL;
 				}
-				while (total_bytes < s.st_size)
+
+				while (total_bytes < size)
 				{
-					if (0 > (bytes_read = read(fd, fb->buf + total_bytes
-					  , s.st_size - total_bytes)))
+					bytes_read = g_input_stream_read (is,
+							fb->buf + total_bytes,
+							size - total_bytes,
+							NULL,
+							&error);
+					if (bytes_read == -1)
 					{
-						perror(fb->path);
-						close(fd);
-						file_buffer_free(fb);
+						DEBUG_PRINT("Error reading from file: %s", error->message);
+						g_error_free (error);
+						file_buffer_free (fb);
+						g_input_stream_close (is, NULL, NULL);
+						g_object_unref (is);
+						g_object_unref (file);
 						return NULL;
 					}
 					total_bytes += bytes_read;
 				}
-				close(fd);
+
+				g_input_stream_close (is, NULL, NULL);
+				g_object_unref (is);
+				g_object_unref (file);
 				fb->buf[fb->len] = '\0';
 			}
 		}
+		else
+		{
+			if (NULL != fi)
+			{
+				g_object_unref (fi);
+			}
+		}
 	}
+
+	if (!g_utf8_validate (fb->buf, MIN(MAX_VALIDATE, fb->len), NULL))
+	{
+		const AnjutaEncoding *encoding_used = NULL;
+		gchar* converted_text;
+		gsize converted_len;
+		converted_text = anjuta_convert_to_utf8 (fb->buf, 
+												 fb->len, 
+												 &encoding_used,
+												 &converted_len, 
+												 NULL);
+		if (converted_text == NULL)
+		{
+			/* Last change, let's try 8859-15 */
+			encoding_used =  
+				anjuta_encoding_get_from_charset("ISO-8859-15");
+			
+			converted_text = anjuta_convert_to_utf8 (fb->buf,
+												  fb->len,
+												  &encoding_used,
+												  &converted_len,
+												  NULL);
+		}
+		if (converted_text == NULL)
+		{
+			/* Give up */
+			file_buffer_free(fb);
+			return NULL;
+		}
+		else
+		{
+			g_free (fb->buf);
+			fb->buf = converted_text;
+			fb->len = converted_len;
+		}
+	}
+	
 	if (pos <= 0 || pos > fb->len)
 	{
 		fb->pos = 0;
@@ -300,9 +322,10 @@ file_buffer_line_from_pos(FileBuffer *fb, int pos)
 	g_return_val_if_fail(fb && pos >= 0, 1);
 	if (FB_FILE == fb->type)
 	{
+		gchar* p = g_utf8_offset_to_pointer(fb->buf, pos);
 		for (tmp = fb->lines; tmp; tmp = g_list_next(tmp))
 		{
-			if (pos < GPOINTER_TO_INT(tmp->data))
+			if (p - fb->buf < GPOINTER_TO_INT(tmp->data))
 				return lineno;
 			++ lineno;
 		}
@@ -324,35 +347,17 @@ gchar *
 file_match_line_from_pos(FileBuffer *fb, int pos)
 {
 	gint length=1;
+	int start;
 	gint i;
 	g_return_val_if_fail(fb && pos >= 0, NULL);
 
-	for (i= pos+1; ((fb->buf[i] != '\n') && (fb->buf[i] != '\0')); i++, length++);
-	for (i= pos-1; (fb->buf[i] != '\n') && (i >= 0); i--, length++);
+	start = g_utf8_offset_to_pointer(fb->buf, pos) - fb->buf;
+	for (i= start+1; ((fb->buf[i] != '\n') && (fb->buf[i] != '\0')); i++, length++);
+	for (i= start-1; (fb->buf[i] != '\n') && (i >= 0); i--, length++);
 	
 	return g_strndup (fb->buf + i + 1, length);
 }
 
-/* Generate a list of files to search in. Call with start = TRUE and
-** top_dir = sf->top_dir. This is used when the search range is specified as
-SR_FILES */
-static GList *
-create_search_files_list(SearchFiles *sf, const char *top_dir)
-{
-	TMFileEntry *entry;
-	GList *files;
-
-	g_return_val_if_fail(sf && top_dir, NULL);
-	entry = tm_file_entry_new(top_dir, NULL, sf->recurse, sf->match_files
-	  , sf->ignore_files, sf->match_dirs, sf->ignore_dirs
-	  , sf->ignore_hidden_files, sf->ignore_hidden_dirs);
-	if (!entry)
-		return NULL;
-	files = tm_file_entry_list(entry, NULL);
-	tm_file_entry_free(entry);
-	return files;
-}
-
 /* Get a list of all project files */
 static GList *
 get_project_file_list(void)
@@ -385,7 +390,7 @@ get_project_file_list(void)
 				gchar *file_path;
 		
 				uri = (const gchar *)node->data;
-				file_path = gnome_vfs_get_local_path_from_uri (uri);
+				file_path = anjuta_util_get_local_path_from_uri (uri);
 				if (file_path)
 				files = g_list_prepend (files, file_path);
 				node = g_list_next (node);
@@ -406,27 +411,29 @@ isawordchar (gunichar c)
 }
 
 static gboolean
-extra_match (FileBuffer *fb, gchar* begin, gchar* end, SearchExpression *s)
+extra_match (gboolean at_start, gchar* begin, gchar* end, SearchExpression *s)
 {
 	gunichar b, e;
 	
 	b = g_utf8_get_char (g_utf8_prev_char (begin));
 	e = g_utf8_get_char (end);
+	if (g_unichar_ismark(e))
+		return FALSE;  /* matched character continues past match end */
 	
 	if (s->whole_line)
-		if ((fb->pos == 0 || b == '\n' || b == '\r') &&
+		if ((at_start || b == '\n' || b == '\r') &&
 			(e == '\0'	|| e == '\n' || e == '\r'))
 			return TRUE;
 		else
 			return FALSE;
 	else if (s->whole_word)
-		if ((fb->pos ==0 || !isawordchar(b)) && 
+		if ((at_start || !isawordchar(b)) && 
 			(e=='\0' || !isawordchar(e)))
 			return TRUE;
 		else
 			return FALSE;
 	else if (s->word_start)
-		if (fb->pos ==0 || !isawordchar(b))
+		if (at_start || !isawordchar(b))
 			return TRUE;
 		else
 			return FALSE;	
@@ -434,188 +441,237 @@ extra_match (FileBuffer *fb, gchar* begin, gchar* end, SearchExpression *s)
 		return TRUE;
 }
 
-/* Returns the next match in the passed buffer. The search expression should
-   be pre-compiled. The returned pointer should be freed with match_info_free()
-   when no longer required. */
-MatchInfo *
-get_next_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
+static MatchInfo *
+get_next_regex_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
 {
 	MatchInfo *mi = NULL;
-
-	g_return_val_if_fail(fb && s, NULL);
+	GMatchInfo* match_info;
 	
-	if (s->regex)
+	if (s->regex_info == NULL)
 	{
-		/* Regular expression match */
-		int options = PCRE_NOTEMPTY;
-		int status;
-		if (NULL == s->re)
+		GError* error = NULL;
+		GRegexCompileFlags compile_flags = 0;
+		GRegexMatchFlags match_flags = 0;
+	
+		match_flags |= G_REGEX_MATCH_NOTEMPTY;
+		if (!s->match_case)
 		{
-			if (NULL == (s->re = pcre_info_new(s)))
-				return NULL;
+			compile_flags |= G_REGEX_CASELESS;
 		}
-		status = pcre_exec (s->re->re, s->re->extra, fb->buf, fb->len, fb->pos,
-							options, s->re->ovector, 3 * (s->re->ovec_count + 1));
-		if (0 == status)
+		if (!s->greedy)
 		{
-			/* ovector too small - this should never happen ! */
-			g_warning("BUG ! ovector found to be too small");
+			compile_flags |= G_REGEX_UNGREEDY;
+		}
+		s->regex_info = g_regex_new (s->search_str, compile_flags,
+									 match_flags, &error);
+		if (error)
+		{
+			anjuta_util_dialog_error (NULL, error->message);
+			g_error_free(error);
+			s->regex_info = NULL;
 			return NULL;
 		}
-		else if (0 > status && status != PCRE_ERROR_NOMATCH)
+	}
+
+	g_regex_match_full (s->regex_info, fb->buf, fb->len,
+				   g_utf8_offset_to_pointer (fb->buf, fb->pos) - fb->buf, 
+				   G_REGEX_MATCH_NOTEMPTY, &match_info, NULL);
+	
+	if (g_match_info_matches (match_info))
+	{
+		gint start;
+		gint end;
+		int i;
+		mi = g_new0(MatchInfo, 1);
+		if (g_match_info_fetch_pos (match_info, 0, &start, &end))
 		{
-			/* match error - again, this should never happen */
-			g_warning("PCRE Match error");
-			return NULL;
-		}
-		else if (PCRE_ERROR_NOMATCH != status)
-		{
-			mi = g_new0(MatchInfo, 1);
-			mi->pos = s->re->ovector[0];
-			mi->len = s->re->ovector[1] - s->re->ovector[0];
+			DEBUG_PRINT ("Regex: %d %d", start, end);
+			mi->pos = g_utf8_pointer_to_offset (fb->buf, fb->buf + start);
+			mi->len = g_utf8_pointer_to_offset (fb->buf, fb->buf + end) - mi->pos;
 			mi->line = file_buffer_line_from_pos(fb, mi->pos);
-			if (status > 1) /* Captured subexpressions */
-			{
-				int i;
-				MatchSubStr *ms;
-				for (i=1; i < status; ++i)
-				{
-					ms = g_new0(MatchSubStr, 1);
-					ms->start = s->re->ovector[i * 2];
-					ms->len = s->re->ovector[i * 2 + 1] - ms->start;
-					mi->subs = g_list_prepend(mi->subs, ms);
-				}
-				mi->subs = g_list_reverse(mi->subs);
-			}
-			fb->pos = s->re->ovector[1];
 		}
+		for (i = 1; i < g_match_info_get_match_count(match_info); i++) /* Captured subexpressions */
+		{
+			MatchSubStr *ms;
+			ms = g_new0(MatchSubStr, 1);
+			if (g_match_info_fetch_pos (match_info, i, &start, &end))
+			{
+				ms->start = g_utf8_pointer_to_offset (fb->buf, fb->buf + start);
+				ms->len = g_utf8_pointer_to_offset (fb->buf, fb->buf + end) - ms->start;
+			}
+			mi->subs = g_list_prepend(mi->subs, ms);
+		}
+		mi->subs = g_list_reverse(mi->subs);
+		fb->pos = g_utf8_pointer_to_offset (fb->buf, fb->buf + end);
+	}
+	return mi;
+}
+
+static MatchInfo *match_info(FileBuffer *fb, gchar* match, gchar* match_end,
+							 SearchDirection direction)
+{
+	MatchInfo *mi = g_new0 (MatchInfo, 1);
+	mi->pos = g_utf8_pointer_to_offset(fb->buf, match);
+	mi->len = g_utf8_pointer_to_offset(match, match_end);
+	mi->line = file_buffer_line_from_pos (fb, mi->pos);
+
+	if (direction == SD_BACKWARD)
+		fb->pos = mi->pos;
+	else
+		fb->pos = mi->pos + mi->len;
+	
+	return mi;
+}
+
+/* Get the next match, given that the search string is ASCII (as will nearly
+ * always be the case).  In this case we don't need to perform any UTF-8
+ * normalization, since any bytes with values 0-127 in a UTF-8 string always
+ * actually represent ASCII characters. */
+static MatchInfo *
+get_next_ascii_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
+{
+	int len = strlen(s->search_str);
+	gint (*compare)(const gchar *, const gchar *, gsize) =
+		s->match_case ? strncmp : g_ascii_strncasecmp;
+	gchar *p = g_utf8_offset_to_pointer (fb->buf, fb->pos);
+	
+	if (direction == SD_BACKWARD)
+	{
+		for (p -= len; p >= fb->buf; --p)
+			if (!compare(p, s->search_str, len) &&
+				extra_match(p == fb->buf, p, p + len, s))
+				return match_info(fb, p, p + len, direction);
 	}
 	else
 	{
-		/* Simple string search - this needs to be performance-tuned */
-		gboolean found;
-		gint match_len;
+		for (; *p ; ++p)
+			if (!compare(p, s->search_str, len) &&
+				extra_match(p == fb->buf, p, p + len, s))
+				return match_info(fb, p, p + len, direction);
+	}
+	
+	return NULL;			
+}
 
-		match_len = strlen (s->search_str);
-		if (match_len == 0)
-			return NULL;
+/* Normalize a UTF-8 string, optionally folding case.  Returns NULL if invalid. */
+static gchar* normalize(gchar *s, int len_bytes, gboolean match_case) {
+	gchar *c, *n;
+	
+	if (match_case)
+		return g_utf8_normalize(s, len_bytes, G_NORMALIZE_NFD);
+	
+	c = g_utf8_casefold(s, len_bytes);
+	n = g_utf8_normalize(c, -1, G_NORMALIZE_NFD);
+	g_free(c);
+	return n;
+}
 
-		found = FALSE;
-		if (SD_BACKWARD == direction)
+/* Given a UTF-8 string s, advance past a prefix whose normalized length in
+ * bytes is [normal_bytes]. */
+static gchar* normal_advance(gchar *s, int normal_bytes, gboolean match_case)
+{
+	while (*s && normal_bytes > 0)
+	{
+		gchar *next = g_utf8_next_char(s);
+		gchar *n = normalize(s, next - s, match_case);
+		normal_bytes -= strlen(n);
+		g_free(n);
+		s = next;
+	}
+	return s;
+}
+
+static MatchInfo *
+get_next_utf8_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
+{
+	gchar* search_key = normalize(s->search_str, -1, s->match_case);
+	gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
+	gchar* search_buf;
+	gchar* p = NULL;
+	MatchInfo *mi = NULL;
+	int keylen;
+	
+	if (!search_key)
+		return NULL;
+	keylen = strlen(search_key);
+	
+	if (direction == SD_BACKWARD)
+	{
+		search_buf = normalize(fb->buf, current - fb->buf, s->match_case);
+		if (search_buf)
 		{
-			/* Backward matching. */
-			if (s->ignore_case)
+			p = search_buf + strlen(search_buf);
+			while ((p = g_strrstr_len(search_buf, p - search_buf, search_key)) != NULL)
 			{
-				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
-				gint len = g_utf8_strlen (s->search_str, -1);
-				gchar* search_caseless = g_utf8_casefold (s->search_str, len);
-				for (; fb->pos >= len; --fb->pos)
-				{
-					gchar* current_caseless = g_utf8_casefold (current, len);
-					if (g_utf8_collate (current_caseless, search_caseless) == 0 &&
-						extra_match (fb, current, current + strlen (search_caseless),
-									 s))
-					{
-						found = TRUE;
-						g_free (current_caseless);
-						break;
-					}
-					else
-						current = g_utf8_prev_char (current);
-				}
-				g_free (search_caseless);
+				if (extra_match (p == search_buf, p, p + keylen, s))
+					break;
+				p = p + keylen - 1;
 			}
-			else
-			{
-				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
-				gint len = g_utf8_strlen (s->search_str, -1);
-				gchar* search_key = g_utf8_collate_key (s->search_str, len);
-				for (; fb->pos >= len; --fb->pos)
-				{
-					gchar* current_key = g_utf8_collate_key (current, len);
-					if (g_str_equal (current_key, search_key) &&
-						extra_match (fb, current, current + strlen (s->search_str),
-									 s))
-					{
-						found = TRUE;
-						g_free (current_key);
-						break;
-					}
-					else
-						current = g_utf8_prev_char (current);
-				}
-				g_free (search_key);
-			}
-		}
-		else
-		{
-			/* Forward match */
-			if (s->ignore_case)
-			{
-				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
-				gint len = g_utf8_strlen (s->search_str, -1);
-				gchar* search_caseless = g_utf8_casefold (s->search_str, len);
-				gint buf_len = g_utf8_strlen (fb->buf, fb->len);
-				for (; fb->pos < buf_len; ++fb->pos)
-				{
-					gchar* current_caseless = g_utf8_casefold (current, len);
-					if (g_utf8_collate (current_caseless, search_caseless) == 0 &&
-						extra_match (fb, current, current + strlen (search_caseless),
-									 s))
-					{
-						found = TRUE;
-						g_free (current_caseless);
-						break;
-					}
-					else
-						current = g_utf8_next_char (current);
-				}
-				g_free (search_caseless);
-			}
-			else
-			{
-				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
-				gint len = g_utf8_strlen (s->search_str, -1);
-				gint buf_len = g_utf8_strlen (fb->buf, fb->len);
-				gchar* search_key = g_utf8_collate_key (s->search_str, len);
-				for (; fb->pos < buf_len; ++fb->pos)
-				{
-					gchar* current_key = g_utf8_collate_key (current, len);
-					if (g_str_equal (current_key, search_key) &&
-						extra_match (fb, current, current + strlen (s->search_str),
-									 s))
-					{
-						found = TRUE;
-						g_free (current_key);
-						break;
-					}
-					else
-						current = g_utf8_next_char (current);
-				}
-				g_free (search_key);
-			}
-		}
-		if (found)
-		{
-			mi = g_new0 (MatchInfo, 1);	//better to abort than silently fail to report match ?
-//			mi = g_try_new0 (MatchInfo, 1);
-//			if (mi)
-//			{
-				mi->pos = fb->pos;
-				mi->len = match_len;
-				mi->line = file_buffer_line_from_pos (fb, fb->pos);
-//			}
-//			else
-//				WARN USER ABOUT MEMORY ERROR
-			if (direction == SD_BACKWARD)
-				fb->pos -= match_len;
-			else
-				fb->pos += match_len;
 		}
 	}
+	else
+	{  /* forward search */
+		search_buf = normalize(current, -1, s->match_case);
+		if (search_buf)
+		{
+			p = search_buf;
+			while ((p = strstr(p, search_key)) != NULL)
+			{
+				if (extra_match (fb->pos == 0 && p == search_buf, p, p + keylen, s))
+					break;
+				++p;
+			}
+		}
+	}
+	g_free(search_key);
+	
+	if (p)
+	{
+		/* We've found a match in the normalized buffer.  Now find the
+		 * corresponding position in the source buffer. */
+		gchar* match = normal_advance(
+								direction == SD_BACKWARD ? fb->buf : current,
+								p - search_buf, s->match_case);
+		gchar* match_end = normal_advance(match, keylen, s->match_case);
+		mi = match_info(fb, match, match_end, direction);
+	}
+	
+	g_free(search_buf);
 	return mi;
 }
 
+static gboolean str_is_ascii(gchar* s)
+{
+	for (; *s; ++s)
+		if (!isascii(*s))
+			return FALSE;
+	return TRUE;
+}
+
+/* Returns the next match in the passed buffer.  If direction is SD_BACKWARD,
+   finds the previous match whose end falls before the current position.
+   The search expression should be pre-compiled.  The returned pointer should
+   be freed with match_info_free() when no longer required. */
+MatchInfo *
+get_next_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
+{
+	g_return_val_if_fail(fb && s, NULL);
+	
+	if (s->regex)
+		return get_next_regex_match(fb, direction, s);
+	
+	return str_is_ascii(s->search_str) ?
+		get_next_ascii_match(fb, direction, s) :
+		get_next_utf8_match(fb, direction, s);
+}
+
+static gint search_entry_compare(gconstpointer a, gconstpointer b)
+{
+	gchar* a_path = ((SearchEntry *) a)->path;
+	gchar* b_path = ((SearchEntry *) b)->path;
+	return strcmp(a_path, b_path);
+}
+
 /* Create list of search entries */
 GList *
 create_search_entries (Search *s)
@@ -761,7 +817,7 @@ create_search_entries (Search *s)
 				entries = g_list_prepend(entries, se);
 			}
 			}
-			entries = g_list_reverse(entries);
+			entries = g_list_sort(entries, search_entry_compare);
 			g_list_free (editors);
 			break;
 		case SR_FILES:
@@ -776,7 +832,7 @@ create_search_entries (Search *s)
 							  &dir_uri, NULL);
 			// FIXME : Replace Standard UNIX IO functions by gnome-vfs 
 			if (dir_uri)			
-				dir = gnome_vfs_get_local_path_from_uri(dir_uri);
+				dir = anjuta_util_get_local_path_from_uri(dir_uri);
 			
 			if (!dir)
 			{
@@ -785,10 +841,7 @@ create_search_entries (Search *s)
 				dir = g_get_current_dir();
 			}
 
-			if (SR_FILES == s->range.type)
-				files = create_search_files_list(&(s->range.files), dir);
-			else /* if (SR_PROJECT == s->range.type) */
-				files = get_project_file_list();	
+			files = get_project_file_list();	
 			
 			if (files)
 			{
@@ -804,7 +857,7 @@ create_search_entries (Search *s)
 					entries = g_list_prepend(entries, se);
 				}
 				g_list_free(files);
-				entries = g_list_reverse(entries);
+				entries = g_list_sort(entries, search_entry_compare);
 			}
 			g_free(dir);
 			g_free(dir_uri);
@@ -870,8 +923,11 @@ void
 clear_search_replace_instance(void)
 {
 	g_free (sr->search.expr.search_str);
-	g_free (sr->search.expr.re);
-	FREE_FN(pcre_info_free, sr->search.expr.re);
+	if (sr->search.expr.regex_info)
+	{
+		g_regex_unref (sr->search.expr.regex_info);
+		sr->search.expr.regex_info = NULL;
+	}
 	if (SR_FILES == sr->search.range.type)
 	{
 		FREE_FN(anjuta_util_glist_strings_free, sr->search.range.files.match_files);
@@ -884,17 +940,15 @@ clear_search_replace_instance(void)
 	FREE_FN(anjuta_util_glist_strings_free, sr->replace.expr_history);
 }
 
-void
-clear_pcre(void)
-{
-	FREE_FN(pcre_info_free, sr->search.expr.re);
-}
-
 SearchReplace *
 create_search_replace_instance(IAnjutaDocumentManager *docman)
 {
-	if (NULL == sr) /* Create a new SearchReplace instance */
+	/* Create a new SearchReplace instance */
+	if (NULL == sr)
+	{
 		sr = g_new0(SearchReplace, 1);
+		sr->search.expr.regex_info = NULL;
+	}
 	else
 		clear_search_replace_instance ();
 	if (docman)
