@@ -32,7 +32,6 @@
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
 #include <libanjuta/interfaces/ianjuta-markable.h>
 #include <libanjuta/interfaces/ianjuta-indicable.h>
-#include <libanjuta/interfaces/ianjuta-bookmark.h>
 #include <libanjuta/interfaces/ianjuta-print.h>
 #include <libanjuta/interfaces/ianjuta-language-support.h>
 #include <libanjuta/interfaces/ianjuta-document.h>
@@ -441,7 +440,6 @@ on_open_finish(SourceviewIO* io, Sourceview* sv)
     g_signal_emit_by_name(G_OBJECT(sv), "save_point",
 						  TRUE);
 	
-	sv->priv->loading = FALSE;
 	if (sv->priv->goto_line > 0)
 	{
 		goto_line (sv, sv->priv->goto_line);
@@ -598,6 +596,9 @@ sourceview_dispose(GObject *object)
 		gtk_widget_destroy(GTK_WIDGET(cobj->priv->assist_tip));
 	g_object_unref (cobj->priv->io);
 	
+	if (cobj->priv->tooltip_cell)
+		g_object_unref (cobj->priv->tooltip_cell);
+	
 	for (node = cobj->priv->idle_sources; node != NULL; node = g_slist_next (node))
 	{
 		g_source_remove (GPOINTER_TO_UINT (node->data));
@@ -617,7 +618,7 @@ sourceview_finalize(GObject *object)
 	
 	g_slice_free(SourceviewPrivate, cobj->priv);
 	G_OBJECT_CLASS(parent_class)->finalize(object);
-	DEBUG_PRINT("=========== finalise =============");
+	DEBUG_PRINT("%s", "=========== finalise =============");
 }
 
 /* Create a new sourceview instance. If uri is valid,
@@ -653,7 +654,7 @@ sourceview_new(GFile* file, const gchar* filename, AnjutaPlugin* plugin)
 	else if (filename != NULL && strlen(filename) > 0)
 		sourceview_io_set_filename (sv->priv->io, filename);
 		
-	DEBUG_PRINT("============ Creating new editor =============");
+	DEBUG_PRINT("%s", "============ Creating new editor =============");
 	
 	g_signal_emit_by_name (G_OBJECT(sv), "update-ui");
 	
@@ -1312,9 +1313,14 @@ idocument_iface_init (IAnjutaDocumentIface *iface)
 static void
 set_select(Sourceview* sv, GtkTextIter* start_iter, GtkTextIter* end_iter)
 {
-	gtk_text_buffer_select_range (GTK_TEXT_BUFFER (sv->priv->document), start_iter, end_iter);
-	gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(sv->priv->view),
-								 gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(sv->priv->document)));
+	GtkTextBuffer* buffer = GTK_TEXT_BUFFER (sv->priv->document);
+	gtk_text_buffer_select_range (buffer, start_iter, end_iter);
+	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (sv->priv->view),
+								  gtk_text_buffer_get_insert (buffer),
+								  0.25,
+								  FALSE,
+								  0.0,
+								  0.0);
 }
 
 /* IAnjutaEditorSelection */
@@ -1326,55 +1332,71 @@ iselect_to_brace(IAnjutaEditorSelection* edit, GError** e)
 	
 }
 
+/* Find the previous open brace that begins the current indentation level. */
+static gboolean find_open_bracket(GtkTextIter *iter)
+{
+	int level = 1;
+	
+	while (gtk_text_iter_backward_char (iter))
+	{
+		switch (gtk_text_iter_get_char (iter))
+		{
+		case '{':
+			if (!--level)
+				return TRUE;
+			break;
+		case '}':
+			++level;
+			break;
+		}
+	}
+	
+	return FALSE;
+}
+
+/* Find the following close brace that ends the current indentation level. */
+static gboolean find_close_bracket(GtkTextIter *iter)
+{
+	int level = 1;
+	
+	while (gtk_text_iter_forward_char (iter))
+	{
+		switch (gtk_text_iter_get_char (iter))
+		{
+		case '{':
+			++level;
+			break;
+		case '}':
+			if (!--level)
+				return TRUE;
+			break;
+		}
+	}
+	
+	return FALSE;
+}
+
 static void 
 iselect_block(IAnjutaEditorSelection* edit, GError** e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
 	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(sv->priv->document);
-	GtkTextIter start_iter;
-	GtkTextIter end_iter;
-	GtkTextIter iter;
-	gchar *text;
-	gint position;
 	
-	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(sv->priv->document),
-								   &start_iter);
-	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(sv->priv->document),
-								   &end_iter);
-	text = gtk_text_buffer_get_slice
-	(GTK_TEXT_BUFFER(sv->priv->document),
-										&start_iter, &end_iter, TRUE);
-	if (text)
+	GtkTextIter iter;
+	gtk_text_buffer_get_iter_at_mark (buffer, &iter,
+								 gtk_text_buffer_get_insert(buffer));
+	if (find_open_bracket (&iter))
 	{
-		gboolean found = FALSE;
-		gint cpt = 0;
-		gtk_text_buffer_get_iter_at_mark(buffer, &iter, 
-									 gtk_text_buffer_get_insert(buffer));
-		position = gtk_text_iter_get_offset(&iter);
-		
-		while((--position >= 0) && !found)
+		GtkTextIter end_iter;
+		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER(sv->priv->document),
+								     &iter);
+		end_iter = iter;
+		if (find_close_bracket (&end_iter))
 		{
-			if (text[position] == '{')
-				if (cpt-- == 0)
-					found = TRUE;
-			if (text[position] == '}')
-				cpt++;
-		}			
-		if (found)
-		{
-			gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(sv->priv->document),
-									           &start_iter, position + 2);
-			gtk_text_buffer_place_cursor(GTK_TEXT_BUFFER(sv->priv->document),
-									     &start_iter);
-			end_iter = start_iter;
-			found = FALSE;
-			//found = gtk_source_iter_find_matching_bracket (&end_iter);
-			if (found)
-				set_select(sv, &start_iter, &end_iter);
+			gtk_text_iter_forward_char (&end_iter);	  /* move past brace */
+			set_select (sv, &iter, &end_iter);
 		}
-		g_free(text);
 	}
-
 }
 
 static void
@@ -1384,12 +1406,9 @@ iselect_set (IAnjutaEditorSelection* edit,
 			 GError** e)
 {
 	Sourceview* sv = ANJUTA_SOURCEVIEW(edit);
-	gtk_text_buffer_select_range (GTK_TEXT_BUFFER (sv->priv->document),
-								  sourceview_cell_get_iter (SOURCEVIEW_CELL (istart)),
-								  sourceview_cell_get_iter (SOURCEVIEW_CELL (iend)));
-	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (sv->priv->view),
-												 sourceview_cell_get_iter (SOURCEVIEW_CELL (istart)),
-												 0, FALSE, 0, 0);
+	set_select(sv,
+			   sourceview_cell_get_iter (SOURCEVIEW_CELL (istart)),
+			   sourceview_cell_get_iter (SOURCEVIEW_CELL (iend)));
 }
 															
 
@@ -1753,139 +1772,6 @@ iindic_iface_init(IAnjutaIndicableIface* iface)
 	iface->set = iindic_set;
 }
 
-static void
-ibookmark_toggle(IAnjutaBookmark* bmark, gint location, gboolean ensure_visible, GError** e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(bmark);
-	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
-	
-	GSList* markers;
-	
-	markers = gtk_source_buffer_get_source_marks_at_line (buffer, LOCATION_TO_LINE (location),
-												   marker_types[IANJUTA_MARKABLE_BOOKMARK]);
-	if (markers != NULL)
-	{
-		GtkTextIter begin;
-		GtkTextIter end;
-		
-		gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (buffer), &begin, LOCATION_TO_LINE (location));
-		gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (buffer), &end, LOCATION_TO_LINE (location));
-		
-		gtk_source_buffer_remove_source_marks (buffer, &begin, &end, 
-										marker_types[IANJUTA_MARKABLE_BOOKMARK]);
-	}
-	else
-	{
-		GtkTextIter line;
-		GtkSourceMark* bookmark;
-		gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (buffer),
-															 &line, LOCATION_TO_LINE (location));
-		
-		bookmark = 
-			gtk_source_buffer_create_source_mark (buffer, NULL,
-											 marker_types [IANJUTA_MARKABLE_BOOKMARK],
-											 &line);
-	}
-}
-
-static void
-goto_bookmark (Sourceview* sv, GtkTextIter* iter, gboolean backward)
-{
-	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
-	gboolean found = FALSE;
-	
-	if (backward)
-		found = gtk_source_buffer_backward_iter_to_source_mark (buffer, iter, 
-														 marker_types[IANJUTA_MARKABLE_BOOKMARK]);
-	else
-		found = gtk_source_buffer_forward_iter_to_source_mark (buffer, iter, 
-														marker_types[IANJUTA_MARKABLE_BOOKMARK]);
-	if (found)
-	{
-		ianjuta_editor_goto_line(IANJUTA_EDITOR(sv), 
-								 LINE_TO_LOCATION (gtk_text_iter_get_line (iter)), NULL);
-	}
-}
-
-static void
-ibookmark_first(IAnjutaBookmark* bmark, GError** e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(bmark);
-	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
-	GtkTextIter begin;
-	
-	gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (buffer), &begin, 0);
-	
-	goto_bookmark (sv, &begin, FALSE);
-}
-
-
-static void
-ibookmark_last(IAnjutaBookmark* bmark, GError** e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(bmark);
-	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);;
-	GtkTextIter begin;
-	
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (buffer), &begin, -1);
-	
-	goto_bookmark (sv, &begin, TRUE);
-}
-
-static void
-ibookmark_next(IAnjutaBookmark* bmark, GError** e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(bmark);
-	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(sv->priv->document);
-	GtkTextIter begin;
-	
-	gtk_text_buffer_get_iter_at_mark (buffer, &begin,
-									  gtk_text_buffer_get_insert (buffer));
-	gtk_text_iter_forward_line (&begin);
-	
-	goto_bookmark (sv, &begin, FALSE);
-}
-
-static void
-ibookmark_previous(IAnjutaBookmark* bmark, GError** e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(bmark);
-	GtkTextBuffer* buffer = GTK_TEXT_BUFFER(sv->priv->document);
-	GtkTextIter begin;
-	
-	gtk_text_buffer_get_iter_at_mark (buffer, &begin,
-									  gtk_text_buffer_get_insert (buffer));
-	gtk_text_iter_backward_line (&begin);
-	
-	goto_bookmark (sv, &begin, TRUE);
-
-}
-
-static void
-ibookmark_clear_all(IAnjutaBookmark* bmark, GError** e)
-{
-	Sourceview* sv = ANJUTA_SOURCEVIEW(bmark);
-	GtkSourceBuffer* buffer = GTK_SOURCE_BUFFER(sv->priv->document);
-	GtkTextIter begin;
-	GtkTextIter end;
-	
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (buffer), &begin, 0);
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (buffer), &end, -1);
-	
-	gtk_source_buffer_remove_source_marks (buffer, &begin, &end, marker_types[IANJUTA_MARKABLE_BOOKMARK]);
-}
-
-static void
-ibookmark_iface_init(IAnjutaBookmarkIface* iface)
-{
-	iface->toggle = ibookmark_toggle;
-	iface->first = ibookmark_first;
-	iface->last = ibookmark_last;
-	iface->next = ibookmark_next;
-	iface->previous = ibookmark_previous;
-	iface->clear_all = ibookmark_clear_all;
-}
-
 static void
 iprint_print(IAnjutaPrint* print, GError** e)
 {
@@ -2355,7 +2241,6 @@ ANJUTA_TYPE_ADD_INTERFACE(iindic, IANJUTA_TYPE_INDICABLE);
 ANJUTA_TYPE_ADD_INTERFACE(iselect, IANJUTA_TYPE_EDITOR_SELECTION);
 ANJUTA_TYPE_ADD_INTERFACE(iassist, IANJUTA_TYPE_EDITOR_ASSIST);
 ANJUTA_TYPE_ADD_INTERFACE(iconvert, IANJUTA_TYPE_EDITOR_CONVERT);
-ANJUTA_TYPE_ADD_INTERFACE(ibookmark, IANJUTA_TYPE_BOOKMARK);
 ANJUTA_TYPE_ADD_INTERFACE(iprint, IANJUTA_TYPE_PRINT);
 ANJUTA_TYPE_ADD_INTERFACE(ilanguage, IANJUTA_TYPE_EDITOR_LANGUAGE);
 ANJUTA_TYPE_ADD_INTERFACE(isearch, IANJUTA_TYPE_EDITOR_SEARCH);
