@@ -28,7 +28,6 @@
 #include <libanjuta/anjuta-plugin.h>
 #include <libanjuta/anjuta-launcher.h>
 #include "symbol-db-engine.h"
-#include "symbol-db-engine-iterator.h"
 
 
 G_BEGIN_DECLS
@@ -52,18 +51,24 @@ typedef struct _SymbolDBPluginClass SymbolDBPluginClass;
 /* a sort of 'default' value for ctags executable. User must have it installed */
 #define CTAGS_PATH			"/usr/bin/ctags"
 
-struct _SymbolDBPlugin{
+struct _SymbolDBPlugin {
 	AnjutaPlugin parent;
 	AnjutaUI *ui;
 	AnjutaPreferences *prefs;
 	
 	/* project monitor */
 	guint root_watch_id;
-	 
+	
+	/* ui merge */
+	GtkActionGroup *popup_action_group;
+	gint merge_id;
+	
 	/* editor monitor */
 	guint buf_update_timeout_id;
 	gboolean need_symbols_update;
 	GTimer *update_timer;
+	GPtrArray *buffer_update_files;
+	GPtrArray *buffer_update_ids;
 	guint editor_watch_id;
 	gchar *project_root_uri;
 	gchar *project_root_dir;
@@ -115,9 +120,16 @@ struct _SymbolDBPlugin{
 	gint files_count_system_done;
 	gchar *current_scanned_package;
 	GList *session_packages;
+	
+	GTree *proc_id_tree;
+	
+	gboolean is_project_importing;		/* refreshes or resumes after abort */
+	gboolean is_project_updating;		/* makes up to date symbols of the project's files */
+	gboolean is_offline_scanning;		/* detects offline changes to makefile.am */
+	gboolean is_adding_element;			/* we're adding an element */
 };
 
-struct _SymbolDBPluginClass{
+struct _SymbolDBPluginClass {
 	AnjutaPluginClass parent_class;
 	
 	/* signals */
