@@ -84,7 +84,7 @@ on_pkg_config_exit (AnjutaLauncher * launcher, int child_pid,
 				   gpointer user_data);
 
 static void
-on_engine_package_scan_end (SymbolDBEngine *dbe, gpointer user_data);
+on_engine_package_scan_end (SymbolDBEngine *dbe, gint process_id, gpointer user_data);
 
 static void
 destroy_single_scan_data (SingleScanData *ss_data)
@@ -242,7 +242,7 @@ sdb_system_get_normalized_cflags (const gchar *chars)
 	while ((curr_flag = flags[i++]) != NULL)
 	{
 		/* '-I/usr/include/gimp-2.0' would be good, but '/usr/include/' wouldn't. */
-		if (g_regex_match_simple ("\\.*/usr/include/\\w+", curr_flag, 0, 0) == TRUE)
+		if (g_regex_match_simple ("\\.*/include/\\w+", curr_flag, 0, 0) == TRUE)
 		{
 			/* FIXME the +2. It's to skip the -I */
 			DEBUG_PRINT ("adding %s to good_flags", curr_flag +2);
@@ -348,8 +348,8 @@ sdb_system_files_visit_dir (GList **files_list, const gchar* uri)
 			
 			if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) 
 			{				
-				if (strcmp (info->name, ".") == 0 ||
-					strcmp (info->name, "..") == 0)
+				if (g_strcmp0 (info->name, ".") == 0 ||
+					g_strcmp0 (info->name, "..") == 0)
 					continue;
 
 				gchar *tmp = g_strdup_printf ("%s/%s", uri, info->name);
@@ -499,7 +499,7 @@ sdb_system_do_engine_scan (SymbolDBSystem *sdbs, EngineScanData *es_data)
 }
 
 static void
-on_engine_package_scan_end (SymbolDBEngine *dbe, gpointer user_data)
+on_engine_package_scan_end (SymbolDBEngine *dbe, gint process_id, gpointer user_data)
 {
 	SymbolDBSystem *sdbs;
 	SymbolDBSystemPriv *priv;
@@ -588,7 +588,7 @@ sdb_system_do_scan_new_package (SymbolDBSystem *sdbs,
 		/* there's something already working... being this function called in a 
 		 * single-threaded fashion we can put the the next parameter on the queue 
 		 */
-		DEBUG_PRINT ("pushed on queue for later scanning");
+		DEBUG_PRINT ("%s", "pushed on queue for later scanning");
 		g_queue_push_tail (priv->sscan_queue, ss_data);
 		return;
 	}
@@ -626,7 +626,7 @@ on_pkg_config_exit (AnjutaLauncher * launcher, int child_pid,
 	 */
 	if (ss_data->parseable_cb != NULL)
 	{
-		DEBUG_PRINT ("on_pkg_config_exit parseable activated");
+		DEBUG_PRINT ("%s", "on_pkg_config_exit parseable activated");
 		ss_data->parseable_cb (sdbs, cflags == NULL ? FALSE : TRUE, 
 							   ss_data->parseable_data);
 	}
