@@ -29,8 +29,6 @@
 
 #include "symbol-db-view-locals.h"
 #include "symbol-db-engine.h"
-#include "symbol-db-engine-iterator.h"
-#include "symbol-db-engine-iterator-node.h"
 #include "symbol-db-view.h"
 
 
@@ -88,12 +86,6 @@ static void
 trigger_on_symbol_inserted (SymbolDBViewLocals *dbvl, gint symbol_id);
 
 
-static gint
-gtree_compare_func (gconstpointer a, gconstpointer b, gpointer user_data)
-{
-	return (gint)a - (gint)b;
-}
-
 static void
 waiting_for_symbol_destroy (WaitingForSymbol *wfs)
 {
@@ -137,7 +129,7 @@ sdb_view_locals_create_new_store ()
 	GtkTreeStore *store;
 	store = gtk_tree_store_new (COLUMN_MAX, GDK_TYPE_PIXBUF,
 				    G_TYPE_STRING, G_TYPE_INT);	
-	// FIXME? 
+
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
 										  COLUMN_NAME,
 										  GTK_SORT_ASCENDING);
@@ -168,7 +160,7 @@ sdb_view_locals_get_iter_from_row_ref (SymbolDBViewLocals *dbvl,
 	path = gtk_tree_row_reference_get_path (row_ref);
 	if (path == NULL) 
 	{
-		/*DEBUG_PRINT ("sdb_view_locals_get_iter_from_row_ref (): path is null, something "
+		/*DEBUG_PRINT ("%s", "sdb_view_locals_get_iter_from_row_ref (): path is null, something "
 					 "went wrong ?!");*/
 		return FALSE;
 	}
@@ -195,7 +187,7 @@ symbol_db_view_locals_clear_cache (SymbolDBViewLocals *dbvl)
 	
 	priv = dbvl->priv;
 		
-	/*DEBUG_PRINT ("symbol_db_view_locals_clear_cache ()");*/
+	/*DEBUG_PRINT ("%s", "symbol_db_view_locals_clear_cache ()");*/
 	/* check whether we already have the view status saved on hash table or not.
 	 * If we saved that then don't remove it, or there may be a memory leak
 	 */	
@@ -257,7 +249,7 @@ sdb_view_locals_init (SymbolDBViewLocals *dbvl)
 	
 	g_return_if_fail (dbvl != NULL);
 
-	DEBUG_PRINT ("sdb_view_locals_init  ()");
+	DEBUG_PRINT ("%s", "sdb_view_locals_init  ()");
 	dbvl->priv = g_new0 (SymbolDBViewLocalsPriv, 1);		
 	priv = dbvl->priv;
 		
@@ -323,7 +315,7 @@ sdb_view_locals_finalize (GObject *object)
 	locals = SYMBOL_DB_VIEW_LOCALS (object);
 	priv = locals->priv;
 
-	/*DEBUG_PRINT ("finalizing symbol_db_view_locals ()");*/
+	/*DEBUG_PRINT ("%s", "finalizing symbol_db_view_locals ()");*/
 
 	symbol_db_view_locals_clear_cache (locals);
 	g_hash_table_destroy (priv->files_view_status);
@@ -431,7 +423,7 @@ do_add_child_symbol_to_view (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 		return NULL;
 	
 	/* look up the row ref in the hashtable, then get its associated gtktreeiter */
-	row_ref = g_tree_lookup (priv->nodes_displayed, (gpointer)parent_symbol_id);
+	row_ref = g_tree_lookup (priv->nodes_displayed, GINT_TO_POINTER (parent_symbol_id));
 	
 	if (sdb_view_locals_get_iter_from_row_ref (dbvl, row_ref, &iter) == FALSE)
 	{
@@ -479,7 +471,7 @@ traverse_on_scan_end (gpointer key, gpointer value, gpointer data)
 
 	priv = dbvl->priv;
 
-	parent_id = (gint) key;
+	parent_id = GPOINTER_TO_INT(key);
 	/*DEBUG_PRINT ("traverse_on_scan_end (): something has been left on "
 				"waiting_for_tree.. checking for %d", parent_id);*/
 
@@ -497,7 +489,7 @@ traverse_on_scan_end (gpointer key, gpointer value, gpointer data)
 		
 		iter_node = SYMBOL_DB_ENGINE_ITERATOR_NODE (iterator);
 		
-		pixbuf = symbol_db_view_get_pixbuf (
+		pixbuf = symbol_db_util_get_pixbuf (
 						symbol_db_engine_iterator_node_get_symbol_extra_string (
 							iter_node, SYMINFO_KIND),
 						symbol_db_engine_iterator_node_get_symbol_extra_string (
@@ -515,7 +507,7 @@ traverse_on_scan_end (gpointer key, gpointer value, gpointer data)
 		if (grandparent_id > 0) 
 		{
 			row_ref = 
-				 g_tree_lookup (priv->nodes_displayed, (gpointer)grandparent_id);
+				 g_tree_lookup (priv->nodes_displayed, GINT_TO_POINTER (grandparent_id));
 			if (row_ref != NULL)
 			{
 				/* hey we have a namespace or something which is the parent of
@@ -527,7 +519,7 @@ traverse_on_scan_end (gpointer key, gpointer value, gpointer data)
 				row_ref = do_add_child_symbol_to_view (dbvl, grandparent_id, pixbuf, 
 							symbol_name, parent_id);
 				
-				g_tree_insert (priv->nodes_displayed, (gpointer)parent_id, 
+				g_tree_insert (priv->nodes_displayed, GINT_TO_POINTER (parent_id), 
 				   row_ref);
 			}
 		/*	DEBUG_PRINT ("traverse_on_scan_end (): row_ref %d", row_ref);*/
@@ -539,7 +531,7 @@ traverse_on_scan_end (gpointer key, gpointer value, gpointer data)
 			
 			row_ref = do_add_root_symbol_to_view (dbvl, pixbuf, 
 							symbol_name, parent_id);
-			g_tree_insert (priv->nodes_displayed, (gpointer)parent_id, 
+			g_tree_insert (priv->nodes_displayed, GINT_TO_POINTER (parent_id), 
 				   row_ref);			
 		}
 
@@ -573,7 +565,7 @@ add_waiting_for_symbol_to_view (SymbolDBViewLocals *dbvl, WaitingForSymbol *wfs,
 	symbol_id_added = wfs->child_symbol_id;
 	
 	/* add a new entry on gtree 'nodes_displayed' */
-	g_tree_insert (priv->nodes_displayed, (gpointer)wfs->child_symbol_id, 
+	g_tree_insert (priv->nodes_displayed, GINT_TO_POINTER (wfs->child_symbol_id), 
 				   child_tree_row_ref);	
 	
 	/* and now trigger the inserted symbol... (recursive function). */
@@ -595,12 +587,12 @@ trigger_on_symbol_inserted (SymbolDBViewLocals *dbvl, gint symbol_id)
 	/*DEBUG_PRINT ("trigger_on_symbol_inserted (): triggering %d", symbol_id);*/
 	
 	/* try to find a waiting for symbol */
-	slist = g_tree_lookup (priv->waiting_for, (gpointer)symbol_id);
+	slist = g_tree_lookup (priv->waiting_for, GINT_TO_POINTER (symbol_id));
 	
 	if (slist == NULL) 
 	{
 		/* nothing waiting for us */
-		/*DEBUG_PRINT ("trigger_on_symbol_inserted (): no children waiting for us...");*/
+		/*DEBUG_PRINT ("%s", "trigger_on_symbol_inserted (): no children waiting for us...");*/
 		return;
 	}
 	else {
@@ -623,7 +615,7 @@ trigger_on_symbol_inserted (SymbolDBViewLocals *dbvl, gint symbol_id)
 		}
 		
 		/* remove the waiting for key/value */
-		g_tree_remove (priv->waiting_for, (gpointer)symbol_id);		
+		g_tree_remove (priv->waiting_for, GINT_TO_POINTER (symbol_id));
 		g_slist_free (slist);
 	}
 }
@@ -639,6 +631,10 @@ add_new_waiting_for (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 	g_return_if_fail (dbvl != NULL);	
 	priv = dbvl->priv;
 
+	/* we don't want a negative parent_symbol_id */
+	if (parent_symbol_id < 0)
+		return;	
+	
 	/* check if we already have some children waiting for a 
 	 * specific father to be inserted, then add this symbol_id to the list 
 	 * (or create a new one)
@@ -652,7 +648,7 @@ add_new_waiting_for (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 				
 /*	DEBUG_PRINT ("add_new_waiting_for (): looking up waiting_for %d", 
 				 parent_symbol_id);*/
-	node = g_tree_lookup (priv->waiting_for, (gpointer)parent_symbol_id);
+	node = g_tree_lookup (priv->waiting_for, GINT_TO_POINTER (parent_symbol_id));
 	if (node == NULL) 
 	{
 		/* no lists already set. Create one. */
@@ -665,7 +661,7 @@ add_new_waiting_for (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 					 "waiting_for [%d]", parent_symbol_id);*/
 				
 		/* add it to the binary tree. */
-		g_tree_insert (priv->waiting_for, (gpointer)parent_symbol_id, 
+		g_tree_insert (priv->waiting_for, GINT_TO_POINTER (parent_symbol_id), 
 							   slist);
 	}
 	else 
@@ -679,7 +675,7 @@ add_new_waiting_for (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 				 	parent_symbol_id, symbol_name);*/
 		slist = g_slist_prepend (slist, wfs);
 				
-		g_tree_replace (priv->waiting_for, (gpointer)parent_symbol_id, 
+		g_tree_replace (priv->waiting_for, GINT_TO_POINTER (parent_symbol_id), 
 						slist);
 	}	
 }
@@ -730,7 +726,7 @@ do_recurse_subtree_and_invalidate (SymbolDBViewLocals *dbvl,
 						 curr_symbol_id, curr_pixbuf);
 	
 	gtk_tree_store_remove (store, parent_subtree_iter);
-	g_tree_remove (priv->nodes_displayed, (gpointer) curr_symbol_id);
+	g_tree_remove (priv->nodes_displayed, GINT_TO_POINTER (curr_symbol_id));
 
 	/* don't forget to free this gchar */				   
 	g_free (curr_symbol_name);
@@ -764,7 +760,7 @@ prepare_for_adding (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 													 symbol_id);
 		
 		/* we'll fake the gpointer to store an int */
-		g_tree_insert (priv->nodes_displayed, (gpointer)symbol_id, 
+		g_tree_insert (priv->nodes_displayed, GINT_TO_POINTER (symbol_id), 
 					   curr_tree_row_ref);
 		
 		/* let's trigger the insertion of the symbol_id, there may be some children
@@ -778,7 +774,7 @@ prepare_for_adding (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 		/* do we already have that parent_symbol displayed in gtktreeview? 
 		 * If that's the case add it as children.
 		 */
-		node = g_tree_lookup (priv->nodes_displayed, (gpointer)parent_symbol_id);
+		node = g_tree_lookup (priv->nodes_displayed, GINT_TO_POINTER (parent_symbol_id));
 		
 		if (node != NULL) 
 		{
@@ -791,7 +787,7 @@ prepare_for_adding (SymbolDBViewLocals *dbvl, gint parent_symbol_id,
 				   pixbuf, symbol_name, symbol_id);
 			
 			/* add the children_path to the GTree. */
-			g_tree_insert (priv->nodes_displayed, (gpointer)symbol_id, 
+			g_tree_insert (priv->nodes_displayed, GINT_TO_POINTER (symbol_id), 
 						   child_row_ref);
 			trigger_on_symbol_inserted (dbvl, symbol_id);
 		}
@@ -878,7 +874,7 @@ consume_symbols_inserted_queue_idle (gpointer data)
 	/* consume a symbol */
 	if (queue_length > 0)
 	{
-		consumed_symbol_id = (gint) g_queue_pop_head (priv->symbols_inserted_ids);
+		consumed_symbol_id = GPOINTER_TO_INT(g_queue_pop_head (priv->symbols_inserted_ids));
 	}
 	else {
 		return FALSE;
@@ -926,7 +922,7 @@ consume_symbols_inserted_queue_idle (gpointer data)
 		
 		iter_node = SYMBOL_DB_ENGINE_ITERATOR_NODE (iterator);
 		
-		pixbuf = symbol_db_view_get_pixbuf (
+		pixbuf = symbol_db_util_get_pixbuf (
 						symbol_db_engine_iterator_node_get_symbol_extra_string (
 							iter_node, SYMINFO_KIND),
 						symbol_db_engine_iterator_node_get_symbol_extra_string (
@@ -967,7 +963,7 @@ consume_symbols_inserted_queue_idle (gpointer data)
 					symbol_db_engine_iterator_node_get_symbol_id (iter_node);
 
 				row_ref = g_tree_lookup (priv->nodes_displayed,
-										 (gpointer)curr_child_id);
+										 GINT_TO_POINTER (curr_child_id));
 				
 				if (row_ref == NULL)
 					continue;
@@ -975,7 +971,7 @@ consume_symbols_inserted_queue_idle (gpointer data)
 				if (sdb_view_locals_get_iter_from_row_ref (dbvl, 
 								row_ref, &child_iter) == FALSE)
 				{
-					g_warning ("on_symbol_inserted (): something went wrong");
+					/*g_warning ("on_symbol_inserted (): something went wrong");*/
 					continue;
 				}		
 				
@@ -999,7 +995,7 @@ consume_symbols_inserted_queue_idle (gpointer data)
 }
 
 static void
-on_scan_end (SymbolDBEngine *dbe, gpointer data)
+on_scan_end (SymbolDBEngine *dbe, gint process_id, gpointer data)
 {
 	SymbolDBViewLocals *dbvl;
 	SymbolDBViewLocalsPriv *priv;
@@ -1018,7 +1014,7 @@ on_scan_end (SymbolDBEngine *dbe, gpointer data)
 	tdata->dbvl = dbvl;
 	tdata->dbe = dbe;
 	
-	/*DEBUG_PRINT ("locals: on_scan_end");*/
+	/*DEBUG_PRINT ("%s", "locals: on_scan_end");*/
 	if (priv->symbols_inserted_ids != NULL)
 	{
 		if (g_queue_get_length (priv->symbols_inserted_ids) > 0) 
@@ -1069,7 +1065,7 @@ do_recurse_subtree_and_remove (SymbolDBViewLocals *dbvl,
 	}
 
 	gtk_tree_store_remove (store, parent_subtree_iter);
-	g_tree_remove (priv->nodes_displayed, (gpointer) curr_symbol_id);
+	g_tree_remove (priv->nodes_displayed, GINT_TO_POINTER (curr_symbol_id));
 
 	/* don't forget to free this gchar */				   
 	g_free (curr_symbol_name);
@@ -1094,9 +1090,9 @@ on_symbol_removed (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
 		return;
 	}
 	
-	DEBUG_PRINT ("on_symbol_removed (): -local- %d", symbol_id);
+	/*DEBUG_PRINT ("on_symbol_removed (): -local- %d", symbol_id);*/
 
-	row_ref = g_tree_lookup (priv->nodes_displayed, (gpointer)symbol_id);
+	row_ref = g_tree_lookup (priv->nodes_displayed, GINT_TO_POINTER (symbol_id));
 	if (sdb_view_locals_get_iter_from_row_ref (dbvl, row_ref, &iter) == FALSE)
 	{
 		return;
@@ -1123,8 +1119,8 @@ on_symbol_scope_updated (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
 		return;
 	}
 	
-	DEBUG_PRINT ("on_symbol_scope_updated () -local- %d", symbol_id);
-	row_ref = g_tree_lookup (priv->nodes_displayed, (gpointer)symbol_id);
+	/*DEBUG_PRINT ("on_symbol_scope_updated () -local- %d", symbol_id);*/
+	row_ref = g_tree_lookup (priv->nodes_displayed, GINT_TO_POINTER (symbol_id));
 	if (sdb_view_locals_get_iter_from_row_ref (dbvl, row_ref, &iter) == FALSE)
 	{
 		return;
@@ -1133,7 +1129,7 @@ on_symbol_scope_updated (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
 	do_recurse_subtree_and_remove (dbvl, &iter);	
 	
 	/* save the symbol_id to be added in the queue and just return */
-	g_queue_push_head (priv->symbols_inserted_ids, (gpointer)symbol_id);	
+	g_queue_push_head (priv->symbols_inserted_ids, GINT_TO_POINTER (symbol_id));	
 }
 
 static void 
@@ -1153,7 +1149,7 @@ on_symbol_inserted (SymbolDBEngine *dbe, gint symbol_id, gpointer data)
 	}
 	
 	/* save the symbol_id to be added in the queue and just return */
-	g_queue_push_head (priv->symbols_inserted_ids, (gpointer)symbol_id);
+	g_queue_push_head (priv->symbols_inserted_ids, GINT_TO_POINTER (symbol_id));
 }
 
 gint
@@ -1201,7 +1197,7 @@ symbol_db_view_locals_recv_signals_from_engine (SymbolDBViewLocals *dbvl,
 	SymbolDBViewLocalsPriv *priv;
 
 	g_return_if_fail (dbvl != NULL);
-	priv = dbvl->priv;		
+	priv = dbvl->priv;
 	
 	if (enable_status == TRUE) 
 	{
@@ -1238,25 +1234,25 @@ symbol_db_view_locals_recv_signals_from_engine (SymbolDBViewLocals *dbvl,
 		gtk_widget_set_sensitive (GTK_WIDGET (dbvl), FALSE);
 		
 		priv->recv_signals = FALSE;
-		if (priv->insert_handler >= 0) 
+		if (priv->insert_handler > 0) 
 		{
 			g_signal_handler_disconnect (G_OBJECT (dbe), priv->insert_handler);
 			priv->insert_handler = 0;
 		}
 
-		if (priv->remove_handler >= 0)
+		if (priv->remove_handler > 0)
 		{
 			g_signal_handler_disconnect (G_OBJECT (dbe), priv->remove_handler);
 			priv->remove_handler = 0;
 		}	
 
-		if (priv->scan_end_handler >= 0)
+		if (priv->scan_end_handler > 0)
 		{
 			g_signal_handler_disconnect (G_OBJECT (dbe), priv->scan_end_handler);
 			priv->scan_end_handler = 0;
 		}
 		
-		if (priv->scope_update_handler >= 0) 
+		if (priv->scope_update_handler > 0) 
 		{
 			g_signal_handler_disconnect (G_OBJECT (dbe), priv->scope_update_handler);
 			priv->scope_update_handler = 0;
@@ -1288,10 +1284,9 @@ symbol_db_view_locals_display_nothing (SymbolDBViewLocals *dbvl,
 
 void
 symbol_db_view_locals_update_list (SymbolDBViewLocals *dbvl, SymbolDBEngine *dbe,
-							  const gchar* filepath)
+							  const gchar* filepath, gboolean force_update)
 {
 	SymbolDBViewLocalsPriv *priv;
-
 	SymbolDBEngineIterator *iterator;
 	GtkTreeStore *store;	
 	FileSymbolsStatus *fsstatus;
@@ -1302,17 +1297,22 @@ symbol_db_view_locals_update_list (SymbolDBViewLocals *dbvl, SymbolDBEngine *dbe
 
 	priv = dbvl->priv;
 
-	DEBUG_PRINT ("symbol_db_view_locals_update_list () %s", filepath);
+	DEBUG_PRINT ("filepath %s", filepath);
 	
 	/* we're not interested in giving user an updated gtktreestore if recv signals
 	 * is false. In that case we can have a project importing...
+	 * We should pass on this test if the force option is activated (i.e. is TRUE)
 	 */
-	if (priv->recv_signals == FALSE)
+	if (priv->recv_signals == FALSE && force_update == FALSE)
 	{		
 		gtk_tree_view_set_model (GTK_TREE_VIEW (dbvl), NULL);
+		DEBUG_PRINT ("recv signals is false");
 		return;
 	}
 	
+	/* it's a good thing to enable the gtktreewidget */
+	gtk_widget_set_sensitive (GTK_WIDGET (dbvl), TRUE);
+	
 	/* ok, we can have a case where we're revisiting an old file with an already 
 	 * populated GtkTreeStore. We're gonna set that gtktreestore along with the
 	 * GTree(s) for nodes_displayed and waiting_for.
@@ -1353,7 +1353,7 @@ symbol_db_view_locals_update_list (SymbolDBViewLocals *dbvl, SymbolDBEngine *dbe
 				fss->waiting_for = priv->waiting_for;
 				fss->symbols_inserted_ids = priv->symbols_inserted_ids;
 				
-				/*DEBUG_PRINT ("symbol_db_view_locals_update_list (): "
+				/*DEBUG_PRINT ("%s", "symbol_db_view_locals_update_list (): "
 					"g_hash_table_insert ");*/
 				/* insert it */
 				g_hash_table_insert (priv->files_view_status, 
@@ -1374,8 +1374,7 @@ symbol_db_view_locals_update_list (SymbolDBViewLocals *dbvl, SymbolDBEngine *dbe
 		symbol_db_engine_get_file_db_path (dbe, filepath);
 	if (priv->current_db_file == NULL) 
 	{
-		/*DEBUG_PRINT ("symbol_db_view_locals_update_list (): "
-					 "Warning: priv->current_db_file is NULL");*/
+		DEBUG_PRINT ("Warning: priv->current_db_file is NULL");
 		return;
 	}
 	priv->current_local_file_path = g_strdup (filepath);
@@ -1403,7 +1402,7 @@ symbol_db_view_locals_update_list (SymbolDBViewLocals *dbvl, SymbolDBEngine *dbe
 		/* with this set there's no need to re-retrieve the symbols from db,
 		 * speeding up the things.
 		 */
-		/*DEBUG_PRINT ("symbol_db_view_locals_update_list (): "
+		/*DEBUG_PRINT ("%s", "symbol_db_view_locals_update_list (): "
 					 "setting gtk_tree_view_set_model to the saved one");*/
 		gtk_tree_view_set_model (GTK_TREE_VIEW (dbvl), GTK_TREE_MODEL (store));
 		
@@ -1425,19 +1424,19 @@ symbol_db_view_locals_update_list (SymbolDBViewLocals *dbvl, SymbolDBEngine *dbe
 	}
 	else 
 	{	
-		priv->nodes_displayed = g_tree_new_full ((GCompareDataFunc)&gtree_compare_func, 
+		priv->nodes_displayed = g_tree_new_full ((GCompareDataFunc)&symbol_db_gtree_compare_func, 
 										 NULL,
 										 NULL,
 										 (GDestroyNotify)&gtk_tree_row_reference_free);		
 
-		priv->waiting_for = g_tree_new_full ((GCompareDataFunc)&gtree_compare_func, 
+		priv->waiting_for = g_tree_new_full ((GCompareDataFunc)&symbol_db_gtree_compare_func, 
 										 NULL,
 										 NULL,
 										 NULL);
 		
 		priv->symbols_inserted_ids = g_queue_new ();
 
-		/*DEBUG_PRINT ("symbol_db_view_locals_update_list (): creating new store"); */
+		/*DEBUG_PRINT ("%s", "symbol_db_view_locals_update_list (): creating new store"); */
 		store = sdb_view_locals_create_new_store ();
 		gtk_tree_view_set_model (GTK_TREE_VIEW (dbvl), GTK_TREE_MODEL (store));
 		
@@ -1469,9 +1468,10 @@ symbol_db_view_locals_update_list (SymbolDBViewLocals *dbvl, SymbolDBEngine *dbe
 		}
 
 		/* ok, there may be some symbols left on the waiting_for_list...
- 	 	* launch the callback function by hand, flushing the list it in case 
-	 	*/
-		on_scan_end (dbe, dbvl);		
+ 	 	 * launch the callback function by hand, flushing the list it in case 
+		 * The 0 stays for an unused process_id
+		 */		
+		on_scan_end (dbe, 0, dbvl);		
 	}
 	
 	/* only gtk 2.12 ...
