@@ -35,9 +35,10 @@
 #include <string.h>
 
 #include <gconf/gconf-client.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libanjuta/anjuta-debug.h>
+#include <libanjuta/anjuta-utils.h>
 
 #include "vggeneralprefs.h"
 
@@ -52,7 +53,7 @@
 #define RUN_LIBC_FREERES_KEY "/apps/anjuta/valgrind/general/run-libc-freeres"
 #define SUPPRESSIONS_KEY     "/apps/anjuta/valgrind/general/suppressions"
 
-#define SUPPRESSIONS_DEFAULT_FILE	".anjuta/valgrind.supp"
+#define SUPPRESSIONS_DEFAULT_FILE	"valgrind.supp"
 
 static void vg_general_prefs_class_init (VgGeneralPrefsClass *klass);
 static void vg_general_prefs_init (VgGeneralPrefs *prefs);
@@ -175,7 +176,8 @@ vg_general_prefs_init (VgGeneralPrefs *prefs)
 	gtk_box_set_spacing (GTK_BOX (vbox), 6);
 	
 	bool = gconf_client_get_bool (gconf, DEMANGLE_KEY, NULL);
-	widget = gtk_check_button_new_with_label (_("Demangle c++ symbol names"));
+	/* demangle = decode C++ name encoding */
+	widget = gtk_check_button_new_with_label (_("Demangle C++ symbol names"));
 	g_signal_connect (widget, "toggled", G_CALLBACK (toggle_button_toggled), DEMANGLE_KEY);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), bool);
 	prefs->demangle = GTK_TOGGLE_BUTTON (widget);
@@ -257,7 +259,7 @@ vg_general_prefs_init (VgGeneralPrefs *prefs)
 	if (!(str_file = gconf_client_get_string (gconf, SUPPRESSIONS_KEY, &err)) || err != NULL) {
 		int fd;
 		
-		str_file = g_build_filename (g_get_home_dir (), SUPPRESSIONS_DEFAULT_FILE, NULL);
+		str_file = anjuta_util_get_user_config_file_path(SUPPRESSIONS_DEFAULT_FILE, NULL);
 		if ((fd = open (str_file, O_WRONLY | O_CREAT, 0666)) == -1) {
 			g_free (str_file);
 			str_file = NULL;
@@ -274,7 +276,7 @@ vg_general_prefs_init (VgGeneralPrefs *prefs)
 								GTK_FILE_CHOOSER_ACTION_OPEN);
 
 	if ( gtk_file_chooser_select_filename (GTK_FILE_CHOOSER (widget), str_file) == FALSE )
-		DEBUG_PRINT ("error: could not select file uri with gtk_file_chooser_select_filename ()");
+		DEBUG_PRINT ("%s", "error: could not select file uri with gtk_file_chooser_select_filename ()");
 	
 	/* grab every change in file selection */
 	g_signal_connect (widget, "selection-changed", G_CALLBACK (file_entry_changed), SUPPRESSIONS_KEY);
