@@ -33,8 +33,9 @@
 #include <fcntl.h>
 #include <errno.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
+#include <libanjuta/anjuta-utils.h>
 
 #include "vgrule-list.h"
 #include "vgmarshal.h"
@@ -457,7 +458,7 @@ vg_rule_list_destroy (GtkObject *obj)
 	VgRuleList *list = (VgRuleList *) obj;
 	
 	if (list->gio != NULL) {
-		g_io_channel_close (list->gio);
+		g_io_channel_shutdown (list->gio, TRUE, NULL);
 		g_io_channel_unref (list->gio);
 		list->load_id = 0;
 		list->gio = NULL;
@@ -504,7 +505,7 @@ load_rules_step_cb (GIOChannel *gio, GIOCondition condition, gpointer user_data)
 	vg_rule_parser_free (list->parser);
 	list->parser = NULL;
 	
-	g_io_channel_close (list->gio);
+	g_io_channel_shutdown (list->gio, TRUE, NULL);
 	g_io_channel_unref (list->gio);
 	list->load_id = 0;
 	list->gio = NULL;
@@ -569,7 +570,7 @@ vg_rule_list_set_filename (VgRuleList *list, const char *filename)
 		if (list->load_id != 0) {
 			/* this means the file is currently being loaded. ugh */
 			vg_rule_parser_free (list->parser);
-			g_io_channel_close (list->gio);
+			g_io_channel_shutdown (list->gio, TRUE, NULL);
 			g_io_channel_unref (list->gio);
 			list->load_id = 0;
 			list->gio = NULL;
@@ -692,7 +693,7 @@ vg_rule_list_add_rule (VgRuleList *list, const char *title, GtkWindow *parent, V
 		gconf = gconf_client_get_default ();
 		
 		// FIXME: hardcoded path
-		list->filename = g_strdup_printf ("%s/.anjuta/valgrind.supp", getenv ("HOME"));
+		list->filename = anjuta_util_get_user_config_file_path ("valgrind.supp", NULL);
 		gconf_client_set_string (gconf, SUPPRESSIONS_KEY, list->filename, NULL);
 		g_object_unref (gconf);
 	}
