@@ -29,7 +29,7 @@
 #include <sys/stat.h>
 
 #include <libgnome/gnome-program.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libanjuta/resources.h>
 #include <libanjuta/anjuta-debug.h>
@@ -177,6 +177,7 @@ on_message_received (const char *message, gpointer    data)
 	gint i;
 	GdkDisplay *display;
 	GdkScreen *screen;
+	GList* file;
 
 	g_return_if_fail (message != NULL);
 
@@ -232,15 +233,16 @@ on_message_received (const char *message, gpointer    data)
 
 	/* execute the commands */
 
-	while (file_list != NULL)
+	for (file = file_list; file != NULL; file = g_list_next (file))
 	{
-		IAnjutaFileLoader* file =
+		IAnjutaFileLoader* loader =
 			anjuta_shell_get_interface(ANJUTA_SHELL(app), IAnjutaFileLoader, NULL);
-		gchar* uri = g_strdup_printf("%s:%d", (gchar*) file_list->data, line_position);
-		
-		ianjuta_file_loader_load(file, file_list->data, FALSE, NULL);
-		g_free(uri);
-		file_list = g_list_next(file_list);
+		GFile* gfile = g_file_new_for_commandline_arg(file->data);
+		if (gfile)
+		{
+			ianjuta_file_loader_load(loader, gfile, FALSE, NULL);
+			g_object_unref (gfile);
+		}
 	}
 
 	/* free the file list and reset to default */
@@ -342,7 +344,10 @@ main (int argc, char *argv[])
 								  GNOME_PARAM_APP_DATADIR, data_dir,
 								  GNOME_PARAM_NONE);
 	g_free (data_dir);
-	
+
+	/* Init debug helpers */
+	anjuta_debug_init ();
+
 	/* Get the command line files */
 	get_command_line_args ();
 	
@@ -351,9 +356,9 @@ main (int argc, char *argv[])
 	if (connection != NULL)
 	{
 		if (!bacon_message_connection_get_is_server (connection) &&
-			 no_client ==FALSE) 
+			 no_client == FALSE) 
 		{
-			DEBUG_PRINT("Client");
+			DEBUG_PRINT("%s", "Client");
 			send_bacon_message ();
 
 			/* we never popup a window... tell startup-notification
@@ -367,7 +372,7 @@ main (int argc, char *argv[])
 		}
 		else 
 		{
-			DEBUG_PRINT("Server");
+			DEBUG_PRINT("%s", "Server");
 			bacon_message_connection_set_callback (connection,
 							       on_message_received,
 							       NULL);
@@ -382,7 +387,7 @@ main (int argc, char *argv[])
 
 	im_file = anjuta_res_get_pixmap_file (ANJUTA_PIXMAP_SPLASH_SCREEN);
 	
-	/* Initialize applicatoin */
+	/* Initialize application */
 	app = anjuta_new (argv[0], file_list, no_splash, no_session, no_files,
 					  im_file, proper_shutdown, anjuta_geometry);
 	
